@@ -146,6 +146,7 @@
 #include "gui_object.h"
 #include "gui_thread.h"
 #include "idleometer.h"
+#include "instrument_selector.h"
 #include "io_plugin_window.h"
 #include "keyboard.h"
 #include "keyeditor.h"
@@ -215,6 +216,12 @@ static const gchar *_record_mode_strings[] = {
 static bool
 ask_about_configuration_copy (string const & old_dir, string const & new_dir, int version)
 {
+#ifndef __APPLE__
+	/* guess screen scaling */
+	UIConfiguration::instance ().set_font_scale (1024 * guess_default_ui_scale ());
+	UIConfiguration::instance ().reset_dpi ();
+#endif
+
 	ArdourMessageDialog msg (string_compose (
 	                          _("%1 %2.x has discovered configuration files from %1 %3.x.\n\n"
 	                            "Would you like these files to be copied and used for %1 %2.x?\n\n"
@@ -248,7 +255,7 @@ libxml_generic_error_func (void* /* parsing_context*/,
 
 static void
 libxml_structured_error_func (void* /* parsing_context*/,
-                              xmlErrorPtr err)
+                              const xmlError *err)
 {
 	string msg;
 
@@ -367,6 +374,7 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], const char* localedir)
 	record_mode_strings = I18N (_record_mode_strings);
 
 	if (ARDOUR::handle_old_configuration_files (boost::bind (ask_about_configuration_copy, _1, _2, _3))) {
+
 		{
 			/* "touch" the been-here-before path now that config has been migrated */
 			PBD::ScopedFileDescriptor fout (g_open (been_here_before_path ().c_str(), O_CREAT|O_TRUNC|O_RDWR, 0666));
@@ -395,7 +403,11 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], const char* localedir)
 	/* stop libxml from spewing to stdout/stderr */
 
 	xmlSetGenericErrorFunc (this, libxml_generic_error_func);
-	xmlSetStructuredErrorFunc (this, libxml_structured_error_func);
+
+	/* Cast to xmlStructuredErrorFunc to cope with different constness in different
+	 * versions of libxml2. */
+
+	xmlSetStructuredErrorFunc (this, (xmlStructuredErrorFunc)libxml_structured_error_func);
 
 	/* Set this up early */
 
@@ -863,6 +875,8 @@ ARDOUR_UI::~ARDOUR_UI ()
 		recorder->cleanup ();
 	}
 
+	InstrumentSelector::DropPluginInfoPtr ();
+
 	if (getenv ("ARDOUR_RUNNING_UNDER_VALGRIND")) {
 		// don't bother at 'real' exit. the OS cleans up for us.
 		delete big_clock; big_clock = 0;
@@ -1193,12 +1207,14 @@ ARDOUR_UI::every_point_zero_something_seconds ()
 			_editor_meter_peaked = false;
 		}
 
-		const float mpeak = editor_meter->update_meters();
-		const bool peaking = mpeak > UIConfiguration::instance().get_meter_peak();
+		if (!UIConfiguration::instance().get_no_strobe()) {
+			const float mpeak = editor_meter->update_meters();
+			const bool peaking = mpeak > UIConfiguration::instance().get_meter_peak();
 
-		if (!_editor_meter_peaked && peaking) {
-			editor_meter_peak_display.set_active_state ( Gtkmm2ext::ExplicitActive );
-			_editor_meter_peaked = true;
+			if (!_editor_meter_peaked && peaking) {
+				editor_meter_peak_display.set_active_state (Gtkmm2ext::ExplicitActive);
+				_editor_meter_peaked = true;
+			}
 		}
 	}
 }
@@ -1416,6 +1432,10 @@ ARDOUR_UI::format_disk_space_label (float remain_sec)
 
 	std::string label = string_compose (X_("<span weight=\"ultralight\">%1</span>: "), _("Rec"));
 
+	if (_session && FLAC == _session->config.get_native_file_header_format () && remain_sec <= 86400) {
+		label += u8"\u2265"; // Greater-Than or Equal To
+	}
+
 	if (remain_sec > 86400) {
 		disk_space_label.set_markup (label + _(">24h"));
 	} else if (remain_sec > 32400 /* 9 hours */) {
@@ -2172,7 +2192,7 @@ ARDOUR_UI::blink_handler (bool blink_on)
 {
 	sync_blink (blink_on);
 
-	if (!UIConfiguration::instance().get_blink_alert_indicators()) {
+	if (UIConfiguration::instance().get_no_strobe() || !UIConfiguration::instance().get_blink_alert_indicators()) {
 		blink_on = true;
 	}
 	error_blink (blink_on);
@@ -2194,6 +2214,15 @@ ARDOUR_UI::update_clocks ()
 void
 ARDOUR_UI::start_clocking ()
 {
+	if (UIConfiguration::instance().get_no_strobe()) {
+		if (!_session) {
+			return;
+		}
+		_session->TransportStateChange.connect (clock_state_connection, MISSING_INVALIDATOR, sigc::mem_fun (*this, &ARDOUR_UI::update_clocks), gui_context());
+		_session->Located.connect (clock_state_connection, MISSING_INVALIDATOR, sigc::mem_fun (*this, &ARDOUR_UI::update_clocks), gui_context());
+		return;
+	}
+
 	if (UIConfiguration::instance().get_super_rapid_clock_update()) {
 		clock_signal_connection = Timers::fps_connect (sigc::mem_fun(*this, &ARDOUR_UI::update_clocks));
 	} else {
@@ -2204,6 +2233,7 @@ ARDOUR_UI::start_clocking ()
 void
 ARDOUR_UI::stop_clocking ()
 {
+	clock_state_connection.drop_connections ();
 	clock_signal_connection.disconnect ();
 }
 
@@ -2345,12 +2375,11 @@ ARDOUR_UI::route_setup_info (const std::string& script_path)
 		return rv;
 	}
 
-	LuaState lua;
+	LuaState lua (true, true);
 	lua.Print.connect (&_lua_print);
-	lua.sandbox (true);
 
 	lua_State* L = lua.getState();
-	LuaInstance::register_classes (L);
+	LuaInstance::register_classes (L, true);
 	LuaBindings::set_session (L, _session);
 	luabridge::push <PublicEditor *> (L, &PublicEditor::instance());
 	lua_setglobal (L, "Editor");
@@ -2398,12 +2427,11 @@ ARDOUR_UI::meta_route_setup (const std::string& script_path)
 		return;
 	}
 
-	LuaState lua;
+	LuaState lua (true, true);
 	lua.Print.connect (&_lua_print);
-	lua.sandbox (true);
 
 	lua_State* L = lua.getState();
-	LuaInstance::register_classes (L);
+	LuaInstance::register_classes (L, true);
 	LuaBindings::set_session (L, _session);
 	luabridge::push <PublicEditor *> (L, &PublicEditor::instance());
 	lua_setglobal (L, "Editor");
@@ -2444,12 +2472,13 @@ ARDOUR_UI::meta_session_setup (const std::string& script_path)
 		return;
 	}
 
-	LuaState lua;
+	bool sandbox = UIConfiguration::instance().get_sandbox_all_lua_scripts ();
+
+	LuaState lua (true, sandbox);
 	lua.Print.connect (&_lua_print);
-	lua.sandbox (false);
 
 	lua_State* L = lua.getState();
-	LuaInstance::register_classes (L);
+	LuaInstance::register_classes (L, sandbox);
 	LuaBindings::set_session (L, _session);
 	luabridge::push <PublicEditor *> (L, &PublicEditor::instance());
 	lua_setglobal (L, "Editor");
