@@ -48,8 +48,6 @@
 #include <list>
 #include <cmath>
 
-#include <boost/utility.hpp>
-
 #include "pbd/xml++.h"
 #include <gtkmm/box.h>
 #include <gtkmm/frame.h>
@@ -228,6 +226,8 @@ public:
 	int load_session (const std::string& path, const std::string& snapshot, std::string mix_template = std::string());
 	bool session_load_in_progress;
 	int build_session (std::string const& path, std::string const& snapshot, std::string const& session_template, ARDOUR::BusProfile const&, bool from_startup_fsm, bool unnamed, Temporal::TimeDomain domain);
+	int build_session_stage_two (std::string const& path, std::string const& snapshot, std::string const& session_template, ARDOUR::BusProfile const&, bool unnamed, Temporal::TimeDomain domain, ARDOUR::samplecnt_t samplerate = 0);
+
 	bool session_is_new() const { return _session_is_new; }
 
 	ARDOUR::Session* the_session() { return _session; }
@@ -250,6 +250,8 @@ public:
 	int  save_state_canfail (std::string state_name = "", bool switch_to_it = false);
 	void save_state (const std::string & state_name = "", bool switch_to_it = false);
 
+	int new_session_from_aaf (std::string const&, std::string const&, std::string&, std::string&);
+
 	static ARDOUR_UI *instance () { return theArdourUI; }
 
 	/* signal emitted when escape key is pressed. All UI components that
@@ -271,8 +273,6 @@ public:
 
 	void reset_focus (Gtk::Widget*);
 
-	static PublicEditor* _instance;
-
 	/** Emitted frequently with the audible sample, false, and the edit point as
 	 *  parameters respectively.
 	 *
@@ -474,7 +474,6 @@ private:
 	void audio_midi_setup_reconfigure_done (int response, std::string path, std::string snapshot, std::string mix_template);
 	int  load_session_stage_two (const std::string& path, const std::string& snapshot, std::string mix_template = std::string());
 	void audio_midi_setup_for_new_session_done (int response, std::string path, std::string snapshot, std::string session_template, ARDOUR::BusProfile const&, bool unnamed, Temporal::TimeDomain domain);
-	int  build_session_stage_two (std::string const& path, std::string const& snapshot, std::string const& session_template, ARDOUR::BusProfile const&, bool unnamed, Temporal::TimeDomain domain);
 	sigc::connection _engine_dialog_connection;
 
 	void save_session_at_its_request (std::string);
