@@ -246,7 +246,13 @@ ARDOUR_UI::toggle_punch_out ()
 void
 ARDOUR_UI::show_loop_punch_ruler_and_disallow_hide ()
 {
-	Glib::RefPtr<ToggleAction> tact = ActionManager::get_toggle_action (X_("Rulers"), "toggle-loop-punch-ruler");
+	return;
+
+	/* This is what this method used to do  but after ruler consolidation
+	 * it is not clear what the correct semantics are/should be.
+	 */
+
+	Glib::RefPtr<ToggleAction> tact; //  = ActionManager::get_toggle_action (X_("Rulers"), "toggle-loop-punch-ruler");
 
 	tact->set_sensitive (false);
 
@@ -259,9 +265,15 @@ ARDOUR_UI::show_loop_punch_ruler_and_disallow_hide ()
 void
 ARDOUR_UI::reenable_hide_loop_punch_ruler_if_appropriate ()
 {
+	return;
+
+	/* This is what this method used to do  but after ruler consolidation
+	 * it is not clear what the correct semantics are/should be.
+	 */
+
 	if (!_session->config.get_punch_in() && !_session->config.get_punch_out()) {
 		/* if punch in/out are now both off, reallow hiding of the loop/punch ruler */
-		Glib::RefPtr<Action> act = ActionManager::get_action (X_("Rulers"), "toggle-loop-punch-ruler");
+		Glib::RefPtr<Action> act; // = ActionManager::get_action (X_("Rulers"), "toggle-loop-punch-ruler");
 		if (act) {
 			act->set_sensitive (true);
 		}
