@@ -35,6 +35,8 @@
 #include "ardour/tempo.h"
 #include "ardour/types.h"
 
+#include "evoral/ControlList.h"
+
 #include "canvas/types.h"
 
 #include "gtkmm2ext/bindings.h"
@@ -56,6 +58,7 @@ namespace ArdourCanvas {
 	class Item;
 	class Line;
 	class Rectangle;
+	class Lollipop;
 }
 
 namespace PBD {
@@ -81,6 +84,7 @@ class ControlPoint;
 class AudioRegionView;
 class AutomationLine;
 class AutomationTimeAxisView;
+class VelocityGhostRegion;
 
 /** Class to manage current drags */
 class DragManager
@@ -91,7 +95,7 @@ public:
 	~DragManager ();
 
 	bool motion_handler (GdkEvent *, bool);
-
+	bool mid_drag_key_event (GdkEventKey*);
 	void abort ();
 	void add (Drag *);
 	void set (Drag *, GdkEvent *, Gdk::Cursor* c = MouseCursors::invalid_cursor());
@@ -148,7 +152,7 @@ class Drag
 {
 public:
 	Drag (Editor *, ArdourCanvas::Item *, Temporal::TimeDomain td, bool trackview_only = true, bool hide_snapped_cursor = true);
-	virtual ~Drag () {}
+	virtual ~Drag ();
 
 	void set_manager (DragManager* m) {
 		_drags = m;
@@ -234,6 +238,8 @@ public:
 		return true;
 	}
 
+	virtual bool mid_drag_key_event (GdkEventKey*)  { return false; }
+
 	bool initially_vertical() const {
 		return _initially_vertical;
 	}
@@ -321,7 +327,7 @@ protected:
 
 private:
 	bool _trackview_only; ///< true if pointer y value should always be relative to the top of the trackview group
-	bool _hide_snapped_cursor; ///< set true of ::motion does not call `set_snapped_cursor_position`
+	bool _hide_snapped_cursor; ///< set true of Drag::motion does not call `set_snapped_cursor_position`
 	bool _move_threshold_passed; ///< true if the move threshold has been passed, otherwise false
 	bool _starting_point_passed; ///< true if we called move () with first_move flag, otherwise false
 	bool _initially_vertical; ///< true if after move threshold is passed we appear to be moving vertically; undefined before that
@@ -1170,6 +1176,8 @@ private:
 		CopiedLocationMarkerInfo (ARDOUR::Location* l, ArdourMarker* m);
 	};
 
+	std::vector<ARDOUR::Location::ChangeSuspender> lcs;
+
 	typedef std::list<CopiedLocationMarkerInfo> CopiedLocationInfo;
 	CopiedLocationInfo _copied_locations;
 	ArdourCanvas::Points _points;
@@ -1339,18 +1347,6 @@ private:
 	bool _dragging_start;
 };
 
-/** Scrub drag in audition mode */
-class ScrubDrag : public Drag
-{
-public:
-	ScrubDrag (Editor *, ArdourCanvas::Item *);
-
-	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
-	void motion (GdkEvent *, bool);
-	void finished (GdkEvent *, bool);
-	void aborted (bool);
-};
-
 /** Drag in range select mode */
 class SelectionDrag : public Drag
 {
@@ -1537,7 +1533,7 @@ class RegionMarkerDrag : public Drag
 		return false;
 	}
 
-	void setup_pointer_sample_offset ();
+	void setup_pointer_offset ();
 
   private:
 	RegionView* rv;
@@ -1546,4 +1542,94 @@ class RegionMarkerDrag : public Drag
 	ARDOUR::CueMarker dragging_model;
 };
 
+
+class LollipopDrag : public Drag
+{
+  public:
+	LollipopDrag (Editor*, ArdourCanvas::Item*);
+	~LollipopDrag ();
+
+	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
+	void motion (GdkEvent *, bool);
+	void finished (GdkEvent *, bool);
+	void aborted (bool);
+
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	bool allow_horizontal_autoscroll () const {
+		return false;
+	}
+
+	bool x_movement_matters () const {
+		return false;
+	}
+
+	void setup_pointer_offset ();
+
+  private:
+	VelocityGhostRegion*      _region;
+	ArdourCanvas::Lollipop*   _primary;
+};
+
+template<typename OrderedPointList, typename OrderedPoint>
+class FreehandLineDrag : public Drag
+{
+  public:
+	FreehandLineDrag (Editor*, ArdourCanvas::Item*, ArdourCanvas::Rectangle&, Temporal::TimeDomain);
+	~FreehandLineDrag ();
+
+	void motion (GdkEvent*, bool);
+	void finished (GdkEvent*, bool);
+	bool mid_drag_key_event (GdkEventKey*);
+	virtual void point_added  (ArdourCanvas::Duple const & d, ArdourCanvas::Rectangle const & r, double last_x) {}
+	virtual void line_extended  (ArdourCanvas::Duple const & from, ArdourCanvas::Duple const & to, ArdourCanvas::Rectangle const & r, double last_x) {
+		point_added (to, r, last_x);
+	}
+
+  protected:
+	ArdourCanvas::Item* parent; /* we do not own this. If null, use base_rect as the parent */
+	ArdourCanvas::Rectangle& base_rect; /* we do not own this */
+	ArdourCanvas::PolyLine* dragging_line;
+	int direction;
+	int edge_x;
+	bool did_snap;
+	bool line_break_pending;
+	OrderedPointList drawn_points;
+	ArdourCanvas::Coord line_start_x;
+	ArdourCanvas::Coord line_start_y;
+
+	void maybe_add_point (GdkEvent*, Temporal::timepos_t const &, bool first_move);
+};
+
+class AutomationDrawDrag : public FreehandLineDrag<Evoral::ControlList::OrderedPoints, Evoral::ControlList::OrderedPoint>
+{
+  public:
+	AutomationDrawDrag (Editor*, ArdourCanvas::Item*, ArdourCanvas::Rectangle&, Temporal::TimeDomain);
+	~AutomationDrawDrag ();
+
+	void finished (GdkEvent*, bool);
+	void aborted (bool) {}
+};
+
+class VelocityLineDrag : public FreehandLineDrag<Evoral::ControlList::OrderedPoints, Evoral::ControlList::OrderedPoint>
+{
+  public:
+	VelocityLineDrag (Editor*, ArdourCanvas::Rectangle&, Temporal::TimeDomain);
+	~VelocityLineDrag ();
+
+	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
+	void finished (GdkEvent*, bool);
+	void aborted (bool);
+	void point_added  (ArdourCanvas::Duple const & d, ArdourCanvas::Rectangle const & r, double last_x);
+	void line_extended (ArdourCanvas::Duple const & from, ArdourCanvas::Duple const & to, ArdourCanvas::Rectangle const & r, double last_x);
+
+ private:
+	VelocityGhostRegion* grv;
+	bool drag_did_change;
+};
+
+
+
 #endif /* __gtk2_ardour_editor_drag_h_ */
