@@ -106,6 +106,17 @@ Editor::quick_export ()
 	dialog.run();
 }
 
+void
+Editor::surround_export ()
+{
+	if (!_session || !_session->vapor_export_barrier ()) {
+		return;
+	}
+	SimpleExportDialog dialog (*this, true);
+	dialog.set_session (_session);
+	dialog.run();
+}
+
 void
 Editor::loudness_assistant_marker ()
 {
@@ -173,10 +184,11 @@ Editor::measure_master_loudness (samplepos_t start, samplepos_t end, bool is_ran
 	ARDOUR::TimelineRange ar (timepos_t (start), timepos_t (end), 0);
 
 	LoudnessDialog ld (_session, ar, is_range_selection);
-
+#ifndef __APPLE__
 	if (own_window ()) {
 		ld.set_transient_for (*own_window ());
 	}
+#endif
 
 	ld.run ();
 }
