@@ -69,6 +69,8 @@ Editor::clear_marker_display ()
 		delete i->second;
 	}
 
+	entered_marker = 0;
+
 	location_markers.clear ();
 	_sorted_marker_lists.clear ();
 }
@@ -90,6 +92,34 @@ Editor::add_new_location (Location *location)
 	if (location->is_auto_loop()) {
 		update_loop_range_view ();
 	}
+
+	if (location->is_section ()) {
+		update_section_rects ();
+	}
+}
+
+static ArdourMarker::Type
+marker_type (Location* l, bool start = true)
+{
+	if (l->is_mark()) {
+		if (l->is_cd_marker()) {
+			return ArdourMarker::Mark;
+		} else if (l->is_cue_marker()) {
+			return ArdourMarker::Cue;
+		} else if (l->is_section()) {
+			return ArdourMarker::Section;
+		} else {
+			return ArdourMarker::Mark;
+		}
+	} else if (l->is_auto_loop()) {
+		return start ? ArdourMarker::LoopStart : ArdourMarker::LoopEnd;
+	} else if (l->is_auto_punch()) {
+		return start ? ArdourMarker::PunchIn : ArdourMarker::PunchOut;
+	} else if (l->is_session_range()) {
+		return start ? ArdourMarker::SessionStart : ArdourMarker::SessionEnd;
+	} else {
+		return start ? ArdourMarker::RangeStart : ArdourMarker::RangeEnd;
+	}
 }
 
 /** Add a new location, without a time-consuming update of all marker labels;
@@ -101,34 +131,50 @@ Editor::add_new_location_internal (Location* location)
 {
 	LocationMarkers *lam = new LocationMarkers;
 	std::string color;
+	MarkerBarType mark_type = MarkerBarType (0);
+	RangeBarType range_type = RangeBarType (0);
 
 	/* make a note here of which group this marker ends up in */
 	ArdourCanvas::Container* group = 0;
 
 	if (location->is_cd_marker()) {
 		color = X_("location cd marker");
+	} else if (location->is_section()) {
+		color = X_("location arrangement marker");
 	} else if (location->is_mark()) {
 		color = X_("location marker");
 	} else if (location->is_auto_loop()) {
 		color = X_("location loop");
 	} else if (location->is_auto_punch()) {
 		color = X_("location punch");
+	} else if (location->is_scene()) {
+		color = X_("location scene");
 	} else {
 		color = X_("location range");
 	}
 
 	if (location->is_mark()) {
 
-		if (location->is_cd_marker() && ruler_cd_marker_action->get_active()) {
-			lam->start = new ArdourMarker (*this, *cd_marker_group, color, location->name(), ArdourMarker::Mark, location->start());
-			group = cd_marker_group;
-		} else if (location->is_cue_marker() && ruler_cue_marker_action->get_active()) {
-			lam->start = new ArdourMarker (*this, *cue_marker_group, color, location->name(), ArdourMarker::Cue, location->start());
-			lam->start->set_cue_index(location->cue_id());
-			group = cue_marker_group;
-		} else {
-			lam->start = new ArdourMarker (*this, *marker_group, color, location->name(), ArdourMarker::Mark, location->start());
+		if (location->is_cd_marker()) {
 			group = marker_group;
+			mark_type = CDMarks;
+			lam->start = new ArdourMarker (*this, *group, color, location->name(), marker_type (location), location->start());
+		} else if (location->is_cue_marker()) {
+			group = marker_group;
+			mark_type = CueMarks;
+			lam->start = new ArdourMarker (*this, *group, color, location->name(), marker_type (location), location->start());
+			lam->start->set_cue_index(location->cue_id());
+		} else if (location->is_section()) {
+			group = section_marker_group;
+			lam->start = new ArdourMarker (*this, *group, color, location->name(), marker_type (location), location->start());
+		} else if (location->is_scene()) {
+			mark_type = CueMarks;
+			group = marker_group;
+			lam->start = new ArdourMarker (*this, *group, color, location->name(), marker_type (location), location->start());
+		} else {
+			group = marker_group;
+			mark_type = LocationMarks;
+			lam->start = new ArdourMarker (*this, *group, color, location->name(), marker_type (location), location->start());
 		}
 
 		lam->end = 0;
@@ -136,53 +182,65 @@ Editor::add_new_location_internal (Location* location)
 	} else if (location->is_auto_loop()) {
 
 		// transport marker
-		lam->start = new ArdourMarker (*this, *transport_marker_group, color,
-					 location->name(), ArdourMarker::LoopStart, location->start());
-		lam->end   = new ArdourMarker (*this, *transport_marker_group, color,
-					 location->name(), ArdourMarker::LoopEnd, location->end());
-		group = transport_marker_group;
+		group = range_marker_group;
+		range_type = LoopRange;
+		lam->start = new ArdourMarker (*this, *group, color,
+					 location->name(), marker_type (location), location->start());
+		lam->end   = new ArdourMarker (*this, *group, color,
+					 location->name(), marker_type (location, false), location->end());
 
 	} else if (location->is_auto_punch()) {
 
 		// transport marker
-		lam->start = new ArdourMarker (*this, *transport_marker_group, color,
-					 location->name(), ArdourMarker::PunchIn, location->start());
-		lam->end   = new ArdourMarker (*this, *transport_marker_group, color,
-					 location->name(), ArdourMarker::PunchOut, location->end());
-		group = transport_marker_group;
+		group = range_marker_group;
+		range_type = PunchRange;
+		lam->start = new ArdourMarker (*this, *group, color,
+					 location->name(), marker_type (location), location->start());
+		lam->end   = new ArdourMarker (*this, *group, color,
+					 location->name(), marker_type (location, false), location->end());
 
 	} else if (location->is_session_range()) {
 
 		// session range
-		lam->start = new ArdourMarker (*this, *marker_group, color, _("start"), ArdourMarker::SessionStart, location->start());
-		lam->end = new ArdourMarker (*this, *marker_group, color, _("end"), ArdourMarker::SessionEnd, location->end());
-		group = marker_group;
+		group = range_marker_group;
+		range_type = SessionRange;
+		lam->start = new ArdourMarker (*this, *group, color, _("start"), marker_type (location), location->start());
+		lam->end = new ArdourMarker (*this, *group, color, _("end"), marker_type (location, false), location->end());
 
 	} else {
 		// range marker
-		if (location->is_cd_marker() && ruler_cd_marker_action->get_active()) {
-			lam->start = new ArdourMarker (*this, *cd_marker_group, color,
-						 location->name(), ArdourMarker::RangeStart, location->start());
-			lam->end   = new ArdourMarker (*this, *cd_marker_group, color,
-						 location->name(), ArdourMarker::RangeEnd, location->end());
-			group = cd_marker_group;
-		} else {
-			lam->start = new ArdourMarker (*this, *range_marker_group, color,
-						 location->name(), ArdourMarker::RangeStart, location->start());
-			lam->end   = new ArdourMarker (*this, *range_marker_group, color,
-						 location->name(), ArdourMarker::RangeEnd, location->end());
-			group = range_marker_group;
-		}
+		group = range_marker_group;
+		range_type = OtherRange;
+		lam->start = new ArdourMarker (*this, *group, color,
+		                               location->name(), marker_type (location), location->start());
+		lam->end   = new ArdourMarker (*this, *group, color,
+		                               location->name(), marker_type (location, false), location->end());
 	}
 
+#if 0
 	if (location->position_time_domain() == Temporal::BeatTime) {
 		lam->set_name (string_compose ("%1%2", u8"\u266B", location->name ())); // BEAMED EIGHTH NOTES
 	}
+#endif
 
 	if (location->is_hidden ()) {
 		lam->hide();
 	} else {
-		lam->show ();
+		if (mark_type) {
+			if (!(_visible_marker_types & mark_type)) {
+				lam->hide ();
+			} else {
+				lam->show ();
+			}
+		} else if (range_type) {
+			if (!(_visible_range_types & range_type)) {
+				lam->hide ();
+			} else {
+				lam->show ();
+			}
+		} else {
+			lam->show ();
+		}
 	}
 
 	location->NameChanged.connect (*this, invalidator (*this), boost::bind (&Editor::location_changed, this, location), gui_context());
@@ -227,11 +285,7 @@ Editor::location_changed (Location *location)
 		return;
 	}
 
-	if (location->position_time_domain() == Temporal::BeatTime) {
-		lam->set_name (string_compose ("%1%2", u8"\u266B", location->name ())); // BEAMED EIGHTH NOTES
-	} else {
-		lam->set_name (location->name ());
-	}
+	lam->set_name (location->name ());
 
 	if (location->is_cue_marker()) {
 		lam->start->set_cue_index (location->cue_id());
@@ -249,6 +303,9 @@ Editor::location_changed (Location *location)
 	if (lam->end) {
 		check_marker_label (lam->end);
 	}
+	if (location->is_section ()) {
+		update_section_rects ();
+	}
 }
 
 /** Look at a marker and check whether its label, and those of the previous and next markers,
@@ -265,7 +322,10 @@ Editor::check_marker_label (ArdourMarker* m)
 
 	list<ArdourMarker*>::iterator prev = sorted.end ();
 	list<ArdourMarker*>::iterator next = i;
-	++next;
+
+	if (next != sorted.end()) {
+		++next;
+	}
 
 	/* Look to see if the previous marker is still behind `m' in time */
 	if (i != sorted.begin()) {
@@ -273,7 +333,7 @@ Editor::check_marker_label (ArdourMarker* m)
 		prev = i;
 		--prev;
 
-		if ((*prev)->position() > m->position()) {
+		if ((*prev)->position() >= m->position()) {
 			/* This marker is no longer in the correct order with the previous one, so
 			 * update all the markers in this group.
 			 */
@@ -310,8 +370,11 @@ Editor::check_marker_label (ArdourMarker* m)
 		}
 	}
 
-	if (next != sorted.end()) {
+	while (next != sorted.end() && (*next)->position () == m->position ()) {
+		++next;
+	}
 
+	if (next != sorted.end()) {
 		/* Update just the available space between this marker and the next */
 
 		double const p = sample_to_pixel (m->position().distance ((*next)->position()).samples());
@@ -332,10 +395,21 @@ Editor::check_marker_label (ArdourMarker* m)
 
 struct MarkerComparator {
 	bool operator() (ArdourMarker const * a, ArdourMarker const * b) {
+		if (a->position() == b->position()) {
+			return a->label_on_left ();
+		}
 		return a->position() < b->position();
 	}
 };
 
+void
+Editor::update_all_marker_lanes ()
+{
+	for (auto & lam : location_markers) {
+		lam.second->set_position (lam.first->start(), lam.first->end());
+	}
+}
+
 /** Update all marker labels in all groups */
 void
 Editor::update_marker_labels ()
@@ -371,14 +445,29 @@ Editor::update_marker_labels (ArdourCanvas::Item* group)
 	while (i != sorted.end()) {
 
 		if (prev != sorted.end()) {
-			double const p = sample_to_pixel ((*prev)->position().distance ((*i)->position()).samples());
+
+			list<ArdourMarker*>::iterator pi = prev;
+			while (pi != sorted.begin () && (*pi)->position () == (*i)->position()) {
+				--pi;
+			}
+
+			double p = sample_to_pixel ((*pi)->position().distance ((*i)->position()).samples());
+
+			if (p == 0) {
+				p = DBL_MAX;
+			}
 
 			if ((*prev)->label_on_left()) {
 				(*i)->set_left_label_limit (p);
 			} else {
 				(*i)->set_left_label_limit (p / 2);
 			}
+		} else {
+			(*i)->set_left_label_limit (DBL_MAX);
+		}
 
+		while (next != sorted.end() && (*next)->position () == (*i)->position ()) {
+			++next;
 		}
 
 		if (next != sorted.end()) {
@@ -391,6 +480,8 @@ Editor::update_marker_labels (ArdourCanvas::Item* group)
 			}
 
 			++next;
+		} else {
+			(*i)->set_right_label_limit (DBL_MAX);
 		}
 
 		prev = i;
@@ -410,11 +501,22 @@ Editor::location_flags_changed (Location *location)
 		return;
 	}
 
-	// move cd markers to/from cd marker bar as appropriate
-	ensure_cd_marker_updated (lam, location);
+	if (lam->start->type () != marker_type (location)) {
+		/* this removes the current location and calls
+		 * refresh_location_display () which re-adds it
+		 * using the correct type.
+		 */
+		location_gone (location);
+		return;
+	}
+
+	// moved markers to/from cd marker bar as appropriate
+	ensure_marker_updated (lam, location);
 
 	if (location->is_cd_marker()) {
 		lam->set_color ("location cd marker");
+	} else if (location->is_section()) {
+		lam->set_color ("location arrangement marker");
 	} else if (location->is_mark()) {
 		lam->set_color ("location marker");
 	} else if (location->is_auto_punch()) {
@@ -433,97 +535,40 @@ Editor::location_flags_changed (Location *location)
 }
 
 void
-Editor::update_cd_marker_display ()
+Editor::update_marker_display ()
 {
-	for (LocationMarkerMap::iterator i = location_markers.begin(); i != location_markers.end(); ++i) {
-		LocationMarkers * lam = i->second;
-		Location * location = i->first;
-
-		ensure_cd_marker_updated (lam, location);
-	}
-}
-
-
-void Editor::ensure_cd_marker_updated (LocationMarkers * lam, Location * location)
-{
-	if (location->is_cd_marker()
-	    && (ruler_cd_marker_action->get_active() &&  lam->start->get_parent() != cd_marker_group))
-	{
-		//cerr << "reparenting non-cd marker so it can be relocated: " << location->name() << endl;
-		if (lam->start) {
-			lam->start->reparent (*cd_marker_group);
-		}
-		if (lam->end) {
-			lam->end->reparent (*cd_marker_group);
-		}
-	}
-	else if ( (!location->is_cd_marker() || !ruler_cd_marker_action->get_active())
-		  && (lam->start->get_parent() == cd_marker_group))
-	{
-		//cerr << "reparenting non-cd marker so it can be relocated: " << location->name() << endl;
-		if (location->is_mark()) {
-			if (lam->start) {
-				lam->start->reparent (*marker_group);
-			}
-			if (lam->end) {
-				lam->end->reparent (*marker_group);
-			}
-		}
-		else {
-			if (lam->start) {
-				lam->start->reparent (*range_marker_group);
-			}
-			if (lam->end) {
-				lam->end->reparent (*range_marker_group);
-			}
-		}
+	for (auto const& i : location_markers) {
+		ensure_marker_updated (i.second, i.first);
 	}
 }
 
 void
-Editor::update_cue_marker_display ()
+Editor::reparent_location_markers (LocationMarkers* lam, ArdourCanvas::Item* new_parent)
 {
-	for (LocationMarkerMap::iterator i = location_markers.begin(); i != location_markers.end(); ++i) {
-		LocationMarkers * lam = i->second;
-		Location * location = i->first;
-
-		ensure_cue_marker_updated (lam, location);
+	if (lam->start && lam->start->get_parent() != new_parent) {
+		lam->start->reparent (*new_parent);
+		remove_sorted_marker (lam->start);
+		_sorted_marker_lists[new_parent].push_back (lam->start);
+	}
+	if (lam->end && lam->end->get_parent() != new_parent) {
+		lam->end->reparent (*new_parent);
+		remove_sorted_marker (lam->end);
+		_sorted_marker_lists[new_parent].push_back (lam->end);
 	}
 }
 
-void Editor::ensure_cue_marker_updated (LocationMarkers * lam, Location * location)
+void Editor::ensure_marker_updated (LocationMarkers* lam, Location* location)
 {
-	if (location->is_cd_marker()
-	    && (ruler_cd_marker_action->get_active() &&  lam->start->get_parent() != cd_marker_group))
-	{
-		//cerr << "reparenting non-cd marker so it can be relocated: " << location->name() << endl;
-		if (lam->start) {
-			lam->start->reparent (*cd_marker_group);
-		}
-		if (lam->end) {
-			lam->end->reparent (*cd_marker_group);
-		}
-	}
-	else if ( (!location->is_cd_marker() || !ruler_cd_marker_action->get_active())
-		  && (lam->start->get_parent() == cd_marker_group))
-	{
-		//cerr << "reparenting non-cd marker so it can be relocated: " << location->name() << endl;
-		if (location->is_mark()) {
-			if (lam->start) {
-				lam->start->reparent (*marker_group);
-			}
-			if (lam->end) {
-				lam->end->reparent (*marker_group);
-			}
-		}
-		else {
-			if (lam->start) {
-				lam->start->reparent (*range_marker_group);
-			}
-			if (lam->end) {
-				lam->end->reparent (*range_marker_group);
-			}
-		}
+	if (location->is_cd_marker()) {
+		reparent_location_markers (lam, marker_group);
+	} else if (location->is_scene()) {
+		reparent_location_markers (lam, marker_group);
+	} else if (location->is_section()) {
+		reparent_location_markers (lam, section_marker_group);
+	} else if (location->is_cue_marker()) {
+		reparent_location_markers (lam, marker_group);
+	} else if (location->is_mark() || location->matches (Location::Flags(0))) {
+		reparent_location_markers (lam, marker_group);
 	}
 }
 
@@ -601,8 +646,10 @@ Editor::refresh_location_display_internal (const Locations::LocationList& locati
 		LocationMarkerMap::iterator x;
 
 		if ((x = location_markers.find (*i)) != location_markers.end()) {
-			x->second->valid = true;
-			continue;
+			if (x->second->start && x->second->start->type () == marker_type (*i)) {
+				x->second->valid = true;
+				continue;
+			}
 		}
 
 		add_new_location_internal (*i);
@@ -626,6 +673,11 @@ Editor::refresh_location_display_internal (const Locations::LocationList& locati
 
 			LocationMarkers* m = i->second;
 			location_markers.erase (i);
+
+			if (m && (entered_marker == m->start || entered_marker == m->end)) {
+				entered_marker = 0;
+			}
+
 			delete m;
 		}
 
@@ -648,6 +700,47 @@ Editor::refresh_location_display ()
 	update_marker_labels ();
 }
 
+void
+Editor::update_section_rects ()
+{
+	ENSURE_GUI_THREAD (*this, &Editor::update_section_rects);
+	if (!_session) {
+		return;
+	}
+	section_marker_bar->clear (true);
+
+	timepos_t start;
+	timepos_t end;
+	std::vector<Locations::LocationPair> locs;
+
+	Locations* loc = _session->locations ();
+	Location*  l   = NULL;
+	bool bright    = false;
+
+	do {
+		l = loc->next_section_iter (l, start, end, locs);
+		if (l) {
+			double const left  = sample_to_pixel (start.samples ());
+			double const right = sample_to_pixel (end.samples ());
+
+			ArdourCanvas::Rectangle* rect = new ArdourCanvas::Rectangle (section_marker_bar, ArdourCanvas::Rect (left, 1, right, timebar_height));
+			rect->set_fill (true);
+			rect->set_outline_what(ArdourCanvas::Rectangle::What(0));
+			rect->raise_to_top ();
+
+			std::string const color = bright ? "arrangement rect" : "arrangement rect alt";
+			rect->set_fill_color (UIConfiguration::instance().color (color));
+			rect->Event.connect (sigc::bind (sigc::mem_fun (this, &Editor::section_rect_event), l,  rect, color));
+
+			Editor::LocationMarkers* markers = find_location_markers (l);
+			if (markers) {
+				markers->set_color (color);
+			}
+			bright = !bright;
+		}
+	} while (l);
+}
+
 void
 Editor::LocationMarkers::hide()
 {
@@ -735,50 +828,6 @@ Editor::LocationMarkers::setup_lines ()
 	}
 }
 
-void
-Editor::mouse_add_new_marker (timepos_t where, Location::Flags extra_flags, int32_t cue_id)
-{
-	if (!_session) {
-		return;
-	}
-
-	string markername;
-	string namebase;
-	Location::Flags flags = Location::Flags (extra_flags|Location::IsMark);
-
-	std::cerr << "adding marker @ " << where << std::endl;
-
-	if (flags & Location::IsCueMarker) {
-		/* XXX i18n needed for cue letter names */
-		markername = string_compose (_("cue %1"), cue_marker_name (cue_id));
-	} else {
-		namebase = _("mark");
-		_session->locations()->next_available_name (markername, namebase);
-
-		if (!choose_new_marker_name (markername)) {
-			return;
-		}
-	}
-
-	Location *location = new Location (*_session, where, where, markername, flags, cue_id);
-	begin_reversible_command (_("add marker"));
-
-	XMLNode &before = _session->locations()->get_state();
-	_session->locations()->add (location, true);
-	XMLNode &after = _session->locations()->get_state();
-	_session->add_command (new MementoCommand<Locations>(*(_session->locations()), &before, &after));
-
-	/* find the marker we just added */
-
-	LocationMarkers *lam = find_location_markers (location);
-	if (lam) {
-		/* make it the selected marker */
-		selection->set (lam->start);
-	}
-
-	commit_reversible_command ();
-}
-
 void
 Editor::mouse_add_new_loop (timepos_t where)
 {
@@ -860,6 +909,10 @@ Editor::remove_marker (ArdourMarker* marker)
 		return;
 	}
 
+	if (entered_marker == marker) {
+		entered_marker = 0;
+	}
+
 	if (marker->type() == ArdourMarker::RegionCue) {
 		Glib::signal_idle().connect (sigc::bind (sigc::mem_fun(*this, &Editor::really_remove_region_marker), marker));
 	} else {
@@ -934,6 +987,11 @@ Editor::location_gone (Location *location)
 
 			LocationMarkers* m = i->second;
 			location_markers.erase (i);
+
+			if (m && (entered_marker == m->start || entered_marker == m->end)) {
+				entered_marker = 0;
+			}
+
 			delete m;
 
 			/* Markers that visually overlap with this (removed) marker
@@ -945,6 +1003,10 @@ Editor::location_gone (Location *location)
 			break;
 		}
 	}
+
+	if (location->is_section ()) {
+		update_section_rects ();
+	}
 }
 
 void
@@ -971,17 +1033,14 @@ Editor::tempo_map_marker_context_menu (GdkEventButton* ev, ArdourCanvas::Item* i
 	if (mm) {
 		can_remove = !mm->meter().map().is_initial (mm->meter());
 		build_meter_marker_menu (mm, can_remove);
-		meter_marker_menu->popup (1, ev->time);
+		meter_marker_menu->popup (ev->button, ev->time);
 	} else if (tm) {
-		if (!tm->tempo().active()) {
-			return;
-		}
 		can_remove = !tm->tempo().map().is_initial(tm->tempo()) && !tm->tempo().locked_to_meter();
 		build_tempo_marker_menu (tm, can_remove);
-		tempo_marker_menu->popup (1, ev->time);
+		tempo_marker_menu->popup (ev->button, ev->time);
 	} else if (bm) {
 		build_bbt_marker_menu (bm);
-		bbt_marker_menu->popup (1, ev->time);
+		bbt_marker_menu->popup (ev->button, ev->time);
 	} else {
 		return;
 	}
@@ -1010,7 +1069,7 @@ Editor::marker_context_menu (GdkEventButton* ev, ArdourCanvas::Item* item)
 		build_range_marker_menu (loc, loc == transport_loop_location() || loc == transport_punch_location(), loc->is_session_range());
 
 		marker_menu_item = item;
-		range_marker_menu->popup (1, ev->time);
+		range_marker_menu->popup (ev->button, ev->time);
 
 	} else if (loc->is_mark()) {
 
@@ -1031,12 +1090,12 @@ Editor::marker_context_menu (GdkEventButton* ev, ArdourCanvas::Item* item)
 			}
 #endif
 			marker_menu_item = item;
-			marker_menu->popup (1, ev->time);
+			marker_menu->popup (ev->button, ev->time);
 
 	} else if (loc->is_range_marker()) {
 		build_range_marker_menu (loc, false, false);
 		marker_menu_item = item;
-		range_marker_menu->popup (1, ev->time);
+		range_marker_menu->popup (ev->button, ev->time);
 	}
 }
 
@@ -1047,7 +1106,7 @@ Editor::new_transport_marker_context_menu (GdkEventButton* ev, ArdourCanvas::Ite
 		build_new_transport_marker_menu ();
 	}
 
-	new_transport_marker_menu->popup (1, ev->time);
+	new_transport_marker_menu->popup (ev->button, ev->time);
 
 }
 
@@ -1085,7 +1144,6 @@ Editor::build_marker_menu (Location* loc)
 		items.push_back (MenuElem (_("Create Range to Next Marker"), sigc::mem_fun(*this, &Editor::marker_menu_range_to_next)));
 
 		items.push_back (MenuElem (_("Promote to Time Origin"), sigc::mem_fun(*this, &Editor::marker_menu_set_origin)));
-		items.push_back (MenuElem (_("Hide"), sigc::mem_fun(*this, &Editor::marker_menu_hide)));
 		items.push_back (MenuElem (_("Rename..."), sigc::mem_fun(*this, &Editor::marker_menu_rename)));
 
 		items.push_back (CheckMenuElem (_("Lock")));
@@ -1096,14 +1154,18 @@ Editor::build_marker_menu (Location* loc)
 		lock_item->signal_activate().connect (sigc::mem_fun (*this, &Editor::toggle_marker_menu_lock));
 	}
 
-	items.push_back (CheckMenuElem (_("Glue to Bars and Beats")));
-	Gtk::CheckMenuItem* glue_item = static_cast<Gtk::CheckMenuItem*> (&items.back());
-	glue_item->set_active (loc->position_time_domain() == Temporal::BeatTime);
-
-	glue_item->signal_activate().connect (sigc::mem_fun (*this, &Editor::toggle_marker_menu_glue));
-
 	items.push_back (SeparatorElem());
 
+	if (!loc->is_range () && !loc->is_xrun ()) {
+		items.push_back (CheckMenuElem (_("Arrangement Boundary")));
+		Gtk::CheckMenuItem* item = static_cast<Gtk::CheckMenuItem*> (&items.back());
+		if (loc->is_section ()) {
+			item->set_active ();
+		}
+		item->signal_activate().connect (sigc::mem_fun (*this, &Editor::toggle_marker_section));
+		items.push_back (SeparatorElem());
+	}
+
 	items.push_back (MenuElem (_("Remove"), sigc::mem_fun(*this, &Editor::marker_menu_remove)));
 }
 
@@ -1130,13 +1192,6 @@ Editor::build_range_marker_menu (Location* loc, bool loop_or_punch, bool session
 
 	items.push_back (MenuElem (_("Zoom to Range"), sigc::mem_fun (*this, &Editor::marker_menu_zoom_to_range)));
 
-	items.push_back (SeparatorElem());
-	items.push_back (CheckMenuElem (_("Glue to Bars and Beats")));
-
-	Gtk::CheckMenuItem* glue_item = static_cast<Gtk::CheckMenuItem*> (&items.back());
-	glue_item->set_active (loc->position_time_domain() == Temporal::BeatTime);
-	glue_item->signal_activate().connect (sigc::mem_fun (*this, &Editor::toggle_marker_menu_glue));
-
 	items.push_back (SeparatorElem());
 	items.push_back (MenuElem (_("Loudness Assistant..."), sigc::mem_fun(*this, &Editor::loudness_assistant_marker)));
 	items.push_back (MenuElem (_("Export Range..."), sigc::mem_fun(*this, &Editor::export_range)));
@@ -1312,7 +1367,7 @@ Editor::marker_menu_select_all_selectables_using_range ()
 	bool is_start;
 
 	if (((l = find_location_from_marker (marker, is_start)) != 0) && (l->end() > l->start())) {
-		select_all_within (l->start(), l->end(), 0,  DBL_MAX, track_views, Selection::Set, false);
+		select_all_within (l->start(), l->end(), 0,  DBL_MAX, track_views, SelectionSet, false);
 	}
 
 }
@@ -1737,6 +1792,28 @@ Editor::toggle_marker_menu_lock ()
 	}
 }
 
+void
+Editor::toggle_marker_section ()
+{
+	ArdourMarker* marker;
+
+	if ((marker = reinterpret_cast<ArdourMarker *> (marker_menu_item->get_data ("marker"))) == 0) {
+		fatal << _("programming error: marker canvas item has no marker object pointer!") << endmsg;
+		abort(); /*NOTREACHED*/
+	}
+
+	Location* loc;
+	bool ignored;
+
+	loc = find_location_from_marker (marker, ignored);
+
+	if (!loc) {
+		return;
+	}
+
+	loc->set_section (!loc->is_section ());
+}
+
 void
 Editor::marker_menu_rename ()
 {
@@ -1772,10 +1849,12 @@ Editor::rename_marker(ArdourMarker *marker)
 
 	dialog.set_prompt (_("New Name:"));
 
-	if (loc->is_mark()) {
-		dialog.set_title (_("Rename Mark"));
-	} else {
+	if (loc->is_section()) {
+		dialog.set_title (_("Rename Arrangement Section"));
+	} else if (loc->is_range()) {
 		dialog.set_title (_("Rename Range"));
+	} else {
+		dialog.set_title (_("Rename Mark"));
 	}
 
 	dialog.set_name ("MarkRenameWindow");
@@ -1811,8 +1890,6 @@ void
 Editor::new_transport_marker_menu_popdown ()
 {
 	// hide rects
-	transport_bar_drag_rect->hide();
-
 	_drags->abort ();
 }
 
@@ -1951,39 +2028,6 @@ Editor::jump_to_loop_marker (bool start)
 	}
 }
 
-void
-Editor::toggle_marker_menu_glue ()
-{
-	ArdourMarker* marker;
-
-	if ((marker = reinterpret_cast<ArdourMarker *> (marker_menu_item->get_data ("marker"))) == 0) {
-		fatal << _("programming error: marker canvas item has no marker object pointer!") << endmsg;
-		abort(); /*NOTREACHED*/
-	}
-
-	Location* loc;
-	bool ignored;
-
-	loc = find_location_from_marker (marker, ignored);
-
-	if (!loc) {
-		return;
-	}
-
-	begin_reversible_command (_("change marker lock style"));
-	XMLNode &before = _session->locations()->get_state();
-
-	if (loc->position_time_domain() == Temporal::BeatTime) {
-		loc->set_position_time_domain (Temporal::AudioTime);
-	} else {
-		loc->set_position_time_domain (Temporal::BeatTime);
-	}
-
-	XMLNode &after = _session->locations()->get_state();
-	_session->add_command(new MementoCommand<Locations>(*(_session->locations()), &before, &after));
-	commit_reversible_command ();
-}
-
 void
 Editor::toggle_marker_lines ()
 {
@@ -2041,3 +2085,152 @@ Editor::toggle_cue_behavior ()
 		_session->config.set_cue_behavior (ARDOUR::CueBehavior (cb | ARDOUR::FollowCues));
 	}
 }
+
+void
+Editor::set_visible_marker_types (MarkerBarType mbt)
+{
+	_visible_marker_types = mbt;
+	update_mark_and_range_visibility ();
+	VisibleMarkersChanged ();
+}
+
+void
+Editor::set_visible_range_types (RangeBarType rbt)
+{
+	_visible_range_types = rbt;
+	update_mark_and_range_visibility ();
+	VisibleRangesChanged ();
+}
+
+Editor::MarkerBarType
+Editor::visible_marker_types () const
+{
+	return _visible_marker_types;
+}
+
+
+Editor::RangeBarType
+Editor::visible_range_types () const
+{
+	return _visible_range_types;
+}
+
+void
+Editor::update_mark_and_range_visibility ()
+{
+	for (auto & l : location_markers) {
+
+		Location* location = l.first;
+		LocationMarkers* lam = l.second;
+
+		MarkerBarType mark_type = MarkerBarType (0);
+		RangeBarType range_type = RangeBarType (0);
+
+		if (location->is_mark()) {
+
+			if (location->is_cd_marker()) {
+				mark_type = CDMarks;
+			} else if (location->is_cue_marker()) {
+				mark_type = CueMarks;
+			} else if (location->is_section()) {
+
+			} else if (location->is_scene()) {
+				mark_type = SceneMarks;
+			} else {
+				mark_type = LocationMarks;
+			}
+
+		} else if (location->is_auto_loop()) {
+			range_type = LoopRange;
+		} else if (location->is_auto_punch()) {
+			range_type = PunchRange;
+		} else if (location->is_session_range()) {
+			range_type = SessionRange;
+
+		} else {
+			range_type = OtherRange;
+		}
+
+		if (location->is_hidden ()) {
+			lam->hide();
+		} else {
+			if (mark_type) {
+				if (!(_visible_marker_types & mark_type)) {
+					lam->hide ();
+				} else {
+					lam->show ();
+				}
+			} else if (range_type) {
+				if (!(_visible_range_types & range_type)) {
+					lam->hide ();
+				} else {
+					lam->show ();
+				}
+			} else {
+				lam->show ();
+			}
+		}
+	}
+}
+
+void
+Editor::show_marker_type (MarkerBarType mbt)
+{
+	Glib::RefPtr<Gtk::RadioAction> action;
+	switch (mbt) {
+	case CDMarks:
+		action = cd_marker_action;
+		break;
+	case CueMarks:
+		action = cue_marker_action;
+		break;
+	case SceneMarks:
+		action = scene_marker_action;
+		break;
+	case LocationMarks:
+		action = location_marker_action;
+		break;
+	default:
+		action = all_marker_action;
+		break;
+	}
+
+	if (action->get_active()) {
+		/* Only change things for the currently active action, since
+		   this will be called for both the deactivated action, and the
+		   newly activated one.
+		*/
+		set_visible_marker_types (mbt);
+	}
+}
+
+void
+Editor::show_range_type (RangeBarType rbt)
+{
+	Glib::RefPtr<Gtk::RadioAction> action;
+	switch (rbt) {
+	case OtherRange:
+		action = other_range_action;
+		break;
+	case PunchRange:
+		action = punch_range_action;
+		break;
+	case LoopRange:
+		action = loop_range_action;
+		break;
+	case SessionRange:
+		action = session_range_action;
+		break;
+	default:
+		action = all_range_action;
+		break;
+	}
+
+	if (action->get_active()) {
+		/* Only change things for the currently active action, since
+		   this will be called for both the deactivated action, and the
+		   newly activated one.
+		*/
+		set_visible_range_types (rbt);
+	}
+}
