@@ -82,6 +82,7 @@
 #include "mouse_cursors.h"
 #include "editor_cursors.h"
 #include "region_peak_cursor.h"
+#include "velocity_ghost_region.h"
 #include "verbose_cursor.h"
 #include "note.h"
 
@@ -269,10 +270,10 @@ Editor::get_mouse_mode_action(MouseMode m) const
 		return ActionManager::get_action (X_("MouseMode"), X_("set-mouse-mode-draw"));
 	case MouseTimeFX:
 		return ActionManager::get_action (X_("MouseMode"), X_("set-mouse-mode-timefx"));
+	case MouseGrid:
+		return ActionManager::get_action (X_("MouseMode"), X_("set-mouse-mode-grid"));
 	case MouseContent:
 		return ActionManager::get_action (X_("MouseMode"), X_("set-mouse-mode-content"));
-	case MouseAudition:
-		return ActionManager::get_action (X_("MouseMode"), X_("set-mouse-mode-audition"));
 	}
 	return Glib::RefPtr<Action>();
 }
@@ -288,12 +289,6 @@ Editor::set_mouse_mode (MouseMode m, bool force)
 		return;
 	}
 
-	if (ARDOUR::Profile->get_mixbus()) {
-		if (m == MouseAudition) {
-			m = MouseRange;
-		}
-	}
-
 	Glib::RefPtr<Action>       act  = get_mouse_mode_action(m);
 	Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic(act);
 
@@ -307,12 +302,6 @@ Editor::set_mouse_mode (MouseMode m, bool force)
 void
 Editor::mouse_mode_toggled (MouseMode m)
 {
-	if (ARDOUR::Profile->get_mixbus()) {
-		if (m == MouseAudition)  {
-			m = MouseRange;
-		}
-	}
-
 	Glib::RefPtr<Action>       act  = get_mouse_mode_action(m);
 	Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic(act);
 
@@ -324,13 +313,6 @@ Editor::mouse_mode_toggled (MouseMode m)
 		return;
 	}
 
-	if (_session && mouse_mode == MouseAudition) {
-		/* stop transport and reset default speed to avoid oddness with
-		   auditioning */
-		_session->request_stop ();
-		_session->reset_transport_speed ();
-	}
-
 	const bool was_internal = internal_editing();
 
 	mouse_mode = m;
@@ -375,6 +357,15 @@ Editor::mouse_mode_toggled (MouseMode m)
 		_draw_box_spacer.hide();
 	}
 
+	if (mouse_mode == MouseGrid) {
+		grid_box.show();
+		_grid_box_spacer.show ();
+		_canvas_grid_zone->set_ignore_events (false);
+	} else {
+		grid_box.hide ();
+		_grid_box_spacer.hide ();
+		_canvas_grid_zone->set_ignore_events (true);
+	}
 
 	if (internal_editing()) {
 
@@ -396,16 +387,11 @@ Editor::mouse_mode_toggled (MouseMode m)
 		*/
 		ActionManager::set_sensitive (_midi_actions, true);
 
-		/* mark "magic widget focus" so that we handle key events
-		 * correctly
-		 */
-		Keyboard::magic_widget_grab_focus ();
 	} else {
 		/* undo some of the above actions, since we're not in internal
 		   edit mode.
 		*/
 		ActionManager::set_sensitive (_midi_actions, false);
-		Keyboard::magic_widget_drop_focus ();
 	}
 
 	if (was_internal && !internal_editing()) {
@@ -420,6 +406,11 @@ Editor::mouse_mode_toggled (MouseMode m)
 	update_all_enter_cursors ();
 
 	MouseModeChanged (); /* EMIT SIGNAL */
+
+	if ((was_internal && !internal_editing()) ||
+	    (!was_internal && internal_editing())) {
+		queue_redisplay_track_views ();
+	}
 }
 
 bool
@@ -442,10 +433,16 @@ Editor::update_time_selection_display ()
 		break;
 	case MouseDraw:
 		/* Clear regions, but not time or tracks, since that
-		 would destroy the range selection rectangle, which we need to stick
-		 around for AutomationRangeDrag. */
+		 * would destroy the range selection rectangle, which we need to stick
+		 * around for AutomationRangeDrag. */
 		selection->clear_regions ();
 		selection->clear_playlists ();
+		/* .. unless there is no track selection (i.e. arrangement section
+		 * selection). In which case time is cleared too.
+		 */
+		if (selection->tracks.empty ()) {
+			selection->clear_time ();
+		}
 		break;
 	case MouseContent:
 		/* This handles internal edit.
@@ -469,14 +466,7 @@ Editor::update_time_selection_display ()
 		selection->clear_tracks ();
 		break;
 
-	case MouseAudition:
-		/*Don't lose lines or points if no action in this mode */
-		selection->clear_regions ();
-		selection->clear_playlists ();
-		selection->clear_time ();
-		selection->clear_tracks ();
-		break;
-
+	case MouseGrid:
 	default:
 		/*Clear everything */
 		selection->clear_objects();
@@ -543,7 +533,6 @@ Editor::button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType it
 	}
 
 	if (((mouse_mode != MouseObject) &&
-	     (mouse_mode != MouseAudition || item_type != RegionItem) &&
 	     (mouse_mode != MouseTimeFX || item_type != RegionItem) &&
 	     (mouse_mode != MouseDraw) &&
 	     (mouse_mode != MouseContent || item_type == RegionItem)) ||
@@ -565,7 +554,7 @@ Editor::button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType it
 		}
 	}
 
-	Selection::Operation op = ArdourKeyboard::selection_type (event->button.state);
+	SelectionOperation op = ArdourKeyboard::selection_type (event->button.state);
 	bool press = (event->type == GDK_BUTTON_PRESS);
 
 	if (press) {
@@ -586,7 +575,7 @@ Editor::button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType it
 					 * so, "collapse" the selection to just this track
 					 */
 					if (!selection->selected (clicked_axisview)) {
-						set_selected_track_as_side_effect (Selection::Set);
+						set_selected_track_as_side_effect (SelectionSet);
 					}
 				}
 			} else {
@@ -622,7 +611,7 @@ Editor::button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType it
 				if (event->button.button != 3) {
 					_mouse_changed_selection |= set_selected_control_point_from_click (press, op);
 				} else {
-					_mouse_changed_selection |= set_selected_control_point_from_click (press, Selection::Set);
+					_mouse_changed_selection |= set_selected_control_point_from_click (press, SelectionSet);
 				}
 			}
 			break;
@@ -643,34 +632,37 @@ Editor::button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType it
 				selectables.push_back (argl->nth (after));
 
 				switch (op) {
-					case Selection::Set:
-						if (press) {
-							selection->set (selectables);
-							_mouse_changed_selection = true;
-						}
-						break;
-					case Selection::Add:
-						if (press) {
-							selection->add (selectables);
-							_mouse_changed_selection = true;
-						}
-						break;
-					case Selection::Toggle:
-						if (press) {
-							selection->toggle (selectables);
-							_mouse_changed_selection = true;
-						}
-						break;
+				case SelectionSet:
+					if (press) {
+						selection->set (selectables);
+						_mouse_changed_selection = true;
+					}
+					break;
+				case SelectionAdd:
+					if (press) {
+						selection->add (selectables);
+						_mouse_changed_selection = true;
+					}
+					break;
+				case SelectionToggle:
+					if (press) {
+						selection->toggle (selectables);
+						_mouse_changed_selection = true;
+					}
+					break;
 
-					case Selection::Extend:
-						/* XXX */
-						break;
+				case SelectionExtend:
+					/* XXX */
+					break;
+				case SelectionRemove:
+					/* XXX */
+					break;
 				}
 			}
 			break;
 
 		case AutomationLineItem:
-			if (eff_mouse_mode != MouseRange) {
+			if (eff_mouse_mode != MouseRange && eff_mouse_mode != MouseDraw) {
 				AutomationLine* al = reinterpret_cast<AutomationLine*> (item->get_data ("line"));
 				std::list<Selectable*> selectables;
 				double mx = event->button.x;
@@ -689,28 +681,30 @@ Editor::button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType it
 				selectables.push_back (al->nth (after));
 
 				switch (op) {
-					case Selection::Set:
-						if (press) {
-							selection->set (selectables);
-							_mouse_changed_selection = true;
-						}
-						break;
-					case Selection::Add:
-						if (press) {
-							selection->add (selectables);
-							_mouse_changed_selection = true;
-						}
-						break;
-					case Selection::Toggle:
-						if (press) {
-							selection->toggle (selectables);
-							_mouse_changed_selection = true;
-						}
-						break;
-
-					case Selection::Extend:
-						/* XXX */
-						break;
+				case SelectionSet:
+					if (press) {
+						selection->set (selectables);
+						_mouse_changed_selection = true;
+					}
+					break;
+				case SelectionAdd:
+					if (press) {
+						selection->add (selectables);
+						_mouse_changed_selection = true;
+					}
+					break;
+				case SelectionToggle:
+					if (press) {
+						selection->toggle (selectables);
+						_mouse_changed_selection = true;
+					}
+					break;
+				case SelectionExtend:
+					/* XXX */
+					break;
+				case SelectionRemove:
+					/* not relevant */
+					break;
 				}
 			}
 			break;
@@ -728,7 +722,7 @@ Editor::button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType it
 			break;
 
 		case AutomationTrackItem:
-			if (eff_mouse_mode != MouseDraw && op == Selection::Set) {
+			if (eff_mouse_mode != MouseDraw && op == SelectionSet) {
 				set_selected_track_as_side_effect (op);
 			}
 			break;
@@ -786,6 +780,9 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 			}
 		}
 		return true;
+	case SceneMarkerItem:
+		_drags->set (new MarkerDrag (this, item), event);
+		return true;
 
 	case TempoMarkerItem:
 		if (ArdourKeyboard::indicates_constraint (event->button.state)) {
@@ -820,10 +817,6 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 		return true;
 		break;
 
-	case MappingBarItem:
-		choose_mapping_drag (item, event);
-		return true;
-
 	case TempoBarItem:
 	case TempoCurveItem:
 		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)
@@ -853,6 +846,7 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 	case SamplesRulerItem:
 	case MinsecRulerItem:
 	case MarkerBarItem:
+	case SectionMarkerBarItem:
 		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)
 		    && !ArdourKeyboard::indicates_constraint (event->button.state)) {
 			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
@@ -870,29 +864,17 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 		return true;
 		break;
 
-	case CdMarkerBarItem:
-		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)) {
-			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
-		} else {
-			_drags->set (new RangeMarkerBarDrag (this, item, RangeMarkerBarDrag::CreateCDMarker), event);
-		}
+	case VelocityItem:
+		_drags->set (new LollipopDrag (this, item), event);
 		return true;
 		break;
 
-	case CueMarkerBarItem:
-		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)) {
-			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
-		} else {
-			/* no range dragging on this ruler/bar */
-		}
-		return true;
-		break;
-
-	case TransportMarkerBarItem:
-		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)) {
-			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
-		} else {
-			_drags->set (new RangeMarkerBarDrag (this, item, RangeMarkerBarDrag::CreateTransportMarker), event);
+	case VelocityBaseItem:
+		{
+			VelocityGhostRegion* grv = static_cast<VelocityGhostRegion*> (item->get_data ("ghostregionview"));
+			if (grv) {
+				_drags->set (new VelocityLineDrag (this, grv->base_item(), Temporal::BeatTime), event);
+			}
 		}
 		return true;
 		break;
@@ -1031,6 +1013,9 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 
 		case StreamItem:
 			/* in the past, we created a new midi region here, but perhaps that is best left to the Draw mode */
+			/* .. now we allow for rubberband selection (region gain) */
+			_drags->set (new EditorRubberbandSelectDrag (this, item), event);
+			return true;
 			break;
 
 		case AutomationTrackItem:
@@ -1040,11 +1025,9 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 			break;
 
 		case RegionItem:
-			if (dynamic_cast<AutomationRegionView*>(clicked_regionview)) {
-				/* rubberband drag to select automation points */
-				_drags->set (new EditorRubberbandSelectDrag (this, item), event);
-				return true;
-			}
+			/* rubberband drag to select region gain points */
+			_drags->set (new EditorRubberbandSelectDrag (this, item), event);
+			return true;
 			break;
 
 		default:
@@ -1219,6 +1202,12 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 		return true;
 		break;
 
+	case MouseGrid:
+		/* MouseGrid clicks are handled by _canvas_grid_zone */
+		assert (0);
+		abort(); /*NOTREACHED*/
+		break;
+
 	case MouseDraw:
 		switch (item_type) {
 		case GainLineItem:
@@ -1319,7 +1308,14 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 			}
 
 		case AutomationLineItem:
-			_drags->set (new LineDrag (this, item), event);
+			/* fallthrough */
+		case AutomationTrackItem:
+			{
+				AutomationTimeAxisView* atv = static_cast<AutomationTimeAxisView*> (item->get_data ("trackview"));
+				if (atv) {
+					_drags->set (new AutomationDrawDrag (this, nullptr, atv->base_item(), Temporal::AudioTime), event);
+				}
+			}
 			break;
 
 		case NoteItem:
@@ -1340,6 +1336,14 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 				_drags->set (new RegionCreateDrag (this, item, clicked_axisview), event);
 			}
 			return true;
+		case RegionItem: {
+			RegionView* rv;
+			if ((rv = dynamic_cast<RegionView*> (clicked_regionview))) {
+				ArdourCanvas::Rectangle* r = dynamic_cast<ArdourCanvas::Rectangle*> (rv->get_canvas_frame());
+				_drags->set (new AutomationDrawDrag (this, rv->get_canvas_group(), *r, Temporal::AudioTime), event);
+			}
+		}
+			break;
 
 		default:
 			break;
@@ -1363,15 +1367,6 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 		}
 		break;
 
-	case MouseAudition:
-		_drags->set (new ScrubDrag (this, item), event, _cursors->transparent);
-		scrub_reversals = 0;
-		scrub_reverse_distance = 0;
-		last_scrub_x = event->button.x;
-		scrubbing_direction = 0;
-		return true;
-		break;
-
 	default:
 		break;
 	}
@@ -1554,6 +1549,7 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 		/* see if we're finishing a drag */
 
 		if (_drags->active ()) {
+
 			bool const r = _drags->end_grab (event);
 			if (r) {
 				/* grab dragged, so do nothing else */
@@ -1691,10 +1687,8 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 
 			case MarkerBarItem:
 			case RangeMarkerBarItem:
-			case TransportMarkerBarItem:
-			case CdMarkerBarItem:
+			case SectionMarkerBarItem:
 			case TempoBarItem:
-			case MappingBarItem:
 			case TempoCurveItem:
 			case MeterBarItem:
 			case VideoBarItem:
@@ -1703,11 +1697,7 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 			case MinsecRulerItem:
 			case BBTRulerItem:
 			case SelectionMarkerItem:
-				popup_ruler_menu (where, item_type);
-				break;
-
-			case CueMarkerBarItem:
-				snap_to (where, Temporal::RoundNearest, SnapToGrid_Scaled, true);
+				snap_to (where, Temporal::RoundNearest, SnapToGrid_Scaled, false);
 				popup_ruler_menu (where, item_type);
 				break;
 
@@ -1715,6 +1705,10 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 				marker_context_menu (&event->button, item);
 				break;
 
+			case SceneMarkerItem:
+				marker_context_menu (&event->button, item);
+				break;
+
 			case TempoMarkerItem:
 			case MeterMarkerItem:
 			case BBTMarkerItem:
@@ -1735,6 +1729,7 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 				}
 				break;
 
+			case GridZoneItem:
 			default:
 				break;
 			}
@@ -1803,27 +1798,18 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 		case MarkerBarItem:
 			if (!_dragging_playhead) {
 				snap_to_with_modifier (where, event, Temporal::RoundNearest, SnapToGrid_Scaled);
-				mouse_add_new_marker (where);
+				add_location_mark (where);
 			}
 			return true;
 
-		case CdMarkerBarItem:
-			if (!_dragging_playhead) {
-				/* if we get here then a dragged range wasn't done */
+		case SectionMarkerBarItem:
+			if (!_dragging_playhead && Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)) {
 				snap_to_with_modifier (where, event, Temporal::RoundNearest, SnapToGrid_Scaled);
-				mouse_add_new_marker (where, Location::IsCDMarker);
+				add_location_mark (where, Location::Flags (Location::IsMark | Location::IsSection));
 			}
 			return true;
 
-		case CueMarkerBarItem:
-			if (!_dragging_playhead) {
-				/* if we get here then a dragged range wasn't done */
-				snap_to_with_modifier (where, event, Temporal::RoundNearest, SnapToGrid_Scaled);  //TODO: force to-measure?
-				mouse_add_new_marker (where, Location::IsCueMarker);
-			}
-			return true;
-
-		case MappingBarItem:
+		case GridZoneItem:
 			return true;
 
 		case TempoBarItem:
@@ -1857,15 +1843,23 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 			switch (item_type) {
 			case RegionItem:
 			{
-				/* check that we didn't drag before releasing, since
-				   its really annoying to create new control
-				   points when doing this.
-				*/
-				AudioRegionView* arv = dynamic_cast<AudioRegionView*> (clicked_regionview);
-				if (!were_dragging && arv) {
+				/* since we have FreehandLineDrag we can only get here after a drag, when no movement has happend.
+				 * Except when a drag was aborted by pressing Esc.
+				 */
+				if (!were_dragging) {
+					return true;
+				}
+
+				AudioRegionView*      arv = dynamic_cast<AudioRegionView*> (clicked_regionview);
+				AutomationRegionView* atv = dynamic_cast<AutomationRegionView*> (clicked_regionview);
+
+				if (arv) {
 					bool with_guard_points = Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier);
 					arv->add_gain_point_event (item, event, with_guard_points);
+				} else if (atv) {
+					atv->add_automation_event (event);
 				}
+
 				return true;
 				break;
 			}
@@ -1884,20 +1878,11 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 			}
 			break;
 
-		case MouseAudition:
-			if (scrubbing_direction == 0) {
-				/* no drag, just a click */
-				switch (item_type) {
-				case RegionItem:
-					play_selected_region ();
-					break;
-				default:
-					break;
-				}
-			} else if (_session) {
-				/* make sure we stop */
-				_session->request_stop ();
-			}
+		case MouseGrid:
+			/* MouseGrid clicks are handled by _canvas_grid_zone ,
+			 * We can still get here by single-click on a Tempo, Meter or BBT Marker in the ruler.
+			 */
+			return true;
 			break;
 
 		default:
@@ -1987,7 +1972,7 @@ Editor::enter_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemType item_
 	choose_canvas_cursor_on_entry (item_type);
 
 	switch (item_type) {
-	case MappingBarItem:
+	case GridZoneItem:
 		break;
 
 	case ControlPointItem:
@@ -2135,7 +2120,7 @@ Editor::leave_handler (ArdourCanvas::Item* item, GdkEvent*, ItemType item_type)
 	}
 
 	switch (item_type) {
-	case MappingBarItem:
+	case GridZoneItem:
 		break;
 
 	case ControlPointItem:
@@ -2200,89 +2185,32 @@ Editor::leave_handler (ArdourCanvas::Item* item, GdkEvent*, ItemType item_type)
 	break;
 
 	default:
+		_region_peak_cursor->hide ();
 		break;
 	}
 
 	return ret;
 }
 
-void
-Editor::scrub (samplepos_t sample, double current_x)
+GridType
+Editor::determine_mapping_grid_snap(timepos_t t)
 {
-	double delta;
+	timepos_t snapped          = _snap_to_bbt (t, RoundNearest, SnapToGrid_Unscaled, GridTypeBeat);
+	timepos_t snapped_to_bar   = _snap_to_bbt (t, RoundNearest, SnapToGrid_Unscaled, GridTypeBar);
+	const double unsnapped_pos = time_to_pixel_unrounded (t);
+	const double snapped_pos   = time_to_pixel_unrounded (snapped);
 
-	if (scrubbing_direction == 0) {
-		/* first move */
-		_session->request_locate (sample, false, MustStop);
-		_session->request_transport_speed (0.1);
-		scrubbing_direction = 1;
-
-	} else {
-
-		if (last_scrub_x > current_x) {
-
-			/* pointer moved to the left */
-
-			if (scrubbing_direction > 0) {
-
-				/* we reversed direction to go backwards */
-
-				scrub_reversals++;
-				scrub_reverse_distance += (int) (last_scrub_x - current_x);
-
-			} else {
-
-				/* still moving to the left (backwards) */
-
-				scrub_reversals = 0;
-				scrub_reverse_distance = 0;
-
-				delta = 0.01 * (last_scrub_x - current_x);
-				_session->request_transport_speed_nonzero (_session->actual_speed() - delta);
-			}
+	double ruler_line_granularity = UIConfiguration::instance().get_ruler_granularity () * UIConfiguration::instance().get_ui_scale(); // in pixels
 
+	if (std::abs (snapped_pos - unsnapped_pos) < ruler_line_granularity) {
+		if (snapped == snapped_to_bar) {
+			return GridTypeBar;
 		} else {
-			/* pointer moved to the right */
-
-			if (scrubbing_direction < 0) {
-				/* we reversed direction to go forward */
-
-				scrub_reversals++;
-				scrub_reverse_distance += (int) (current_x - last_scrub_x);
-
-			} else {
-				/* still moving to the right */
-
-				scrub_reversals = 0;
-				scrub_reverse_distance = 0;
-
-				delta = 0.01 * (current_x - last_scrub_x);
-				_session->request_transport_speed_nonzero (_session->actual_speed() + delta);
-			}
-		}
-
-		/* if there have been more than 2 opposite motion moves detected, or one that moves
-		   back more than 10 pixels, reverse direction
-		*/
-
-		if (scrub_reversals >= 2 || scrub_reverse_distance > 10) {
-
-			if (scrubbing_direction > 0) {
-				/* was forwards, go backwards */
-				_session->request_transport_speed (-0.1);
-				scrubbing_direction = -1;
-			} else {
-				/* was backwards, go forwards */
-				_session->request_transport_speed (0.1);
-				scrubbing_direction = 1;
-			}
-
-			scrub_reverse_distance = 0;
-			scrub_reversals = 0;
+			return GridTypeBeat;
 		}
+	} else {
+		return GridTypeNone;
 	}
-
-	last_scrub_x = current_x;
 }
 
 bool
@@ -2319,6 +2247,7 @@ Editor::motion_handler (ArdourCanvas::Item* item, GdkEvent* event, bool from_aut
 	update_join_object_range_location (event->motion.y);
 
 	if (_drags->active ()) {
+		_region_peak_cursor->hide ();
 		//drags change the snapped_cursor location, because we are snapping the thing being dragged, not the actual mouse cursor
 		return _drags->motion_handler (event, from_autoscroll);
 	} else {
@@ -2338,46 +2267,25 @@ Editor::motion_handler (ArdourCanvas::Item* item, GdkEvent* event, bool from_aut
 				}
 			}
 
+			/* show snapped cursor */
 			timepos_t t (where);
 			bool move_snapped_cursor = true;
-
-			if (item == mapping_bar) {
-
-				/* Snap to the nearest beat, and figure out how
-				 * many pixels from the pointer cursor that is.
-				 */
-
-				Editor::EnterContext* ctx = get_enter_context (MappingBarItem);
-
-				if (ctx) {
-					timepos_t snapped = _snap_to_bbt (t, RoundNearest, SnapToGrid_Unscaled, GridTypeBeat);
-					timepos_t snapped_to_bar = _snap_to_bbt (t, RoundNearest, SnapToGrid_Unscaled, GridTypeBar);
-					const double unsnapped_pos = time_to_pixel_unrounded (t);
-					const double snapped_pos = time_to_pixel_unrounded (snapped);
-
-					if (std::abs (snapped_pos - unsnapped_pos) < 10 * UIConfiguration::instance().get_ui_scale()) {
-
-						/* Close to a beat, so snap the mapping
-						 * cursor *and* the snapped cursor to
-						 * the beat.
-						 */
-
-						if (snapped == snapped_to_bar) {
-							ctx->cursor_ctx->change (cursors()->time_fx);
-						} else {
-							/* snapped to a beat, not a bar .... we'll implement a TWIST here */
-							ctx->cursor_ctx->change (cursors()->expand_left_right);
-						}
-					} else {
-						ctx->cursor_ctx->change (cursors()->grabber);
-					}
-				}
-			}
-
 			if (move_snapped_cursor) {
 				snap_to_with_modifier (t, event);
 				set_snapped_cursor_position (t);
 			}
+
+			/* if tempo-mapping, set a cursor to indicate whether we are close to a bar line, beat line, or neither */
+			if (mouse_mode == MouseGrid && item == _canvas_grid_zone) {
+				GridType gt = determine_mapping_grid_snap (t);
+				if (gt == GridTypeBar) {
+					set_canvas_cursor (cursors()->time_fx);
+				} else if (gt == GridTypeBeat) {
+					set_canvas_cursor (cursors()->expand_left_right);
+				} else {
+					set_canvas_cursor (cursors()->grabber);
+				}
+			}
 		}
 
 		if (!peaks_visible) {
@@ -2399,7 +2307,7 @@ Editor::can_remove_control_point (ArdourCanvas::Item* item)
 	}
 
 	AutomationLine& line = control_point->line ();
-	if (dynamic_cast<AudioRegionGainLine*> (&line)) {
+	if (dynamic_cast<RegionFxLine*> (&line)) {
 		/* we shouldn't remove the first or last gain point in region gain lines */
 		if (line.is_last_point(*control_point) || line.is_first_point(*control_point)) {
 			return false;
@@ -2436,13 +2344,29 @@ Editor::edit_control_point (ArdourCanvas::Item* item)
 		abort(); /*NOTREACHED*/
 	}
 
-	ControlPointDialog d (p);
+	std::vector<ControlPoint*> cps;
+
+	for (auto const& cp : selection->points) {
+		if (&cp->line() == &p->line ()) {
+			cps.push_back (cp);
+		}
+	}
+
+	assert (cps.size() > 0);
+
+	ControlPointDialog d (p, cps.size() > 1);
 
 	if (d.run () != RESPONSE_ACCEPT) {
 		return;
 	}
 
-	p->line().modify_point_y (*p, d.get_y_fraction ());
+	if (d.all_selected_points ()) {
+		p->line().modify_points_y (cps, d.get_y_fraction ());
+	} else {
+		cps.clear ();
+		cps.push_back (p);
+		p->line().modify_points_y (cps, d.get_y_fraction ());
+	}
 }
 
 void
@@ -2852,6 +2776,11 @@ Editor::update_join_object_range_location (double y)
 
 	if (entered_regionview) {
 
+		if (dynamic_cast<AutomationRegionView*> (entered_regionview)) {
+			_join_object_range_state = JOIN_OBJECT_RANGE_NONE;
+			return;
+		}
+
 		/* TODO: there is currently a bug here(?)
 		 * when we are inside a region fade handle, it acts as though we are in range mode because it is in the top half of the region
 		 * can it be fixed here?
@@ -2938,32 +2867,35 @@ Editor::choose_mapping_drag (ArdourCanvas::Item* item, GdkEvent* event)
 {
 	/* In a departure from convention, this event is not handled by a widget
 	 * 'on' the ruler-bar, like a tempo marker, but is instead handled by the
-	 * mapping-bar widget itself. The intent is for the user to feel that they
+	 * whole canvas. The intent is for the user to feel that they
 	 * are manipulating the 'beat and bar grid' which may or may not have tempo
 	 * markers already assigned at the point under the mouse.
 	 */
 
-	if (item != mapping_bar) {
-		return;
-	}
+	bool ignored;
+	samplepos_t where;
 
-	if (_cursor_stack.empty()) {
+	if (!mouse_sample (where, ignored)) {
 		return;
 	}
 
+	/* if tempo-mapping, set a cursor to indicate whether we are close to a bar line, beat line, or neither */
 	bool ramped = false;
-	if (_cursor_stack.back() == cursors()->time_fx) {
-		/* We are on a BAR line  ... the user can drag the line exactly where it's needed */
-		ramped = false;
-	} else if (_cursor_stack.back() == cursors()->expand_left_right) {
-		/* We are on a BEAT line  ... we will nudge the beat line via ramping, without moving any tempo markers */
-		ramped = true;
-	} else {
-		/* Not close enough to a beat line to start any mapping drag */
-		return;
+	if (mouse_mode == MouseGrid && item ==_canvas_grid_zone) {
+		GridType gt = determine_mapping_grid_snap (timepos_t (where));
+		if (gt == GridTypeBar) {
+			ramped = false;
+		} else if (gt == GridTypeBeat) {
+			ramped = true;
+		} else {
+			return; // neither a bar nor a beat; don't start a drag
+		}
 	}
 
-	Temporal::TempoMap::WritableSharedPtr map = begin_tempo_mapping ();
+	/* The reversible command starts here, must be ended/aborted in drag */
+	begin_reversible_command ("");
+	domain_bounce_info = new Temporal::DomainBounceInfo (Temporal::BeatTime, Temporal::AudioTime, should_stretch_markers());
+	Temporal::TempoMap::WritableSharedPtr map = begin_tempo_mapping (*domain_bounce_info);
 
 	/* Decide between a mid-twist, which we do if the
 	 * pointer is between two tempo markers, and an end-stretch,
@@ -2998,11 +2930,11 @@ Editor::choose_mapping_drag (ArdourCanvas::Item* item, GdkEvent* event)
 	/* Create a new marker, or use the one under the mouse */
 
 	if (tempo.bbt() == bbt) {
-		std::cerr << "we are on the RIGHT side of an EXISTING tempo marker" << bbt << " == " << tempo.bbt() << "\n";
 
 		/* special case 1: we are on the right side of the FIRST marker: do not allow the user to manipulate the very first (session global) tempo */
 		if (!before) {
 			abort_tempo_mapping ();
+			abort_reversible_command ();
 			return;
 		}
 
@@ -3014,8 +2946,6 @@ Editor::choose_mapping_drag (ArdourCanvas::Item* item, GdkEvent* event)
 		}
 
 	} else if ((after && after->bbt() == bbt )) {
-		std::cerr << "we are on the LEFT side of an EXISTING tempo marker" << bbt << " == " << after->bbt() << "\n";
-
 		before = const_cast<TempoPoint*> (&tempo);
 		focus = after;
 
@@ -3024,7 +2954,7 @@ Editor::choose_mapping_drag (ArdourCanvas::Item* item, GdkEvent* event)
 		if (after_after) {
 			after = after_after;
 		} else {
-			at_end = true; 
+			at_end = true;
 		}
 
 	} else if (ramped) {
@@ -3033,8 +2963,6 @@ Editor::choose_mapping_drag (ArdourCanvas::Item* item, GdkEvent* event)
 		focus = &tempo;
 
 	} else {
-		std::cerr << "ADD TEMPO MARKER " << bbt << " != " << tempo.bbt() << "\n";
-
 		/* Add a new tempo marker at the nearest beat point
 		   (essentially the snapped grab point for the drag), so that
 		   it becomes the middle one of three used by the twist tempo
@@ -3046,27 +2974,23 @@ Editor::choose_mapping_drag (ArdourCanvas::Item* item, GdkEvent* event)
 		TempoPoint& added = const_cast<TempoPoint&> (map->set_tempo (copied_no_ramp, bbt));
 		focus = &added;
 		reset_tempo_marks ();
-
-		map->dump (std::cerr);
+		reset_bbt_marks ();
 	}
 
-	/* Reversible commands start here, must be ended/aborted in drag */
+	/* Reversible commands get named here, now that we understand what we're doing */
 
 	if (at_end) {
-		begin_reversible_command (_("tempo mapping: end-stretch"));
-		std::cerr << "END STRETCH\n";
+		_session->current_reversible_command()->set_name (_("tempo mapping: end-stretch"));
 		_drags->set (new MappingEndDrag (this, item, map, tempo, *focus, *before_state), event);
 		return;
 	}
 
 	if (before && focus && after) {
-		std::cerr << "TWIST\n";
-		begin_reversible_command (_("tempo mapping: mid-twist"));
+		_session->current_reversible_command()->set_name (_("tempo mapping: mid-twist"));
 		_drags->set (new MappingTwistDrag (this, item, map, *before, *focus, *after, *before_state, ramped), event);
 	} else if (ramped && focus && after) {
 		/* special case 4: user is manipulating a beat line after the INITIAL tempo marker, so there is no prior marker*/
-		std::cerr << "TWIST ON START\n";
-		begin_reversible_command (_("tempo mapping: mid-twist"));
+		_session->current_reversible_command()->set_name (_("tempo mapping: mid-twist"));
 		before = focus; /* this is unused in MappingTwistDrag, when ramped is true, but let's not pass in garbage */
 		_drags->set (new MappingTwistDrag (this, item, map, *before, *focus, *after, *before_state, ramped), event);
 	} else {
