@@ -176,10 +176,8 @@ Editor::initialize_rulers ()
 	lab_children.push_back (Element(tempo_label, PACK_SHRINK, PACK_START));
 	lab_children.push_back (Element(meter_label, PACK_SHRINK, PACK_START));
 	lab_children.push_back (Element(range_mark_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(transport_mark_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(cd_mark_label, PACK_SHRINK, PACK_START));
 	lab_children.push_back (Element(mark_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(cue_mark_label, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(section_mark_label, PACK_SHRINK, PACK_START));
 	lab_children.push_back (Element(videotl_label, PACK_SHRINK, PACK_START));
 
 	/* 1 event handler to bind them all ... */
@@ -222,47 +220,17 @@ Editor::popup_ruler_menu (timepos_t const & where, ItemType t)
 	ruler_items.clear();
 
 	switch (t) {
-	case MarkerBarItem:
-		ruler_items.push_back (MenuElem (_("New Location Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::mouse_add_new_marker), where, Location::Flags (0), 0)));
-		ruler_items.push_back (MenuElem (_("Clear All Locations"), sigc::mem_fun(*this, &Editor::clear_markers)));
-		ruler_items.push_back (MenuElem (_("Clear All Xruns"), sigc::mem_fun(*this, &Editor::clear_xrun_markers)));
-		ruler_items.push_back (MenuElem (_("Unhide Locations"), sigc::mem_fun(*this, &Editor::unhide_markers)));
-		break;
-
-	case RangeMarkerBarItem:
-		ruler_items.push_back (MenuElem (_("New Range"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_new_range), where)));
-		ruler_items.push_back (MenuElem (_("Clear All Ranges"), sigc::mem_fun(*this, &Editor::clear_ranges)));
-		ruler_items.push_back (MenuElem (_("Unhide Ranges"), sigc::mem_fun(*this, &Editor::unhide_ranges)));
-		break;
-
-	case TransportMarkerBarItem:
-		ruler_items.push_back (MenuElem (_("New Loop Range"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_new_loop), where)));
-		ruler_items.push_back (MenuElem (_("New Punch Range"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_new_punch), where)));
-		break;
-
-	case CdMarkerBarItem:
-		// TODO
-		ruler_items.push_back (MenuElem (_("New CD Track Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::mouse_add_new_marker), where, Location::IsCDMarker, 0)));
-		break;
-
-	case CueMarkerBarItem:
-		ruler_items.push_back (MenuElem (_("Stop All Cues"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_new_marker), where, Location::IsCueMarker, CueRecord::stop_all)));
-		ruler_items.push_back (MenuElem (_("Clear All Cues"), sigc::mem_fun (*this, &Editor::clear_cues)));
-		for (int32_t n = 0; n < TriggerBox::default_triggers_per_box; ++n) {
-			ruler_items.push_back (MenuElem (string_compose (_("Cue %1"), cue_marker_name (n)), sigc::bind (sigc::mem_fun(*this, &Editor::mouse_add_new_marker), where, Location::IsCueMarker, n)));
-		}
-		break;
-
-	case MappingBarItem:
-#warning paul fix mapping bar context menu
-		ruler_items.push_back (MenuElem (_("New BBT Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::mouse_add_new_tempo_event), where)));
-		ruler_items.push_back (MenuElem (_("New Tempo Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::mouse_add_new_tempo_event), where)));
-		ruler_items.push_back (MenuElem (_("Clear")));
-		break;
-
 	case TempoBarItem:
 	case TempoCurveItem:
-		ruler_items.push_back (MenuElem (_("New Tempo"), sigc::bind (sigc::mem_fun(*this, &Editor::mouse_add_new_tempo_event), where)));
+		ruler_items.push_back (MenuElem (_("Add New Tempo"), sigc::bind (sigc::mem_fun(*this, &Editor::mouse_add_new_tempo_event), where)));
+		ruler_items.push_back (SeparatorElem ());
+		ruler_items.push_back (MenuElem (_("Clear All Tempos"), sigc::mem_fun (*this, &Editor::clear_tempo_markers)));
+		ruler_items.push_back (SeparatorElem ());
+		ruler_items.push_back (MenuElem (_("Clear All Earlier Tempos"), sigc::bind (sigc::mem_fun (*this, &Editor::clear_tempo_markers_before), where, true)));
+		ruler_items.push_back (MenuElem (_("Clear All Later Tempos"), sigc::bind (sigc::mem_fun (*this, &Editor::clear_tempo_markers_after), where, true)));
+		ruler_items.push_back (SeparatorElem ());
+		ruler_items.push_back (MenuElem (_("Clear All Earlier Tempos (w/BBT markers)"), sigc::bind (sigc::mem_fun (*this, &Editor::clear_tempo_markers_before), where, false)));
+		ruler_items.push_back (MenuElem (_("Clear All Later Tempos (w/BBT markers)"), sigc::bind (sigc::mem_fun (*this, &Editor::clear_tempo_markers_after), where, false)));
 		break;
 
 	case MeterBarItem:
@@ -313,10 +281,86 @@ Editor::popup_ruler_menu (timepos_t const & where, ItemType t)
 		ruler_items.push_back (MenuElem (_("Add BBT Marker"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_bbt_marker_event), where)));
 		break;
 
-	default:
+	default: {
+		ruler_items.push_back (MenuElem ("Add..."));
+		Gtk::MenuItem& add_menu = ruler_items.back();
+		Gtk::Menu* a_menu = new Gtk::Menu;
+		MenuList& add_items = a_menu->items();
+		add_items.push_back (MenuElem (_("Location Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags (Location::IsMark), 0)));
+		add_items.push_back (MenuElem (_("Arrangement Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags(Location::IsMark | Location::IsSection), 0)));
+		add_items.push_back (MenuElem (_("CD Track Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags(Location::IsMark |Location::IsCDMarker), 0)));
+		add_items.push_back (MenuElem ("Cue Marker..."));
+		Gtk::MenuItem& cue_submenu = add_items.back();
+		Gtk::Menu* cue_menu = new Gtk::Menu;
+		MenuList& cue_items = cue_menu->items();
+		cue_items.push_back (MenuElem (_("Stop All Cues"), sigc::bind (sigc::mem_fun (*this, &Editor::add_location_mark_with_flag), where, Location::Flags(Location::IsMark |Location::IsCueMarker), CueRecord::stop_all)));
+		for (int32_t n = 0; n < TriggerBox::default_triggers_per_box; ++n) {
+			cue_items.push_back (MenuElem (string_compose (_("Cue %1"), cue_marker_name (n)), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags(Location::IsMark |Location::IsCueMarker), n)));
+		}
+		cue_submenu.set_submenu (*cue_menu);
+
+		add_items.push_back (SeparatorElem());
+
+		add_items.push_back (MenuElem (_("Range"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_new_range), where)));
+		add_items.push_back (MenuElem (_("Loop Range"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_new_loop), where)));
+		add_items.push_back (MenuElem (_("Punch Range"), sigc::bind (sigc::mem_fun (*this, &Editor::mouse_add_new_punch), where)));
+		add_menu.set_submenu (*a_menu);
+
+		ruler_items.push_back (MenuElem ("Remove..."));
+		Gtk::MenuItem& clear_menu = ruler_items.back();
+		Gtk::Menu* c_menu = new Gtk::Menu;
+		MenuList& clear_items = c_menu->items();
+		clear_items.push_back (MenuElem (_("All Locations"), sigc::mem_fun(*this, &Editor::clear_markers)));
+		clear_items.push_back (MenuElem (_("All Ranges"), sigc::mem_fun(*this, &Editor::clear_ranges)));
+		clear_items.push_back (MenuElem (_("All Cues"), sigc::mem_fun (*this, &Editor::clear_cues)));
+		clear_items.push_back (MenuElem (_("All Xruns"), sigc::mem_fun(*this, &Editor::clear_xrun_markers)));
+		clear_items.push_back (MenuElem (_("All (MIDI) Scenes"), sigc::mem_fun (*this, &Editor::clear_scenes)));
+		clear_menu.set_submenu (*c_menu);
+
+		}
 		break;
 	}
 
+	ruler_items.push_back (SeparatorElem());
+
+	/* Gtkmm does not expose the ::set_related_action() API for
+	 * Gtk::Activatable, so we have to drop to C to create menu items
+	 * directly from actions.
+	 */
+
+#define MAKE_ACTION_ITEM(menu,item,action_group,action_name) \
+	(item) = new (Gtk::CheckMenuItem); \
+	gtk_activatable_set_related_action (GTK_ACTIVATABLE((item)->gobj()), ActionManager::get_action (action_group, action_name)->gobj()); \
+	(item)->set_draw_as_radio (); \
+	(menu)->append (*(item));
+
+	Gtk::Menu* ms_menu = new (Gtk::Menu);
+	Gtk::CheckMenuItem* ms_item;
+
+	MAKE_ACTION_ITEM (ms_menu, ms_item, X_("Rulers"), X_("show-all-markers"));
+	MAKE_ACTION_ITEM (ms_menu, ms_item, X_("Rulers"), X_("show-cue-markers"));
+	MAKE_ACTION_ITEM (ms_menu, ms_item, X_("Rulers"), X_("show-cd-markers"));
+	MAKE_ACTION_ITEM (ms_menu, ms_item, X_("Rulers"), X_("show-scene-markers"));
+	MAKE_ACTION_ITEM (ms_menu, ms_item, X_("Rulers"), X_("show-location-markers"));
+
+	ruler_items.push_back (MenuElem (_("Show Locations...")));
+	Gtk::MenuItem& marker_show_menu = ruler_items.back();
+	marker_show_menu.set_submenu (*ms_menu);
+
+	Gtk::Menu* rs_menu = new (Gtk::Menu);
+	Gtk::CheckMenuItem* rs_item;
+	MAKE_ACTION_ITEM (rs_menu, rs_item, X_("Rulers"), X_("show-all-ranges"));
+	MAKE_ACTION_ITEM (rs_menu, rs_item, X_("Rulers"), X_("show-punch-range"));
+	MAKE_ACTION_ITEM (rs_menu, rs_item, X_("Rulers"), X_("show-loop-range"));
+	MAKE_ACTION_ITEM (rs_menu, rs_item, X_("Rulers"), X_("show-session-range"));
+	MAKE_ACTION_ITEM (rs_menu, rs_item, X_("Rulers"), X_("show-other-ranges"));
+
+	ruler_items.push_back (MenuElem (_("Show Ranges...")));
+	Gtk::MenuItem& range_show_menu = ruler_items.back();
+	range_show_menu.set_submenu (*rs_menu);
+
+#undef MAKE_ACTION_ITEM
+
 	if (!ruler_items.empty()) {
 		editor_ruler_menu->popup (1, gtk_get_current_event_time());
 	}
@@ -336,10 +380,8 @@ Editor::store_ruler_visibility ()
 	node->set_property (X_("meter"), ruler_meter_action->get_active());
 	node->set_property (X_("tempo"), ruler_tempo_action->get_active());
 	node->set_property (X_("rangemarker"), ruler_range_action->get_active());
-	node->set_property (X_("transportmarker"), ruler_loop_punch_action->get_active());
-	node->set_property (X_("cdmarker"), ruler_cd_marker_action->get_active());
 	node->set_property (X_("marker"), ruler_marker_action->get_active());
-	node->set_property (X_("cuemarker"), ruler_cue_marker_action->get_active());
+	node->set_property (X_("arrangement"), ruler_section_action->get_active());
 	node->set_property (X_("videotl"), ruler_video_action->get_active());
 
 	_session->add_extra_xml (*node);
@@ -378,46 +420,28 @@ Editor::restore_ruler_visibility ()
 		if (node->get_property ("rangemarker", yn)) {
 			ruler_range_action->set_active (yn);
 		}
-		if (node->get_property ("transportmarker", yn)) {
-			ruler_loop_punch_action->set_active (yn);
-		}
-
-		if (node->get_property ("cdmarker", yn)) {
-			ruler_cd_marker_action->set_active (yn);
+		if (node->get_property ("arrangement", yn)) {
+			ruler_section_action->set_active (yn);
 		} else {
-			// this _session doesn't yet know about the cdmarker ruler
-			// as a benefit to the user who doesn't know the feature exists, show the ruler if
-			// any cd marks exist
-			ruler_cd_marker_action->set_active (false);
-			const Locations::LocationList & locs = _session->locations()->list();
-			for (Locations::LocationList::const_iterator i = locs.begin(); i != locs.end(); ++i) {
-				if ((*i)->is_cd_marker()) {
-					ruler_cd_marker_action->set_active (true);
-					break;
-				}
-			}
-		}
-
-		if (node->get_property ("cuemarker", yn)) {
-			ruler_cue_marker_action->set_active (yn);
-		} else {
-			// this _session doesn't yet know about the cue marker ruler
-			// as a benefit to the user who doesn't know the feature exists, show the ruler if
-			// any cue marks exist
-			ruler_cue_marker_action->set_active (false);
-			const Locations::LocationList & locs = _session->locations()->list();
-			for (Locations::LocationList::const_iterator i = locs.begin(); i != locs.end(); ++i) {
-				if ((*i)->is_cue_marker()) {
-					ruler_cue_marker_action->set_active (true);
-					break;
-				}
-			}
+			ruler_section_action->set_active (true);
 		}
 
 		if (node->get_property ("videotl", yn)) {
 			ruler_video_action->set_active (yn);
 		}
 
+	} else {
+		Temporal::TimeDomain const td (_session->config.get_default_time_domain ());
+		/* New session: no rulers have been displayed yet. let's assign default rulers from the session's time domain */
+		ruler_minsec_action->set_active (td == Temporal::AudioTime);
+		ruler_timecode_action->set_active (td == Temporal::AudioTime);
+		ruler_samples_action->set_active (false);
+		ruler_bbt_action->set_active (td == Temporal::BeatTime);
+		ruler_meter_action->set_active (td == Temporal::BeatTime);
+		ruler_tempo_action->set_active (td == Temporal::BeatTime);
+		ruler_range_action->set_active (true);
+		ruler_marker_action->set_active (true);
+		ruler_section_action->set_active (td == Temporal::BeatTime);
 	}
 
 	no_ruler_shown_update = false;
@@ -450,8 +474,7 @@ Editor::update_ruler_visibility ()
 	meter_label.hide();
 	tempo_label.hide();
 	range_mark_label.hide();
-	transport_mark_label.hide();
-	cd_mark_label.hide();
+	section_mark_label.hide();
 	cue_mark_label.hide();
 	mark_label.hide();
 	videotl_label.hide();
@@ -522,17 +545,17 @@ Editor::update_ruler_visibility ()
 	}
 
 	if (ruler_tempo_action->get_active()) {
-		old_unit_pos = tempo_meta_group->position().y;
+		old_unit_pos = tempo_group->position().y;
 		if (tbpos != old_unit_pos) {
-			tempo_meta_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
+			tempo_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
-		tempo_meta_group->show();
+		tempo_group->show();
 		tempo_label.show();
 		tbpos += timebar_height;
 		tbgpos += timebar_height;
 		visible_timebars++;
 	} else {
-		tempo_meta_group->hide();
+		tempo_group->hide();
 		tempo_label.hide();
 	}
 
@@ -569,46 +592,6 @@ Editor::update_ruler_visibility ()
 		range_mark_label.hide();
 	}
 
-	if (ruler_loop_punch_action->get_active()) {
-		old_unit_pos = transport_marker_group->position().y;
-		if (tbpos != old_unit_pos) {
-			transport_marker_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
-		}
-		transport_marker_group->show();
-		transport_mark_label.show();
-
-		transport_marker_bar->set_outline(false);
-
-		tbpos += timebar_height;
-		tbgpos += timebar_height;
-		visible_timebars++;
-	} else {
-		transport_marker_group->hide();
-		transport_mark_label.hide();
-	}
-
-	if (ruler_cd_marker_action->get_active()) {
-		old_unit_pos = cd_marker_group->position().y;
-		if (tbpos != old_unit_pos) {
-			cd_marker_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
-		}
-		cd_marker_group->show();
-		cd_mark_label.show();
-
-		cd_marker_bar->set_outline(false);
-
-		tbpos += timebar_height;
-		tbgpos += timebar_height;
-		visible_timebars++;
-		// make sure all cd markers show up in their respective places
-		update_cd_marker_display();
-	} else {
-		cd_marker_group->hide();
-		cd_mark_label.hide();
-		// make sure all cd markers show up in their respective places
-		update_cd_marker_display();
-	}
-
 	if (ruler_marker_action->get_active()) {
 		old_unit_pos = marker_group->position().y;
 		if (tbpos != old_unit_pos) {
@@ -627,26 +610,24 @@ Editor::update_ruler_visibility ()
 		mark_label.hide();
 	}
 
-	if (ruler_cue_marker_action->get_active()) {
-		old_unit_pos = cue_marker_group->position().y;
+	if (!Profile->get_livetrax() && ruler_section_action->get_active()) {
+		old_unit_pos = section_marker_group->position().y;
 		if (tbpos != old_unit_pos) {
-			cue_marker_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
+			section_marker_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
-		cue_marker_group->show();
-		cue_mark_label.show();
+		section_marker_group->show();
+		section_mark_label.show();
 
-		cue_marker_bar->set_outline(false);
+		section_marker_bar->set_outline(false);
 
 		tbpos += timebar_height;
 		tbgpos += timebar_height;
 		visible_timebars++;
-		// make sure all cd markers show up in their respective places
-		update_cue_marker_display();
+		update_marker_display();
 	} else {
-		cue_marker_group->hide();
-		cue_mark_label.hide();
-		// make sure all cd markers show up in their respective places
-		update_cue_marker_display();
+		section_marker_group->hide();
+		section_mark_label.hide();
+		update_marker_display();
 	}
 
 	if (ruler_video_action->get_active()) {
@@ -672,13 +653,13 @@ Editor::update_ruler_visibility ()
 		_selection_marker_group->hide ();
 	}
 
-	ruler_separator->set_y_position ((int)(timebar_height * visible_timebars));
-
-	time_bars_vbox.set_size_request (-1, (int)(timebar_height * visible_timebars) + 1);
+	int ruler_separator_y = std::max(1, (int)(timebar_height * visible_timebars));
+	ruler_separator->set_y_position (ruler_separator_y);
+	time_bars_vbox.set_size_request (-1, ruler_separator_y);
 
 	/* move hv_scroll_group (trackviews) to the end of the timebars */
 
-	hv_scroll_group->set_y_position ((int)(timebar_height * visible_timebars));
+	hv_scroll_group->set_y_position (ruler_separator_y);
 
 	compute_fixed_ruler_scale ();
 	update_fixed_rulers();
@@ -887,10 +868,12 @@ Editor::set_timecode_ruler_scale (samplepos_t lower, samplepos_t upper)
 		timecode_ruler_scale = timecode_show_many_hours;
 		timecode_mark_modulo = std::max ((samplecnt_t) 1, 1 + (hours_in_range / timecode_nmarks));
 	}
+
+	timecode_mark_modulo = 1;
 }
 
 void
-Editor::metric_get_timecode (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t lower, int64_t /*upper*/, gint /*maxchars*/)
+Editor::metric_get_timecode (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t lower, int64_t upper, gint /*maxchars*/)
 {
 	samplepos_t pos;
 	samplecnt_t spacer;
@@ -909,146 +892,149 @@ Editor::metric_get_timecode (std::vector<ArdourCanvas::Ruler::Mark>& marks, int6
 		lower = 0;
 	}
 
-	pos = (samplecnt_t) floor (lower);
+	pos = lower;
+	// Find timecode time of this sample (pos)
+	_session->sample_to_timecode (pos, timecode, true /* use_offset */, false /* use_subframes */);
 
 	switch (timecode_ruler_scale) {
 	case timecode_show_bits:
 		// Find timecode time of this sample (pos) with subframe accuracy
-		_session->sample_to_timecode(pos, timecode, true /* use_offset */, true /* use_subframes */);
 		for (n = 0; n < timecode_nmarks; n++) {
 			_session->timecode_to_sample(timecode, pos, true /* use_offset */, true /* use_subframes */);
-			if ((timecode.subframes % timecode_mark_modulo) == 0) {
-				if (timecode.subframes == 0) {
-					mark.style = ArdourCanvas::Ruler::Mark::Major;
-					snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
+			if (pos >= lower) {
+				if ((timecode.subframes % timecode_mark_modulo) == 0) {
+					if (timecode.subframes == 0) {
+						mark.style = ArdourCanvas::Ruler::Mark::Major;
+						snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
+					} else {
+						mark.style = ArdourCanvas::Ruler::Mark::Minor;
+						snprintf (buf, sizeof(buf), ".%02u", timecode.subframes);
+					}
 				} else {
-					mark.style = ArdourCanvas::Ruler::Mark::Minor;
-					snprintf (buf, sizeof(buf), ".%02u", timecode.subframes);
+					snprintf (buf, sizeof(buf)," ");
+					mark.style = ArdourCanvas::Ruler::Mark::Micro;
 				}
-			} else {
-				snprintf (buf, sizeof(buf)," ");
-				mark.style = ArdourCanvas::Ruler::Mark::Micro;
+				mark.label = buf;
+				mark.position = pos;
+				marks.push_back (mark);
 			}
-			mark.label = buf;
-			mark.position = pos;
-			marks.push_back (mark);
 			// Increment subframes by one
 			Timecode::increment_subframes (timecode, _session->config.get_subframes_per_frame());
 		}
 		break;
 
 	case timecode_show_samples:
-		// Find timecode time of this sample (pos)
-		_session->sample_to_timecode (pos, timecode, true /* use_offset */, false /* use_subframes */);
 		// Go to next whole sample down
 		Timecode::frames_floot (timecode);
 		for (n = 0; n < timecode_nmarks; n++) {
 			_session->timecode_to_sample (timecode, pos, true /* use_offset */, false /* use_subframes */);
-			if ((timecode.frames % timecode_mark_modulo) == 0) {
-				if (timecode.frames == 0) {
-					mark.style = ArdourCanvas::Ruler::Mark::Major;
+			if (pos >= lower) {
+				if ((timecode.frames % timecode_mark_modulo) == 0) {
+					if (timecode.frames == 0) {
+						mark.style = ArdourCanvas::Ruler::Mark::Major;
+					} else {
+						mark.style = ArdourCanvas::Ruler::Mark::Minor;
+					}
+					mark.position = pos;
+					snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
 				} else {
-					mark.style = ArdourCanvas::Ruler::Mark::Minor;
+					snprintf (buf, sizeof(buf)," ");
+					mark.style = ArdourCanvas::Ruler::Mark::Micro;
+					mark.position = pos;
 				}
-				mark.position = pos;
-				snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
-			} else {
-				snprintf (buf, sizeof(buf)," ");
-				mark.style = ArdourCanvas::Ruler::Mark::Micro;
-				mark.position = pos;
+				mark.label = buf;
+				marks.push_back (mark);
 			}
-			mark.label = buf;
-			marks.push_back (mark);
 			Timecode::increment (timecode, _session->config.get_subframes_per_frame());
 		}
 		break;
 
 	case timecode_show_seconds:
-		// Find timecode time of this sample (pos)
-		_session->sample_to_timecode (pos, timecode, true /* use_offset */, false /* use_subframes */);
 		// Go to next whole second down
 		Timecode::seconds_floor (timecode);
 		for (n = 0; n < timecode_nmarks; n++) {
 			_session->timecode_to_sample (timecode, pos, true /* use_offset */, false /* use_subframes */);
-			if ((timecode.seconds % timecode_mark_modulo) == 0) {
-				if (timecode.seconds == 0) {
-					mark.style = ArdourCanvas::Ruler::Mark::Major;
-					mark.position = pos;
+			if (pos >= lower) {
+				if ((timecode.seconds % timecode_mark_modulo) == 0) {
+					if (timecode.seconds == 0) {
+						mark.style = ArdourCanvas::Ruler::Mark::Major;
+						mark.position = pos;
+					} else {
+						mark.style = ArdourCanvas::Ruler::Mark::Minor;
+						mark.position = pos;
+					}
+					snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
 				} else {
-					mark.style = ArdourCanvas::Ruler::Mark::Minor;
+					snprintf (buf, sizeof(buf)," ");
+					mark.style = ArdourCanvas::Ruler::Mark::Micro;
 					mark.position = pos;
 				}
-				snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
-			} else {
-				snprintf (buf, sizeof(buf)," ");
-				mark.style = ArdourCanvas::Ruler::Mark::Micro;
-				mark.position = pos;
+				mark.label = buf;
+				marks.push_back (mark);
 			}
-			mark.label = buf;
-			marks.push_back (mark);
 			Timecode::increment_seconds (timecode, _session->config.get_subframes_per_frame());
 		}
 		break;
 
 	case timecode_show_minutes:
-		//Find timecode time of this sample (pos)
-		_session->sample_to_timecode (pos, timecode, true /* use_offset */, false /* use_subframes */);
 		// Go to next whole minute down
 		Timecode::minutes_floor (timecode);
 		for (n = 0; n < timecode_nmarks; n++) {
 			_session->timecode_to_sample (timecode, pos, true /* use_offset */, false /* use_subframes */);
-			if ((timecode.minutes % timecode_mark_modulo) == 0) {
-				if (timecode.minutes == 0) {
-					mark.style = ArdourCanvas::Ruler::Mark::Major;
+			if (pos >= lower) {
+				if ((timecode.minutes % timecode_mark_modulo) == 0) {
+					if (timecode.minutes == 0) {
+						mark.style = ArdourCanvas::Ruler::Mark::Major;
+					} else {
+						mark.style = ArdourCanvas::Ruler::Mark::Minor;
+					}
+					snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
 				} else {
-					mark.style = ArdourCanvas::Ruler::Mark::Minor;
+					snprintf (buf, sizeof(buf)," ");
+					mark.style = ArdourCanvas::Ruler::Mark::Micro;
 				}
-				snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
-			} else {
-				snprintf (buf, sizeof(buf)," ");
-				mark.style = ArdourCanvas::Ruler::Mark::Micro;
+				mark.label = buf;
+				mark.position = pos;
+				marks.push_back (mark);
 			}
-			mark.label = buf;
-			mark.position = pos;
-			marks.push_back (mark);
 			Timecode::increment_minutes (timecode, _session->config.get_subframes_per_frame());
 		}
 		break;
 	case timecode_show_hours:
-		// Find timecode time of this sample (pos)
-		_session->sample_to_timecode (pos, timecode, true /* use_offset */, false /* use_subframes */);
 		// Go to next whole hour down
 		Timecode::hours_floor (timecode);
 		for (n = 0; n < timecode_nmarks; n++) {
 			_session->timecode_to_sample (timecode, pos, true /* use_offset */, false /* use_subframes */);
-			if ((timecode.hours % timecode_mark_modulo) == 0) {
-				mark.style = ArdourCanvas::Ruler::Mark::Major;
-				snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
-			} else {
-				snprintf (buf, sizeof(buf)," ");
-				mark.style = ArdourCanvas::Ruler::Mark::Micro;
+			if (pos >= lower) {
+				if ((timecode.hours % timecode_mark_modulo) == 0) {
+					mark.style = ArdourCanvas::Ruler::Mark::Major;
+					snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
+				} else {
+					snprintf (buf, sizeof(buf)," ");
+					mark.style = ArdourCanvas::Ruler::Mark::Micro;
+				}
+				mark.label = buf;
+				mark.position = pos;
+				marks.push_back (mark);
 			}
-			mark.label = buf;
-			mark.position = pos;
-			marks.push_back (mark);
 			Timecode::increment_hours (timecode, _session->config.get_subframes_per_frame());
 		}
 		break;
 	case timecode_show_many_hours:
-		// Find timecode time of this sample (pos)
-		_session->sample_to_timecode (pos, timecode, true /* use_offset */, false /* use_subframes */);
 		// Go to next whole hour down
 		Timecode::hours_floor (timecode);
 
 		for (n = 0; n < timecode_nmarks;) {
 			_session->timecode_to_sample (timecode, pos, true /* use_offset */, false /* use_subframes */);
-			if ((timecode.hours % timecode_mark_modulo) == 0) {
-				mark.style = ArdourCanvas::Ruler::Mark::Major;
-				snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
-				mark.label = buf;
-				mark.position = pos;
-				marks.push_back (mark);
-				++n;
+			if (pos >= lower) {
+				if ((timecode.hours % timecode_mark_modulo) == 0) {
+					mark.style = ArdourCanvas::Ruler::Mark::Major;
+					snprintf (buf, sizeof(buf), "%s%02u:%02u:%02u:%02u", timecode.negative ? "-" : "", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
+					mark.label = buf;
+					mark.position = pos;
+					marks.push_back (mark);
+					++n;
+				}
 			}
 			/* can't use Timecode::increment_hours() here because we may be traversing thousands of hours
 			 * and doing it 1 hour at a time is just stupid (and slow).
@@ -1059,6 +1045,18 @@ Editor::metric_get_timecode (std::vector<ArdourCanvas::Ruler::Mark>& marks, int6
 	}
 }
 
+uint32_t
+Editor::count_bars (Beats const & start, Beats const & end) const
+{
+	TempoMapPoints bar_grid;
+	TempoMap::SharedPtr tmap (TempoMap::use());
+	bar_grid.reserve (4096);
+	superclock_t s (tmap->superclock_at (start));
+	superclock_t e (tmap->superclock_at (end));
+	tmap->get_grid (bar_grid, s, e, 1);
+	return bar_grid.size();
+}
+
 void
 Editor::compute_bbt_ruler_scale (samplepos_t lower, samplepos_t upper)
 {
@@ -1092,7 +1090,7 @@ Editor::compute_bbt_ruler_scale (samplepos_t lower, samplepos_t upper)
 		return;
 	}
 
-	bbt_bars = tmap->count_bars (floor_lower_beat, ceil_upper_beat);
+	bbt_bars = count_bars (floor_lower_beat, ceil_upper_beat);
 
 	double ruler_line_granularity = UIConfiguration::instance().get_ruler_granularity ();  //in pixels
 	ruler_line_granularity = _visible_canvas_width / (ruler_line_granularity*5);  //fudge factor '5' probably related to (4+1 beats)/measure, I think
@@ -1178,12 +1176,13 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 	const samplecnt_t sr (_session->sample_rate());
 
 	Temporal::TempoMapPoints grid;
+	grid.reserve (4096);
 
 	compute_current_bbt_points (grid, lower, upper);
 
 #if 0 // DEBUG GRID
 	for (auto const& g : grid) {
-		std::cout << "Grid " << g.time() <<  " Beats: " << g.beats() << " BBT: " << g.bbt() << " sample: " << g.sample(_session->nominal_sample_rate ()) << "\n"; 
+		std::cout << "Grid " << g.time() <<  " Beats: " << g.beats() << " BBT: " << g.bbt() << " sample: " << g.sample(_session->nominal_sample_rate ()) << "\n";
 	}
 #endif
 
