@@ -39,6 +39,7 @@
 #include "editor.h"
 #include "editor_drag.h"
 #include "editor_routes.h"
+#include "editor_section_box.h"
 #include "editor_sources.h"
 #include "actions.h"
 #include "audio_time_axis.h"
@@ -205,7 +206,7 @@ Editor::select_all_visible_lanes ()
  *  tracks, in which case nothing will happen unless `force' is true.
  */
 void
-Editor::set_selected_track_as_side_effect (Selection::Operation op, Controllable::GroupControlDisposition gcd)
+Editor::set_selected_track_as_side_effect (SelectionOperation op, Controllable::GroupControlDisposition gcd)
 {
 	if (!clicked_axisview) {
 		return;
@@ -213,74 +214,40 @@ Editor::set_selected_track_as_side_effect (Selection::Operation op, Controllable
 
 	PBD::Unwinder<bool> uw (_editor_track_selection_change_without_scroll, true);
 
-	RouteGroup* group = NULL;
-	if (clicked_routeview) {
-		group = clicked_routeview->route()->route_group();
-	}
-
 	switch (op) {
-	case Selection::Toggle:
+	case SelectionToggle:
 		if (selection->selected (clicked_axisview)) {
-			if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
-				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
-					if ((*i)->route_group() == group && gcd != Controllable::NoGroup) {
-						selection->remove(*i);
-					}
-				}
-			} else {
-				selection->remove (clicked_axisview);
-			}
-		} else {
-			if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
-				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
-					if ((*i)->route_group() == group && gcd != Controllable::NoGroup) {
-						selection->add(*i);
-					}
-				}
-			} else {
-				selection->add (clicked_axisview);
-			}
-		}
-		break;
-
-	case Selection::Add:
-		if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
-			for (TrackViewList::iterator i  = track_views.begin(); i != track_views.end (); ++i) {
-				if ((*i)->route_group() == group && gcd != Controllable::NoGroup) {
-					selection->add(*i);
-				}
-			}
+			selection->remove (clicked_axisview);
 		} else {
 			selection->add (clicked_axisview);
 		}
 		break;
 
-	case Selection::Set:
-		selection->clear();
-		if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
-			for (TrackViewList::iterator i  = track_views.begin(); i != track_views.end (); ++i) {
-				if ((*i)->route_group() == group && gcd != Controllable::NoGroup) {
-					selection->add(*i);
-				}
-			}
-		} else {
-			selection->set (clicked_axisview);
-		}
+	case SelectionAdd:
+		selection->add (clicked_axisview);
 		break;
 
-	case Selection::Extend:
+	case SelectionSet:
 		selection->clear();
+		selection->set (clicked_axisview);
+		break;
+
+	case SelectionExtend:
+		selection->clear();
+		break;
+
+	default:
 		break;
 	}
 }
 
 void
-Editor::set_selected_track (TimeAxisView& view, Selection::Operation op, bool no_remove)
+Editor::set_selected_track (TimeAxisView& view, SelectionOperation op, bool no_remove)
 {
-	begin_reversible_selection_op (X_("Set Selected Track"));
 
 	switch (op) {
-	case Selection::Toggle:
+	case SelectionToggle:
+		begin_reversible_selection_op (X_("Toggle track selection"));
 		if (selection->selected (&view)) {
 			if (!no_remove) {
 				selection->remove (&view);
@@ -290,24 +257,29 @@ Editor::set_selected_track (TimeAxisView& view, Selection::Operation op, bool no
 		}
 		break;
 
-	case Selection::Add:
+	case SelectionAdd:
+		begin_reversible_selection_op (X_("Add track selection"));
 		selection->add (&view);
 		break;
 
-	case Selection::Set:
+	case SelectionSet:
+		begin_reversible_selection_op (X_("Set track selection"));
 		selection->set (&view);
 		break;
 
-	case Selection::Extend:
+	case SelectionExtend:
+		begin_reversible_selection_op (X_("Extend track selection"));
 		extend_selection_to_track (view);
 		break;
+	default:
+		return;
 	}
 
 	commit_reversible_selection_op ();
 }
 
 void
-Editor::set_selected_track_from_click (bool press, Selection::Operation op, bool no_remove)
+Editor::set_selected_track_from_click (bool press, SelectionOperation op, bool no_remove)
 {
 	if (!clicked_routeview) {
 		return;
@@ -321,7 +293,7 @@ Editor::set_selected_track_from_click (bool press, Selection::Operation op, bool
 }
 
 bool
-Editor::set_selected_control_point_from_click (bool press, Selection::Operation op)
+Editor::set_selected_control_point_from_click (bool press, SelectionOperation op)
 {
 	if (!clicked_control_point) {
 		return false;
@@ -330,7 +302,7 @@ Editor::set_selected_control_point_from_click (bool press, Selection::Operation
 	bool ret = false;
 
 	switch (op) {
-	case Selection::Set:
+	case SelectionSet:
 		if (!selection->selected (clicked_control_point)) {
 			selection->set (clicked_control_point);
 			ret = true;
@@ -347,13 +319,13 @@ Editor::set_selected_control_point_from_click (bool press, Selection::Operation
 		}
 		break;
 
-	case Selection::Add:
+	case SelectionAdd:
 		if (press) {
 			selection->add (clicked_control_point);
 			ret = true;
 		}
 		break;
-	case Selection::Toggle:
+	case SelectionToggle:
 
 		/* This is a bit of a hack; if we Primary-Click-Drag a control
 		   point (for push drag) we want the point we clicked on to be
@@ -379,9 +351,11 @@ Editor::set_selected_control_point_from_click (bool press, Selection::Operation
 			_control_point_toggled_on_press = false;
 		}
 		break;
-	case Selection::Extend:
+	case SelectionExtend:
 		/* XXX */
 		break;
+	default:
+		break;
 	}
 
 	return ret;
@@ -587,8 +561,13 @@ Editor::mapped_get_equivalent_regions (RouteTimeAxisView& tv, uint32_t, RegionVi
 		return;
 	}
 
-	if (&tv == &basis->get_time_axis_view()) {
-		/* looking in same track as the original */
+	if (basis->region()->is_explicitly_ungrouped ()) {
+		/* this region is explicitly ungrouped; no need to check further */
+		return;
+	}
+
+	if (&tv == &basis->get_time_axis_view () && basis->region()->is_implicitly_ungrouped ()) {
+		/* fallback to region-equivalence: we do not check for equivalent regions in the same track as the basis */
 		return;
 	}
 
@@ -606,8 +585,12 @@ Editor::mapped_get_equivalent_regions (RouteTimeAxisView& tv, uint32_t, RegionVi
 void
 Editor::get_equivalent_regions (RegionView* basis, vector<RegionView*>& equivalent_regions, PBD::PropertyID property) const
 {
-	mapover_tracks_with_unique_playlists (sigc::bind (sigc::mem_fun (*this, &Editor::mapped_get_equivalent_regions), basis, &equivalent_regions), &basis->get_time_axis_view(), property);
-
+	if (basis->region()->is_explicitly_grouped ()) {
+		/* if the user made an explicit region group, it can span tracks outside of the track-group */
+		mapover_all_tracks_with_unique_playlists (sigc::bind (sigc::mem_fun (*this, &Editor::mapped_get_equivalent_regions), basis, &equivalent_regions));
+	} else {
+		mapover_tracks_with_unique_playlists (sigc::bind (sigc::mem_fun (*this, &Editor::mapped_get_equivalent_regions), basis, &equivalent_regions), &basis->get_time_axis_view(), property);
+	}
 	/* add clicked regionview since we skipped all other regions in the same track as the one it was in */
 
 	equivalent_regions.push_back (basis);
@@ -647,7 +630,7 @@ Editor::get_equivalent_regions (RegionSelection & basis, PBD::PropertyID prop) c
 }
 
 bool
-Editor::set_selected_regionview_from_click (bool press, Selection::Operation op)
+Editor::set_selected_regionview_from_click (bool press, SelectionOperation op)
 {
 	vector<RegionView*> all_equivalent_regions;
 	bool commit = false;
@@ -660,10 +643,10 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op)
 		button_release_can_deselect = false;
 	}
 
-	if (op == Selection::Toggle || op == Selection::Set) {
+	if (op == SelectionToggle || op == SelectionSet) {
 
 		switch (op) {
-		case Selection::Toggle:
+		case SelectionToggle:
 			if (selection->selected (clicked_regionview)) {
 				if (press) {
 
@@ -723,7 +706,7 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op)
 			}
 			break;
 
-		case Selection::Set:
+		case SelectionSet:
 			if (!selection->selected (clicked_regionview)) {
 				if (should_ripple_all()) {
 					get_all_equivalent_regions (clicked_regionview, all_equivalent_regions);
@@ -756,7 +739,7 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op)
 			break;
 		}
 
-	} else if (op == Selection::Extend) {
+	} else if (op == SelectionExtend) {
 
 		list<Selectable*> results;
 		timepos_t last_pos;
@@ -1010,32 +993,37 @@ Editor::set_selected_midi_region_view (MidiRegionView& mrv)
 }
 
 void
-Editor::set_selection (std::list<Selectable*> s, Selection::Operation op)
+Editor::set_selection (std::list<Selectable*> s, SelectionOperation op)
 {
 	if (s.empty()) {
 		return;
 	}
-	begin_reversible_selection_op (X_("set selection"));
 	switch (op) {
-		case Selection::Toggle:
-			selection->toggle (s);
-			break;
-		case Selection::Set:
-			selection->set (s);
-			break;
-		case Selection::Extend:
-			selection->add (s);
-			break;
-		case Selection::Add:
-			selection->add (s);
-			break;
+	case SelectionToggle:
+		begin_reversible_selection_op (X_("toggle selection"));
+		selection->toggle (s);
+		break;
+	case SelectionSet:
+		begin_reversible_selection_op (X_("set selection"));
+		selection->set (s);
+		break;
+	case SelectionExtend:
+		begin_reversible_selection_op (X_("extend selection"));
+		selection->add (s);
+		break;
+	case SelectionAdd:
+		begin_reversible_selection_op (X_("add to selection"));
+		selection->add (s);
+		break;
+	default:
+		return;
 	}
 
 	commit_reversible_selection_op () ;
 }
 
 void
-Editor::set_selected_regionview_from_region_list (std::shared_ptr<Region> region, Selection::Operation op)
+Editor::set_selected_regionview_from_region_list (std::shared_ptr<Region> region, SelectionOperation op)
 {
 	vector<RegionView*> regionviews;
 
@@ -1045,22 +1033,27 @@ Editor::set_selected_regionview_from_region_list (std::shared_ptr<Region> region
 		return;
 	}
 
-	begin_reversible_selection_op (X_("set selected regions"));
 
 	switch (op) {
-	case Selection::Toggle:
+	case SelectionToggle:
 		/* XXX this is not correct */
+		begin_reversible_selection_op (X_("toggle selected regions"));
 		selection->toggle (regionviews);
 		break;
-	case Selection::Set:
+	case SelectionSet:
+		begin_reversible_selection_op (X_("toggle selected regions"));
 		selection->set (regionviews);
 		break;
-	case Selection::Extend:
+	case SelectionExtend:
+		begin_reversible_selection_op (X_("extend selected regions"));
 		selection->add (regionviews);
 		break;
-	case Selection::Add:
+	case SelectionAdd:
+		begin_reversible_selection_op (X_("add selected regions"));
 		selection->add (regionviews);
 		break;
+	default:
+		return;
 	}
 
 	commit_reversible_selection_op () ;
@@ -1251,6 +1244,17 @@ Editor::presentation_info_changed (PropertyChange const & what_changed)
 	}
 }
 
+void
+Editor::update_section_box ()
+{
+	if (selection->tracks.size() == 0 && selection->regions.size () == 0 && selection->time.length() != 0) {
+		_section_box->set_position (selection->time.start_time().samples(), selection->time.end_time().samples());
+		_section_box->show ();
+	} else {
+		_section_box->hide();
+	}
+}
+
 void
 Editor::track_selection_changed ()
 {
@@ -1260,6 +1264,9 @@ Editor::track_selection_changed ()
 
 	if ( _session->solo_selection_active() )
 		play_solo_selection(false);
+
+	update_selection_markers ();
+	update_section_box ();
 }
 
 void
@@ -1278,6 +1285,9 @@ Editor::time_selection_changed ()
 		(*i)->hide_selection ();
 	}
 
+	update_selection_markers ();
+	update_section_box ();
+
 	for (TrackSelection::iterator i = selection->tracks.begin(); i != selection->tracks.end(); ++i) {
 		(*i)->show_selection (selection->time);
 	}
@@ -1300,8 +1310,6 @@ Editor::time_selection_changed ()
 			_session->clear_range_selection ();
 		}
 	}
-
-	update_selection_markers ();
 }
 
 /** Set all region actions to have a given sensitivity */
@@ -1414,8 +1422,6 @@ Editor::sensitize_the_right_region_actions (bool because_canvas_crossing)
 	bool have_unlocked = false;
 	bool have_video_locked = false;
 	bool have_video_unlocked = false;
-	bool have_position_lock_style_audio = false;
-	bool have_position_lock_style_music = false;
 	bool have_muted = false;
 	bool have_unmuted = false;
 	bool have_opaque = false;
@@ -1465,12 +1471,6 @@ Editor::sensitize_the_right_region_actions (bool because_canvas_crossing)
 			have_video_unlocked = true;
 		}
 
-		if (r->position_time_domain() == Temporal::BeatTime) {
-			have_position_lock_style_music = true;
-		} else {
-			have_position_lock_style_audio = true;
-		}
-
 		if (r->muted()) {
 			have_muted = true;
 		} else {
@@ -1612,17 +1612,6 @@ Editor::sensitize_the_right_region_actions (bool because_canvas_crossing)
 		// a->set_inconsistent ();
 	}
 
-	a = Glib::RefPtr<ToggleAction>::cast_dynamic (_region_actions->get_action("toggle-region-lock-style"));
-	a->set_active (have_position_lock_style_music && !have_position_lock_style_audio);
-
-	vector<Widget*> proxies = a->get_proxies();
-	for (vector<Widget*>::iterator p = proxies.begin(); p != proxies.end(); ++p) {
-		Gtk::CheckMenuItem* cmi = dynamic_cast<Gtk::CheckMenuItem*> (*p);
-		if (cmi) {
-			cmi->set_inconsistent (have_position_lock_style_music && have_position_lock_style_audio);
-		}
-	}
-
 	a = Glib::RefPtr<ToggleAction>::cast_dynamic (_region_actions->get_action("toggle-region-mute"));
 	a->set_active (have_muted && !have_unmuted);
 	if (have_muted && have_unmuted) {
@@ -1729,13 +1718,10 @@ Editor::region_selection_changed ()
 	//... otherwise the user is confusingly left with selected regions that can't be manipulated.
 	if (!selection->regions.empty() && !internal_editing()) {
 
-		/* if in MouseAudition and there's just 1 region selected
+		/* if in TimeFX mode and there's just 1 region selected
 		 * (i.e. we just clicked on it), leave things as they are
-		 *
-		 * Ditto for TimeFX mode
 		 */
-
-		if (selection->regions.size() > 1 || (mouse_mode != Editing::MouseAudition && mouse_mode != Editing::MouseTimeFX)) {
+		if (selection->regions.size() > 1 || mouse_mode != Editing::MouseTimeFX) {
 			set_mouse_mode (MouseObject, false);
 		}
 	}
@@ -1751,7 +1737,7 @@ Editor::point_selection_changed ()
 }
 
 void
-Editor::select_all_in_track (Selection::Operation op)
+Editor::select_all_in_track (SelectionOperation op)
 {
 	list<Selectable *> touched;
 
@@ -1759,30 +1745,30 @@ Editor::select_all_in_track (Selection::Operation op)
 		return;
 	}
 
-	begin_reversible_selection_op (X_("Select All in Track"));
-
 	clicked_routeview->get_selectables (timepos_t(), timepos_t::max (Temporal::AudioTime), 0, DBL_MAX, touched);
 
 	switch (op) {
-	case Selection::Toggle:
+	case SelectionToggle:
+		begin_reversible_selection_op (X_("Toggle Select All in Track"));
 		selection->add (touched);
 		break;
-	case Selection::Set:
+	case SelectionSet:
+		begin_reversible_selection_op (X_("Select All in Track"));
 		selection->set (touched);
 		break;
-	case Selection::Extend:
-		/* meaningless, because we're selecting everything */
-		break;
-	case Selection::Add:
+	case SelectionAdd:
+		begin_reversible_selection_op (X_("Add Select All in Track"));
 		selection->add (touched);
 		break;
+	default:
+		return;
 	}
 
 	commit_reversible_selection_op ();
 }
 
 bool
-Editor::select_all_internal_edit (Selection::Operation)
+Editor::select_all_internal_edit (SelectionOperation)
 {
 	bool selected = false;
 
@@ -1806,7 +1792,7 @@ Editor::select_all_internal_edit (Selection::Operation)
 }
 
 void
-Editor::select_all_objects (Selection::Operation op)
+Editor::select_all_objects (SelectionOperation op)
 {
 	list<Selectable *> touched;
 
@@ -1829,20 +1815,21 @@ Editor::select_all_objects (Selection::Operation op)
 		(*iter)->get_selectables (timepos_t(), timepos_t::max (Temporal::AudioTime), 0, DBL_MAX, touched);
 	}
 
-	begin_reversible_selection_op (X_("select all"));
 	switch (op) {
-	case Selection::Add:
+	case SelectionAdd:
+		begin_reversible_selection_op (X_("add select all"));
 		selection->add (touched);
 		break;
-	case Selection::Toggle:
+	case SelectionToggle:
+		begin_reversible_selection_op (X_("toggle select all"));
 		selection->toggle (touched);
 		break;
-	case Selection::Set:
+	case SelectionSet:
+		begin_reversible_selection_op (X_("select all"));
 		selection->set (touched);
 		break;
-	case Selection::Extend:
-		/* meaningless, because we're selecting everything */
-		break;
+	default:
+		return;
 	}
 	commit_reversible_selection_op ();
 }
@@ -1916,7 +1903,7 @@ Editor::invert_selection ()
  *  within the region are already selected.
  */
 void
-Editor::select_all_within (timepos_t const & start, timepos_t const & end, double top, double bot, const TrackViewList& tracklist, Selection::Operation op, bool preserve_if_selected)
+Editor::select_all_within (timepos_t const & start, timepos_t const & end, double top, double bot, const TrackViewList& tracklist, SelectionOperation op, bool preserve_if_selected)
 {
 	list<Selectable*> found;
 
@@ -1935,7 +1922,7 @@ Editor::select_all_within (timepos_t const & start, timepos_t const & end, doubl
 		return;
 	}
 
-	if (preserve_if_selected && op != Selection::Toggle) {
+	if (preserve_if_selected && op != SelectionToggle) {
 		list<Selectable*>::iterator i = found.begin();
 		while (i != found.end() && (*i)->selected()) {
 			++i;
@@ -1946,20 +1933,21 @@ Editor::select_all_within (timepos_t const & start, timepos_t const & end, doubl
 		}
 	}
 
-	begin_reversible_selection_op (X_("select all within"));
 	switch (op) {
-	case Selection::Add:
+	case SelectionAdd:
+		begin_reversible_selection_op (X_("add select all within"));
 		selection->add (found);
 		break;
-	case Selection::Toggle:
+	case SelectionToggle:
+		begin_reversible_selection_op (X_("toggle select all within"));
 		selection->toggle (found);
 		break;
-	case Selection::Set:
+	case SelectionSet:
+		begin_reversible_selection_op (X_("select all within"));
 		selection->set (found);
 		break;
-	case Selection::Extend:
-		/* not defined yet */
-		break;
+	default:
+		return;
 	}
 
 	commit_reversible_selection_op ();
@@ -2395,6 +2383,20 @@ Editor::move_selected_tracks (bool up)
 
 	sl.sort (Stripable::Sorter());
 
+	/* Check if the selected tracks are already at the beginning or end of
+	 * the ordering, depending on direction.
+	 */
+
+	for (auto & s : sl) {
+		if (s->is_selected()) {
+			if (up && (s->presentation_info().order() <= 0)) {
+				return;
+			} else if (!up && (s->presentation_info().order() >= sl.size() - 1)) {
+				return;
+			}
+		}
+	}
+
 	std::list<ViewStripable> view_stripables;
 
 	/* build a list that includes time axis view information */
