@@ -35,6 +35,7 @@
 #include "ardour/filesystem_paths.h"
 #include "ardour/plugin_manager.h"
 #include "ardour/route.h"
+#include "ardour/search_paths.h"
 #include "ardour/session.h"
 #include "ardour/system_exec.h"
 
@@ -42,6 +43,7 @@
 
 #include "ardour_http.h"
 #include "ardour_ui.h"
+#include "audio_region_view.h"
 #include "public_editor.h"
 #include "region_selection.h"
 #include "luadialog.h"
@@ -826,6 +828,14 @@ LuaInstance::register_classes (lua_State* L, bool sandbox)
 		.endClass ()
 
 		.deriveClass <RegionView, TimeAxisViewItem> ("RegionView")
+		.addCast<AudioRegionView> ("to_audioregionview")
+		.addFunction ("show_region_editor", &RegionView::show_region_editor)
+		.addFunction ("hide_region_editor", &RegionView::hide_region_editor)
+		.endClass ()
+
+		.deriveClass <AudioRegionView, RegionView> ("RegionView")
+		.addFunction ("set_region_gain_line", &AudioRegionView::set_region_gain_line)
+		.addFunction ("set_region_fx_line", (bool (AudioRegionView::*)(uint32_t, uint32_t))&AudioRegionView::set_region_fx_line)
 		.endClass ()
 
 		.deriveClass <RouteUI, Selectable> ("RouteUI")
@@ -1027,7 +1037,7 @@ LuaInstance::register_classes (lua_State* L, bool sandbox)
 
 		.addRefFunction ("find_location_from_marker", &PublicEditor::find_location_from_marker)
 		.addFunction ("find_marker_from_location_id", &PublicEditor::find_marker_from_location_id)
-		.addFunction ("mouse_add_new_marker", &PublicEditor::mouse_add_new_marker)
+		.addFunction ("mouse_add_new_marker", &PublicEditor::add_location_mark)
 #if 0
 		.addFunction ("get_regions_at", &PublicEditor::get_regions_at)
 		.addFunction ("get_regions_after", &PublicEditor::get_regions_after)
@@ -1072,10 +1082,10 @@ LuaInstance::register_classes (lua_State* L, bool sandbox)
 		.endNamespace ()
 
 		.beginNamespace ("SelectionOp")
-		.addConst ("Toggle", Selection::Operation(Selection::Toggle))
-		.addConst ("Set", Selection::Operation(Selection::Set))
-		.addConst ("Extend", Selection::Operation(Selection::Extend))
-		.addConst ("Add", Selection::Operation(Selection::Add))
+		.addConst ("Toggle", SelectionOperation(SelectionToggle))
+		.addConst ("Set", SelectionOperation(SelectionSet))
+		.addConst ("Extend", SelectionOperation(SelectionExtend))
+		.addConst ("Add", SelectionOperation(SelectionAdd))
 		.endNamespace ()
 
 		.beginNamespace ("TrackHeightMode")
@@ -1198,11 +1208,16 @@ LuaInstance::~LuaInstance ()
 }
 
 static std::string
-lua_read_script (std::string const& fn)
+lua_read_script (std::string fn)
 {
 	if (!UIConfiguration::instance().get_update_action_scripts ()) {
 		return "";
 	}
+
+	if (Glib::path_is_absolute (fn) && !Glib::file_test (fn, Glib::FILE_TEST_EXISTS | Glib::FILE_TEST_IS_REGULAR)) {
+		PBD::find_file (lua_search_path (), Glib::path_get_basename (fn), fn);
+	}
+
 	try {
 		return Glib::file_get_contents (fn);
 	} catch (...) { }
