@@ -82,7 +82,7 @@ ArdourMarker::color (std::string const& color_name)
 
 
 ArdourMarker::ArdourMarker (PublicEditor& ed, ArdourCanvas::Item& parent, std::string const& color_name, string const& annotation,
-                            Type type, timepos_t const & pos, bool handle_events, RegionView* rv)
+                            Type type, timepos_t const & pos, bool handle_events, RegionView* rv, bool use_tooltip)
 
 	: editor (ed)
 	, _parent (&parent)
@@ -92,6 +92,7 @@ ArdourMarker::ArdourMarker (PublicEditor& ed, ArdourCanvas::Item& parent, std::s
 	, _entered (false)
 	, _shown (false)
 	, _line_shown (false)
+	, _use_tooltip (use_tooltip)
 	, _color (color_name)
 	, _points_color (color_name)
 	, _left_label_limit (DBL_MAX)
@@ -234,6 +235,16 @@ ArdourMarker::ArdourMarker (PublicEditor& ed, ArdourCanvas::Item& parent, std::s
 		_label_offset = 8.0 * scale;
 		break;
 
+	case Section:
+		points = new ArdourCanvas::Points (); //unused
+		points->push_back (ArdourCanvas::Duple (    0.0, 0.0));
+		points->push_back (ArdourCanvas::Duple (M6 + .5, MH * .5));
+		points->push_back (ArdourCanvas::Duple (    0.0, MH));
+		points->push_back (ArdourCanvas::Duple (    0.0, 0.0));
+
+		_shift = 0 * scale;
+		_label_offset = 4.0 * scale;
+		break;
 
 	case PunchOut: /* fallthrough */
 	case LoopEnd: /* fallthrough */
@@ -287,7 +298,7 @@ ArdourMarker::ArdourMarker (PublicEditor& ed, ArdourCanvas::Item& parent, std::s
 	}
 
 	_position = pos;
-	unit_position = editor.sample_to_pixel (pos.samples());
+	unit_position = editor.sample_to_pixel (pos.is_zero () ? 0 : pos.samples());
 	unit_position -= _shift;
 
 	group = new ArdourCanvas::Container (&parent, ArdourCanvas::Duple (unit_position, 1));
@@ -357,13 +368,18 @@ ArdourMarker::ArdourMarker (PublicEditor& ed, ArdourCanvas::Item& parent, std::s
 	int width;
 
 	layout->set_font_description (name_font);
-	Gtkmm2ext::get_ink_pixel_size (layout, width, name_height);
+	Gtkmm2ext::get_ink_pixel_size_with_descent (layout, width, name_height, name_descent);
 
 	_name_item = new ArdourCanvas::Text (group);
 	CANVAS_DEBUG_NAME (_name_item, string_compose ("ArdourMarker::_name_item for %1", annotation));
 	_name_item->set_font_description (name_font);
 	_name_item->set_color (RGBA_TO_UINT (0,0,0,255));
-	_name_item->set_position (ArdourCanvas::Duple (_label_offset, (marker_height - 4)*0.5 - (name_height) * .5 ));
+
+	if (_type==Section) {
+		_name_item->set_position (ArdourCanvas::Duple (_label_offset, floor (.5 * (name_height - name_height))));
+	} else {
+		_name_item->set_position (ArdourCanvas::Duple (_label_offset, floor (.5 * (name_height - name_descent - .5))));
+	}
 
 	apply_color ();
 
@@ -402,6 +418,7 @@ void ArdourMarker::reparent(ArdourCanvas::Item & parent)
 {
 	group->reparent (&parent);
 	_parent = &parent;
+	group->redraw ();
 }
 
 void
@@ -502,15 +519,17 @@ ArdourMarker::set_name (const string& new_name, const string& tooltip)
 {
 	_name = new_name;
 
-	_pcue->set_tooltip (new_name);
-	_pmark->set_tooltip (new_name);
-	if (_name_flag) {
-		_name_flag->set_tooltip (new_name);
-	}
-	if (tooltip.empty()) {
-		_name_item->set_tooltip (new_name);
-	} else {
-		_name_item->set_tooltip (tooltip);
+	if (_use_tooltip) {
+		_pcue->set_tooltip (new_name);
+		_pmark->set_tooltip (new_name);
+		if (_name_flag) {
+			_name_flag->set_tooltip (new_name);
+		}
+		if (tooltip.empty()) {
+			_name_item->set_tooltip (new_name);
+		} else {
+			_name_item->set_tooltip (tooltip);
+		}
 	}
 
 	setup_name_display ();
@@ -534,13 +553,22 @@ ArdourMarker::setup_name_display ()
 		limit = _right_label_limit;
 	}
 
-	float scale = UIConfiguration::instance().get_ui_scale();
+	double scale = UIConfiguration::instance().get_ui_scale();
 
-	const float padding =  std::max(2.f, rintf(2.f * scale));
-	const double M3 = std::max(1.f, rintf(3.f * scale));
+	const double padding =  std::max (2., rint (2. * scale));
+	const double M3 = std::max(1., rint (3. * scale));
 
 	/* Work out how wide the name can be */
-	int name_width = min ((double) pixel_width (_name, name_font) + padding, limit);
+	int name_width;
+	int name_height;
+
+	pixel_size (_name, name_font, name_width, name_height);
+	name_width = min ((double) name_width + padding, limit);
+
+	if (_name_flag) {
+		_name_flag->set_y0 (0);
+		_name_flag->set_y1 (marker_height - padding);
+	}
 
 	if (name_width == 0) {
 		_name_item->hide ();
@@ -551,7 +579,7 @@ ArdourMarker::setup_name_display ()
 			_name_item->set_x_position (-name_width);
 		}
 
-		_name_item->clamp_width (name_width);
+		_name_item->clamp_width (_type==Cue ? name_width*2 : name_width);
 
 		if (_type == Cue) {
 			if (_cue_index != CueRecord::stop_all) {
@@ -596,6 +624,11 @@ ArdourMarker::setup_name_display ()
 					_name_flag->set_x0 (M3);
 					_name_flag->set_x1 (_name_item->position().x + name_width + padding + 1*scale);
 					break;
+				case Section:
+					_name_flag->set_x0 (0);
+					_name_flag->set_x1 (_name_item->position().x + name_width + padding);
+					_name_flag->set_y1 (marker_height+0.5);  //full height
+					break;
 				default:
 					_name_flag->set_x0 (0);
 					_name_flag->set_x1 (_name_item->position().x + name_width + padding);
@@ -604,11 +637,6 @@ ArdourMarker::setup_name_display ()
 			}
 		}
 	}
-
-	if (_name_flag) {
-		_name_flag->set_y0 (0);
-		_name_flag->set_y1 (marker_height - 2);
-	}
 }
 
 void
@@ -733,7 +761,7 @@ ArdourMarker::set_right_label_limit (double p)
 
 MetricMarker::MetricMarker (PublicEditor& ed, ArdourCanvas::Item& parent, std::string const& color_name, const string& annotation,
                             Type type, timepos_t const & pos, bool handle_events)
-	: ArdourMarker (ed, parent, color_name, annotation, type, pos, false)
+	: ArdourMarker (ed, parent, color_name, annotation, type, pos, false, nullptr, false)
 {
 }
 
@@ -750,34 +778,18 @@ SelectionMarker::SelectionMarker (PublicEditor& editor, ArdourCanvas::Item& pare
 
 /***********************************************************************/
 
-TempoMarker::TempoMarker (PublicEditor& editor, ArdourCanvas::Item& parent, ArdourCanvas::Item& text_parent, std::string const& color_name, const string& text, Temporal::TempoPoint const & temp, samplepos_t sample, uint32_t curve_color)
+TempoMarker::TempoMarker (PublicEditor& editor, ArdourCanvas::Item& parent, std::string const& color_name, const string& text, Temporal::TempoPoint const & temp, samplepos_t sample, uint32_t curve_color)
 	: MetricMarker (editor, parent, color_name, text, Tempo, temp.time(), false)
 	, _tempo (&temp)
-	, _mapping_text (new ArdourCanvas::Text (&text_parent))
 {
 	group->Event.connect (sigc::bind (sigc::mem_fun (editor, &PublicEditor::canvas_tempo_marker_event), group, this));
 	/* points[1].x gives the width of the marker */
 	_curve = new TempoCurve (editor, *group, curve_color, temp, true, (*points)[1].x);
 	_curve->the_item().lower_to_bottom ();
-
-	_mapping_text->set_color (0xffffff);
-	_mapping_text->set_font_description (ARDOUR_UI_UTILS::get_font_for_style (N_("MarkerText")));
-	_mapping_text->set_position (ArdourCanvas::Duple (unit_position, 0.0));
-	_mapping_text->set_ignore_events (true);
-
-	char buf[64];
-	snprintf (buf, sizeof (buf), "%.2f", _tempo->note_types_per_minute ());
-	_mapping_text->set (buf);
 }
 
 TempoMarker::~TempoMarker ()
 {
-	/* the mapping text is a little unusual in that we do not leave its
-	   parent (the editor's mapping_group) to manage its lifetime. We must
-	   explicitly remove and delete it.
-	*/
-	_mapping_text->parent()->remove (_mapping_text);
-	delete _mapping_text;
 	delete _curve;
 }
 
@@ -785,20 +797,12 @@ void
 TempoMarker::reposition ()
 {
 	MetricMarker::reposition ();
-
-	_mapping_text->set_position (ArdourCanvas::Duple (std::max (3., unit_position), _mapping_text->position().y));
 }
 
 void
 TempoMarker::update ()
 {
 	set_position (_tempo->time());
-
-	_mapping_text->set_position (ArdourCanvas::Duple (std::max (3., unit_position), _mapping_text->position().y));
-
-	char buf[64];
-	snprintf (buf, sizeof (buf), "%.2f", _tempo->note_types_per_minute ());
-	_mapping_text->set (buf);
 }
 
 TempoCurve&
@@ -856,15 +860,13 @@ BBTMarker::BBTMarker (PublicEditor& editor, ArdourCanvas::Item& parent, std::str
 	: MetricMarker (editor, parent, color_name, p.name(), BBTPosition, p.time(), false)
 	, _point (&p)
 {
-	std::stringstream full_tooltip;
-
-	full_tooltip << name();
-	full_tooltip << " (";
-	full_tooltip << p.bbt();
-	full_tooltip << ')';
-
-	set_name (name(), full_tooltip.str());
+	set_name (name(), string());
 	group->Event.connect (sigc::bind (sigc::mem_fun (editor, &PublicEditor::canvas_bbt_marker_event), group, this));
+
+	tempo_marker = editor.find_marker_for_tempo (p);
+	assert (tempo_marker);
+	meter_marker = editor.find_marker_for_meter (p);
+	assert (meter_marker);
 }
 
 BBTMarker::~BBTMarker ()
@@ -875,6 +877,9 @@ void
 BBTMarker::update ()
 {
 	set_position (_point->time());
+
+	tempo_marker->update ();
+	meter_marker->update ();
 }
 
 void
@@ -888,3 +893,11 @@ BBTMarker::point() const
 {
 	return *_point;
 }
+
+void
+BBTMarker::set_position (Temporal::timepos_t const & pos)
+{
+	ArdourMarker::set_position (pos);
+	tempo_marker->set_position (pos);
+	meter_marker->set_position (pos);
+}
