@@ -54,10 +54,10 @@ namespace MIDI {
 };
 
 class SysEx;
-class NoteBase;
 class Note;
 class Hit;
 class MidiTimeAxisView;
+class NoteBase;
 class GhostRegion;
 class AutomationTimeAxisView;
 class AutomationRegionView;
@@ -67,6 +67,7 @@ class EditNoteDialog;
 class PatchChange;
 class ItemCounts;
 class CursorContext;
+class VelocityGhostRegion;
 
 class MidiRegionView : public RegionView
 {
@@ -131,7 +132,6 @@ public:
 	void paste_internal (Temporal::timepos_t const & pos, unsigned paste_count, float times, const MidiCutBuffer&);
 
 	void add_canvas_patch_change (ARDOUR::MidiModel::PatchChangePtr patch);
-	void remove_canvas_patch_change (PatchChange* pc);
 
 	/** Look up the given time and channel in the 'automation' and set keys accordingly.
 	 * @param time the time of the patch change event
@@ -179,7 +179,11 @@ public:
 	void end_write();
 	void extend_active_notes();
 
+	void begin_drag_edit (std::string const & why);
+	void end_drag_edit ();
+
 	void display_model(std::shared_ptr<ARDOUR::MidiModel> model);
+	std::shared_ptr<ARDOUR::MidiModel> model() const { return _model; }
 
 	/* note_diff commands should start here; this initiates an undo record */
 	void start_note_diff_command (std::string name = "midi edit");
@@ -229,6 +233,8 @@ public:
 	 * @return true iff the note is within the (time) extent of the region.
 	 */
 	bool note_in_region_range(const std::shared_ptr<NoteType> note, bool& visible) const;
+	/* Test if a note is withing this region's time range. Return true if so */
+	bool note_in_region_time_range(const std::shared_ptr<NoteType> note) const;
 
 	/** Get the region position in pixels relative to session. */
 	double get_position_pixels();
@@ -286,6 +292,9 @@ public:
 	void goto_next_note (bool add_to_selection);
 	void change_note_lengths (bool, bool, Temporal::Beats beats, bool start, bool end);
 	void change_velocities (bool up, bool fine, bool allow_smush, bool all_together);
+	void set_velocity (NoteBase* primary, int velocity);
+	bool set_velocity_for_notes (std::vector<NoteBase*>& notes, int velocity);
+	bool set_velocities_for_notes (std::vector<NoteBase*>& notes, std::vector<int>& velocities);
 	void transpose (bool up, bool fine, bool allow_smush);
 	void nudge_notes (bool forward, bool fine);
 	void channel_edit ();
@@ -294,7 +303,7 @@ public:
 	void show_list_editor ();
 
 	typedef std::set<NoteBase*> Selection;
-	Selection selection () const {
+	Selection const & selection () const {
 		return _selection;
 	}
 
@@ -323,6 +332,7 @@ public:
 	ARDOUR::InstrumentInfo& instrument_info() const;
 
 	void note_deleted (NoteBase*);
+	void clear_note_selection ();
 
 	void show_verbose_cursor_for_new_note_value(std::shared_ptr<NoteType> current_note, uint8_t new_note) const;
 
@@ -340,7 +350,6 @@ public:
   protected:
 	friend class Editor;
 
-	void clear_note_selection ();
 	void invert_note_selection ();
 	void extend_note_selection ();
 
@@ -502,6 +511,9 @@ public:
 	std::shared_ptr<PatchChange> find_canvas_patch_change (ARDOUR::MidiModel::PatchChangePtr p);
 	std::shared_ptr<SysEx> find_canvas_sys_ex (ARDOUR::MidiModel::SysExPtr s);
 
+	friend class VelocityGhostRegion;
+	void sync_velocity_drag (double factor);
+
 	void update_note (NoteBase*, bool update_ghost_regions = true);
 	void update_sustained (Note *, bool update_ghost_regions = true);
 	void update_hit (Hit *, bool update_ghost_regions = true);
@@ -528,7 +540,7 @@ public:
 	void maybe_select_by_position (GdkEventButton* ev, double x, double y);
 	void get_events (Events& e, Evoral::Sequence<Temporal::Beats>::NoteOperator op, uint8_t val, int chan_mask = 0);
 
-	void display_patch_changes_on_channel (uint8_t, bool);
+	void display_patch_changes_on_channel (uint8_t, bool, PatchChanges&);
 
 	void connect_to_diskstream ();
 	void data_recorded (std::weak_ptr<ARDOUR::MidiSource>);
@@ -575,6 +587,40 @@ public:
 	void update_sysexes ();
 	void view_changed ();
 	void model_changed ();
+
+	void sync_ghost_selection (NoteBase*);
+
+	struct SplitInfo {
+		Temporal::Beats time;
+		Temporal::Beats base_len;
+		int             note;
+		int             channel;
+		int             velocity;
+		int             off_velocity;
+
+		SplitInfo (Temporal::Beats const & t, Temporal::Beats const & l, int n, int c, int v, int ov)
+			: time (t)
+			, base_len (l)
+			, note (n)
+			, channel (c)
+			, velocity (v)
+			, off_velocity (ov) {}
+	};
+	std::vector<SplitInfo> split_info;
+
+	uint32_t split_tuple;
+	bool     note_splitting;
+
+	void start_note_splitting ();
+	void end_note_splitting ();
+
+	void split_notes_grid ();
+	void split_notes_more ();
+	void split_notes_less ();
+	void join_notes ();
+	void join_notes_on_channel (int channel);
+
+	void add_split_notes ();
 };
 
 
