@@ -74,11 +74,6 @@ MidiStreamView::MidiStreamView (MidiTimeAxisView& tv)
 	, _note_lines (0)
 	, _updates_suspended (false)
 {
-
-	/* use a group dedicated to MIDI underlays. Audio underlays are not in this group. */
-	_midi_underlay = new ArdourCanvas::Container (_canvas_group);
-	_midi_underlay->lower_to_bottom();
-
 	/* use a dedicated group for MIDI regions (on top of the grid and lines) */
 	_region_group = new ArdourCanvas::Container (_canvas_group);
 	_region_group->raise_to_top ();
@@ -336,8 +331,11 @@ MidiStreamView::draw_note_lines()
 	}
 
 	double y;
-	double prev_y = .5;
-	uint32_t color;
+	double prev_y = 0.;
+	Gtkmm2ext::Color black = UIConfiguration::instance().color_mod ("piano roll black", "piano roll black");
+	Gtkmm2ext::Color white = UIConfiguration::instance().color_mod ("piano roll white", "piano roll white");
+	Gtkmm2ext::Color outline = UIConfiguration::instance().color ("piano roll black outline");
+	Gtkmm2ext::Color color;
 
 	ArdourCanvas::LineSet::ResetRAII lr (*_note_lines);
 
@@ -352,19 +350,9 @@ MidiStreamView::draw_note_lines()
 
 	for (int i = highest_note() + 1; i >= lowest_note(); --i) {
 
-		y = floor(note_to_y (i)) + .5;
+		y = floor (note_to_y (i));
 
-		/* this is the line actually corresponding to the division
-		 * between notes
-		 */
-
-		if (i <= highest_note()) {
-			_note_lines->add_coord (y, 1.0, UIConfiguration::instance().color ("piano roll black outline"));
-		}
-
-		/* now add a thicker line/bar which covers the entire vertical
-		 * height of this note.
-		 */
+		/* add a thicker line/bar which covers the entire vertical height of this note. */
 
 		switch (i % 12) {
 		case 1:
@@ -372,18 +360,27 @@ MidiStreamView::draw_note_lines()
 		case 6:
 		case 8:
 		case 10:
-			color = UIConfiguration::instance().color_mod ("piano roll black", "piano roll black");
+			color = black;
 			break;
+		case 4:
+		case 11:
+			/* this is the line actually corresponding to the division between B & C and E & F */
+			_note_lines->add_coord (y, 1.0, outline);
+			/* fallthrough */
 		default:
-			color = UIConfiguration::instance().color_mod ("piano roll white", "piano roll white");
+			color = white;
 			break;
 		}
 
 		double h = y - prev_y;
-		double mid = y + (h/2.0);
+		double middle = y + (h/2.0);
 
-		if (mid >= 0 && h > 1.0) {
-			_note_lines->add_coord (mid, h, color);
+		if (!fmod (h, 2.) && !fmod (middle, 1.)) {
+			middle += 0.5;
+		}
+
+		if (middle >= 0 && h > 1.0) {
+			_note_lines->add_coord (middle, h, color);
 		}
 
 		prev_y = y;
@@ -570,6 +567,10 @@ MidiStreamView::color_handler ()
 	} else {
 		canvas_rect->set_fill_color (UIConfiguration::instance().color ("midi bus base"));
 	}
+
+	for (auto & rv : region_views) {
+		rv->color_handler ();
+	}
 }
 
 void
