@@ -174,6 +174,15 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session* sess, std::shared_ptr<Route> rt,
 {
 	init ();
 	set_route (rt);
+
+	if (is_master () && !_route->comment().empty () && _session->config.get_show_master_bus_comment_on_load () && self_destruct) {
+		open_comment_editor ();
+		_comment_window->hide ();
+		_comment_window->set_position (Gtk::WIN_POS_CENTER_ON_PARENT);
+		_comment_window->present ();
+		/* show only once */
+		_session->config.set_show_master_bus_comment_on_load (false);
+	}
 }
 
 void
@@ -195,7 +204,7 @@ MixerStrip::init ()
 	}
 
 	width_button.set_icon (ArdourIcon::StripWidth);
-	hide_button.set_tweaks (ArdourButton::Square);
+	width_button.set_tweaks (ArdourButton::Square);
 	set_tooltip (width_button, t);
 
 	hide_button.set_icon (ArdourIcon::HideEye);
@@ -294,8 +303,6 @@ MixerStrip::init ()
 	trim_control.set_tooltip_prefix (_("Trim: "));
 	trim_control.set_name ("trim knob");
 	trim_control.set_no_show_all (true);
-	trim_control.StartGesture.connect(sigc::mem_fun(*this, &MixerStrip::trim_start_touch));
-	trim_control.StopGesture.connect(sigc::mem_fun(*this, &MixerStrip::trim_end_touch));
 	input_button_box.pack_start (trim_control, false, false);
 
 	global_vpacker.set_no_show_all ();
@@ -342,6 +349,14 @@ MixerStrip::init ()
 	global_vpacker.pack_start (output_button, Gtk::PACK_SHRINK);
 	global_vpacker.pack_start (_comment_button, Gtk::PACK_SHRINK);
 
+	midi_input_enable_button.set_size_request (PX_SCALE(19), PX_SCALE(19));
+	midi_input_enable_button.set_name ("midi input button");
+	midi_input_enable_button.set_elements ((ArdourButton::Element)(ArdourButton::Edge|ArdourButton::Body|ArdourButton::VectorIcon));
+	midi_input_enable_button.set_icon (ArdourIcon::DinMidi);
+	midi_input_enable_button.signal_button_press_event().connect (sigc::mem_fun (*this, &MixerStrip::input_active_button_press), false);
+	midi_input_enable_button.signal_button_release_event().connect (sigc::mem_fun (*this, &MixerStrip::input_active_button_release), false);
+	set_tooltip (midi_input_enable_button, _("Enable/Disable MIDI input"));
+
 #ifndef MIXBUS
 	//add a spacer underneath the master bus;
 	//this fills the area that is taken up by the scrollbar on the tracks;
@@ -358,6 +373,7 @@ MixerStrip::init ()
 		scrollbar_height += 3; // track_display_frame border/shadow
 	}
 	spacer.set_size_request (-1, scrollbar_height);
+	spacer.set_name ("AudioBusStripBase");
 	global_vpacker.pack_end (spacer, false, false);
 #endif
 
@@ -377,6 +393,7 @@ MixerStrip::init ()
 
 	number_label.signal_button_press_event().connect (sigc::mem_fun(*this, &MixerStrip::number_button_button_press), false);
 
+	name_button.set_fallthrough_to_parent (true);
 	name_button.signal_button_press_event().connect (sigc::mem_fun(*this, &MixerStrip::name_button_button_press), false);
 
 	group_button.signal_button_press_event().connect (sigc::mem_fun(*this, &MixerStrip::select_route_group), false);
@@ -501,45 +518,6 @@ MixerStrip::update_trim_control ()
 	}
 }
 
-void
-MixerStrip::trim_start_touch (int)
-{
-	assert (_route && _session);
-
-	if (!route()->trim() || !route()->trim()->active() || route()->n_inputs().n_audio() <= 0) {
-		return;
-	}
-
-	std::shared_ptr<AutomationControl> control (route()->trim()->gain_control());
-
-	StripableList sl;
-	_session->selection ().get_stripables_for_op (sl, _route, &RouteGroup::is_gain);
-
-	_touch_control_group.reset (new GainControlGroup (TrimAutomation));
-	_touch_control_group->set_mode (ControlGroup::Relative);
-	_touch_control_group->fill_from_stripable_list (sl, control->parameter());
-
-	control->start_touch (timepos_t (_session->transport_sample()));
-}
-
-void
-MixerStrip::trim_end_touch (int)
-{
-	assert (_route && _session);
-
-	if (route()->trim() && route()->trim()->active() && route()->n_inputs().n_audio() > 0) {
-
-		std::shared_ptr<AutomationControl> control (route()->trim()->gain_control());
-
-		control->stop_touch (timepos_t (_session->transport_sample()));
-
-		if (_touch_control_group) {
-			_touch_control_group->pop_all ();
-			_touch_control_group.reset ();
-		}
-	}
-}
-
 void
 MixerStrip::set_route (std::shared_ptr<Route> rt)
 {
@@ -617,6 +595,10 @@ MixerStrip::set_route (std::shared_ptr<Route> rt)
 			monitor_section_button->set_can_focus (false);
 			monitor_section_added_or_removed ();
 		}
+	} else if (route()->is_surround_master()) {
+		mute_solo_table.attach (*mute_button, 0, 2, 0, 1);
+		mute_button->show ();
+		master_volume_table.hide ();
 	} else {
 		bottom_button_table.attach (group_button, 1, 2, 0, 1);
 		mute_solo_table.attach (*mute_button, 0, 1, 0, 1);
@@ -669,15 +651,9 @@ MixerStrip::set_route (std::shared_ptr<Route> rt)
 	update_trim_control();
 
 	if (is_midi_track()) {
-
-		midi_input_enable_button.set_size_request (PX_SCALE(19), PX_SCALE(19));
-		midi_input_enable_button.set_name ("midi input button");
-		midi_input_enable_button.set_elements ((ArdourButton::Element)(ArdourButton::Edge|ArdourButton::Body|ArdourButton::VectorIcon));
-		midi_input_enable_button.set_icon (ArdourIcon::DinMidi);
-		midi_input_enable_button.signal_button_press_event().connect (sigc::mem_fun (*this, &MixerStrip::input_active_button_press), false);
-		midi_input_enable_button.signal_button_release_event().connect (sigc::mem_fun (*this, &MixerStrip::input_active_button_release), false);
-		set_tooltip (midi_input_enable_button, _("Enable/Disable MIDI input"));
-		input_button_box.pack_start (midi_input_enable_button, false, false);
+		if (!midi_input_enable_button.get_parent()) {
+			input_button_box.pack_start (midi_input_enable_button, false, false);
+		}
 
 		/* get current state */
 		midi_input_status_changed ();
@@ -712,7 +688,7 @@ MixerStrip::set_route (std::shared_ptr<Route> rt)
 
 		/* non-master bus */
 
-		if (!_route->is_master()) {
+		if (!_route->is_main_bus ()) {
 			if (ARDOUR::Profile->get_mixbus()) {
 				rec_mon_table.attach (*show_sends_button, 0, 3, 0, 2);
 			} else {
@@ -761,7 +737,7 @@ MixerStrip::set_route (std::shared_ptr<Route> rt)
 	connect_to_pan ();
 	panners.setup_pan ();
 
-	if (has_audio_outputs ()) {
+	if (has_audio_outputs () && !_route->is_surround_master ()) {
 		panners.show_all ();
 	} else {
 		panners.hide_all ();
@@ -945,7 +921,7 @@ MixerStrip::update_input_display ()
 {
 	panners.setup_pan ();
 
-	if (has_audio_outputs ()) {
+	if (has_audio_outputs () && !_route->is_surround_master ()) {
 		panners.show_all ();
 	} else {
 		panners.hide_all ();
@@ -959,7 +935,7 @@ MixerStrip::update_output_display ()
 	gpm.setup_meters ();
 	panners.setup_pan ();
 
-	if (has_audio_outputs ()) {
+	if (has_audio_outputs () && !_route->is_surround_master ()) {
 		panners.show_all ();
 	} else {
 		panners.hide_all ();
@@ -1098,7 +1074,7 @@ MixerStrip::build_route_ops_menu ()
 			items.push_back (SeparatorElem());
 		}
 
-		if (!_route->is_master()
+		if (!_route->is_singleton ()
 #ifdef MIXBUS
 				&& !_route->mixbus()
 #endif
@@ -1114,21 +1090,26 @@ MixerStrip::build_route_ops_menu ()
 			/* do not allow rename if the track is record-enabled */
 			items.back().set_sensitive (!is_track() || !track()->rec_enable_control()->get_value());
 		}
-
-		items.push_back (SeparatorElem());
 	}
 
-	if ((!_route->is_master() || !active)
+	if ((!_route->is_singleton () || !active)
 #ifdef MIXBUS
 			&& !_route->mixbus()
 #endif
 	   )
 	{
+		if (active) {
+			items.push_back (SeparatorElem());
+		}
 		items.push_back (CheckMenuElem (_("Active")));
 		Gtk::CheckMenuItem* i = dynamic_cast<Gtk::CheckMenuItem *> (&items.back());
 		i->set_active (active);
 		i->set_sensitive (!_session->transport_rolling());
 		i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::set_route_active), !_route->active(), false));
+	}
+
+	/* Plugin / Processor related */
+	if (active) {
 		items.push_back (SeparatorElem());
 	}
 
@@ -1137,18 +1118,6 @@ MixerStrip::build_route_ops_menu ()
 		Gtk::CheckMenuItem* i = dynamic_cast<Gtk::CheckMenuItem *> (&items.back());
 		i->set_active (_route->strict_io());
 		i->signal_activate().connect (sigc::hide_return (sigc::bind (sigc::mem_fun (*_route, &Route::set_strict_io), !_route->strict_io())));
-		items.push_back (SeparatorElem());
-	}
-
-	if (active && is_track()) {
-		Gtk::Menu* dio_menu = new Menu;
-		MenuList& dio_items = dio_menu->items();
-		dio_items.push_back (MenuElem (_("Record Pre-Fader"), sigc::bind (sigc::mem_fun (*this, &RouteUI::set_disk_io_point), DiskIOPreFader)));
-		dio_items.push_back (MenuElem (_("Record Post-Fader"), sigc::bind (sigc::mem_fun (*this, &RouteUI::set_disk_io_point), DiskIOPostFader)));
-		dio_items.push_back (MenuElem (_("Custom Record+Playback Positions"), sigc::bind (sigc::mem_fun (*this, &RouteUI::set_disk_io_point), DiskIOCustom)));
-
-		items.push_back (MenuElem (_("Disk I/O..."), *dio_menu));
-		items.push_back (SeparatorElem());
 	}
 
 	uint32_t plugin_insert_cnt = 0;
@@ -1157,7 +1126,28 @@ MixerStrip::build_route_ops_menu ()
 		items.push_back (MenuElem (_("Pin Connections..."), sigc::mem_fun (*this, &RouteUI::manage_pins)));
 	}
 
+	if (active) {
+		items.push_back (CheckMenuElem (_("Protect Against Denormals"), sigc::mem_fun (*this, &RouteUI::toggle_denormal_protection)));
+		denormal_menu_item = dynamic_cast<Gtk::CheckMenuItem *> (&items.back());
+		denormal_menu_item->set_active (_route->denormal_protection());
+	}
+
+	/* Disk I/O */
+
+	if (active && is_track()) {
+		items.push_back (SeparatorElem());
+		Gtk::Menu* dio_menu = new Menu;
+		MenuList& dio_items = dio_menu->items();
+		dio_items.push_back (MenuElem (_("Record Pre-Fader"), sigc::bind (sigc::mem_fun (*this, &RouteUI::set_disk_io_point), DiskIOPreFader)));
+		dio_items.push_back (MenuElem (_("Record Post-Fader"), sigc::bind (sigc::mem_fun (*this, &RouteUI::set_disk_io_point), DiskIOPostFader)));
+		dio_items.push_back (MenuElem (_("Custom Record+Playback Positions"), sigc::bind (sigc::mem_fun (*this, &RouteUI::set_disk_io_point), DiskIOCustom)));
+		items.push_back (MenuElem (_("Disk I/O..."), *dio_menu));
+	}
+
+	/* MIDI */
+
 	if (active && (std::dynamic_pointer_cast<MidiTrack>(_route) || _route->the_instrument ())) {
+		items.push_back (SeparatorElem());
 		items.push_back (MenuElem (_("Patch Selector..."),
 					sigc::mem_fun(*this, &RouteUI::select_midi_patch)));
 	}
@@ -1166,13 +1156,8 @@ MixerStrip::build_route_ops_menu ()
 		// TODO ..->n_audio() > 1 && separate_output_groups) hard to check here every time.
 		items.push_back (MenuElem (_("Fan out to Busses"), sigc::bind (sigc::mem_fun (*this, &RouteUI::fan_out), true, true)));
 		items.push_back (MenuElem (_("Fan out to Tracks"), sigc::bind (sigc::mem_fun (*this, &RouteUI::fan_out), false, true)));
-		items.push_back (SeparatorElem());
 	}
 
-	items.push_back (CheckMenuElem (_("Protect Against Denormals"), sigc::mem_fun (*this, &RouteUI::toggle_denormal_protection)));
-	denormal_menu_item = dynamic_cast<Gtk::CheckMenuItem *> (&items.back());
-	denormal_menu_item->set_active (_route->denormal_protection());
-
 	/* note that this relies on selection being shared across editor and
 	 * mixer (or global to the backend, in the future), which is the only
 	 * sane thing for users anyway.
@@ -1191,7 +1176,7 @@ MixerStrip::build_route_ops_menu ()
 		}
 #endif
 
-		if (!_route->is_master()) {
+		if (!_route->is_singleton ()) {
 			items.push_back (SeparatorElem());
 			items.push_back (MenuElem (_("Duplicate..."), sigc::mem_fun (*this, &RouteUI::duplicate_selected_routes)));
 			items.push_back (SeparatorElem());
@@ -1203,7 +1188,21 @@ MixerStrip::build_route_ops_menu ()
 gboolean
 MixerStrip::name_button_button_press (GdkEventButton* ev)
 {
-	if (ev->button == 1 || ev->button == 3) {
+	if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS) {
+		/* fall thru to mixer */
+		return false;
+	}
+
+	if (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS) {
+		route_rename ();
+		return true;
+	}
+
+	if (ev->button == 3 && ARDOUR::Profile->get_livetrax() && _route && _route->is_singleton ()) {
+		return true;
+	}
+
+	if (ev->button == 3) {
 		list_route_operations ();
 
 		if (ev->button == 1) {
@@ -1677,7 +1676,7 @@ MixerStrip::revert_to_default_display ()
 	panner_ui().setup_pan ();
 	panner_ui().set_send_drawing_mode (false);
 
-	if (has_audio_outputs ()) {
+	if (has_audio_outputs () && !_route->is_surround_master ()) {
 		panners.show_all ();
 	} else {
 		panners.hide_all ();
@@ -1895,7 +1894,7 @@ MixerStrip::update_sensitivity ()
 	bool send = _current_delivery && std::dynamic_pointer_cast<Send>(_current_delivery) != 0;
 	bool aux  = _current_delivery && std::dynamic_pointer_cast<InternalSend>(_current_delivery) != 0;
 
-	if (route()->is_master()) {
+	if (route()->is_main_bus ()) {
 		solo_iso_table.set_sensitive (false);
 		control_slave_ui.set_sensitive (false);
 	} else {
@@ -2134,7 +2133,7 @@ MixerStrip::set_marked_for_display (bool yn)
 void
 MixerStrip::hide_master_spacer (bool yn)
 {
-	if (_mixer_owned && route()->is_master() && !yn) {
+	if (_mixer_owned && (route()->is_master() || route()->is_surround_master ()) && !yn) {
 		spacer.show();
 	} else {
 		spacer.hide();
