@@ -46,13 +46,13 @@
 
 #include "actions.h"
 #include "ardour_ui.h"
-#include "audio_clock.h"
 #include "gui_thread.h"
 #include "keyboard.h"
 #include "main_clock.h"
 #include "public_editor.h"
 #include "region_list_base.h"
 #include "ui_config.h"
+#include "utils.h"
 
 #include "pbd/i18n.h"
 
@@ -441,73 +441,9 @@ RegionListBase::clock_format_changed ()
 }
 
 void
-RegionListBase::format_position (timepos_t const& p, char* buf, size_t bufsize, bool onoff)
+RegionListBase::format_position (timepos_t const& p, char* buf, size_t bufsize, bool onoff) const
 {
-	Temporal::BBT_Time bbt;
-	Timecode::Time     timecode;
-	samplepos_t        pos (p.samples ());
-
-	if (pos < 0) {
-		error << string_compose (_("RegionListBase::format_position: negative timecode position: %1"), pos) << endmsg;
-		snprintf (buf, bufsize, "invalid");
-		return;
-	}
-
-	switch (ARDOUR_UI::instance ()->primary_clock->mode ()) {
-		case AudioClock::BBT:
-			bbt = Temporal::TempoMap::use ()->bbt_at (p);
-			if (onoff) {
-				snprintf (buf, bufsize, "%03d|%02d|%04d", bbt.bars, bbt.beats, bbt.ticks);
-			} else {
-				snprintf (buf, bufsize, "(%03d|%02d|%04d)", bbt.bars, bbt.beats, bbt.ticks);
-			}
-			break;
-
-		case AudioClock::MinSec:
-			samplepos_t left;
-			int         hrs;
-			int         mins;
-			float       secs;
-
-			left = pos;
-			hrs  = (int)floor (left / (_session->sample_rate () * 60.0f * 60.0f));
-			left -= (samplecnt_t)floor (hrs * _session->sample_rate () * 60.0f * 60.0f);
-			mins = (int)floor (left / (_session->sample_rate () * 60.0f));
-			left -= (samplecnt_t)floor (mins * _session->sample_rate () * 60.0f);
-			secs = left / (float)_session->sample_rate ();
-			if (onoff) {
-				snprintf (buf, bufsize, "%02d:%02d:%06.3f", hrs, mins, secs);
-			} else {
-				snprintf (buf, bufsize, "(%02d:%02d:%06.3f)", hrs, mins, secs);
-			}
-			break;
-
-		case AudioClock::Seconds:
-			if (onoff) {
-				snprintf (buf, bufsize, "%.1f", pos / (float)_session->sample_rate ());
-			} else {
-				snprintf (buf, bufsize, "(%.1f)", pos / (float)_session->sample_rate ());
-			}
-			break;
-
-		case AudioClock::Samples:
-			if (onoff) {
-				snprintf (buf, bufsize, "%" PRId64, pos);
-			} else {
-				snprintf (buf, bufsize, "(%" PRId64 ")", pos);
-			}
-			break;
-
-		case AudioClock::Timecode:
-		default:
-			_session->timecode_time (pos, timecode);
-			if (onoff) {
-				snprintf (buf, bufsize, "%02d:%02d:%02d:%02d", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
-			} else {
-				snprintf (buf, bufsize, "(%02d:%02d:%02d:%02d)", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
-			}
-			break;
-	}
+	ARDOUR_UI_UTILS::format_position (_session, p, buf, bufsize, onoff);
 }
 
 void
@@ -546,9 +482,6 @@ RegionListBase::populate_row (std::shared_ptr<Region> region, TreeModel::Row con
 	if (all || what_changed.contains (Properties::locked)) {
 		populate_row_locked (region, row);
 	}
-	if (all || what_changed.contains (Properties::time_domain)) {
-		populate_row_glued (region, row);
-	}
 	if (all || what_changed.contains (Properties::muted)) {
 		populate_row_muted (region, row);
 	}
@@ -665,16 +598,6 @@ RegionListBase::populate_row_locked (std::shared_ptr<Region> region, TreeModel::
 	row[_columns.locked] = region->locked ();
 }
 
-void
-RegionListBase::populate_row_glued (std::shared_ptr<Region> region, TreeModel::Row const& row)
-{
-	if (region->position_time_domain () == Temporal::BeatTime) {
-		row[_columns.glued] = true;
-	} else {
-		row[_columns.glued] = false;
-	}
-}
-
 void
 RegionListBase::populate_row_muted (std::shared_ptr<Region> region, TreeModel::Row const& row)
 {
@@ -690,7 +613,7 @@ RegionListBase::populate_row_opaque (std::shared_ptr<Region> region, TreeModel::
 void
 RegionListBase::populate_row_name (std::shared_ptr<Region> region, TreeModel::Row const& row)
 {
-	row[_columns.name] = Gtkmm2ext::markup_escape_text (region->name ());
+	row[_columns.name] = region->name ();
 
 	if (region->data_type () == DataType::MIDI) {
 		row[_columns.channels] = 0; /*TODO: some better recognition of midi regions*/
@@ -706,9 +629,9 @@ RegionListBase::populate_row_source (std::shared_ptr<Region> region, TreeModel::
 {
 	std::shared_ptr<ARDOUR::Source> source = region->source ();
 	if (std::dynamic_pointer_cast<SilentFileSource> (source)) {
-		row[_columns.path] = _("MISSING ") + Gtkmm2ext::markup_escape_text (source->name ());
+		row[_columns.path] = _("MISSING ") + source->name ();
 	} else {
-		row[_columns.path] = Gtkmm2ext::markup_escape_text (source->name ());
+		row[_columns.path] = source->name ();
 
 		std::shared_ptr<FileSource> fs = std::dynamic_pointer_cast<FileSource> (source);
 		if (fs) {
@@ -716,14 +639,14 @@ RegionListBase::populate_row_source (std::shared_ptr<Region> region, TreeModel::
 			if (afs) {
 				const string audio_directory = _session->session_directory ().sound_path ();
 				if (!PBD::path_is_within (audio_directory, fs->path ())) {
-					row[_columns.path] = Gtkmm2ext::markup_escape_text (fs->path ());
+					row[_columns.path] = fs->path ();
 				}
 			}
 			std::shared_ptr<SMFSource> mfs = std::dynamic_pointer_cast<SMFSource> (source);
 			if (mfs) {
 				const string midi_directory = _session->session_directory ().midi_path ();
 				if (!PBD::path_is_within (midi_directory, fs->path ())) {
-					row[_columns.path] = Gtkmm2ext::markup_escape_text (fs->path ());
+					row[_columns.path] = fs->path ();
 				}
 			}
 		}
@@ -926,19 +849,6 @@ RegionListBase::locked_changed (std::string const& path)
 	}
 }
 
-void
-RegionListBase::glued_changed (std::string const& path)
-{
-	TreeIter i = _model->get_iter (path);
-	if (i) {
-		std::shared_ptr<ARDOUR::Region> region = (*i)[_columns.region];
-		if (region) {
-			/* `glued' means MusicTime, and we're toggling here */
-			region->set_position_time_domain ((*i)[_columns.glued] ? Temporal::AudioTime : Temporal::BeatTime);
-		}
-	}
-}
-
 void
 RegionListBase::muted_changed (std::string const& path)
 {
