@@ -579,13 +579,16 @@ RouteListBase::add_stripables (StripableList& slist)
 			if (route->is_monitor ()) {
 				continue;
 			}
+			if (route->is_surround_master ()) {
+				continue;
+			}
 
 			row = *(_model->insert (insert_iter));
 
 			midi_trk = std::dynamic_pointer_cast<MidiTrack> (stripable);
 
 			row[_columns.is_track]    = (std::dynamic_pointer_cast<Track> (stripable) != 0);
-			row[_columns.activatable] = !stripable->is_master ();
+			row[_columns.activatable] = !stripable->is_singleton ();
 
 			if (midi_trk) {
 				row[_columns.is_input_active] = midi_trk->input_active ();
