@@ -50,6 +50,7 @@
 
 #include "ardour/amp.h"
 #include "ardour/meter.h"
+#include "ardour/pan_controllable.h"
 #include "ardour/pannable.h"
 #include "ardour/panner.h"
 #include "ardour/plugin_insert.h"
@@ -57,6 +58,7 @@
 #include "ardour/profile.h"
 #include "ardour/route_group.h"
 #include "ardour/session.h"
+#include "ardour/surround_send.h"
 #include "ardour/track.h"
 
 #include "canvas/debug.h"
@@ -324,6 +326,7 @@ RouteTimeAxisView::set_route (std::shared_ptr<Route> rt)
 	plist->add (ARDOUR::Properties::group_mute, true);
 	plist->add (ARDOUR::Properties::group_solo, true);
 
+	delete route_group_menu;
 	route_group_menu = new RouteGroupMenu (_session, plist);
 
 	gm.get_level_meter().signal_scroll_event().connect (sigc::mem_fun (*this, &RouteTimeAxisView::controls_ebox_scroll), false);
@@ -404,6 +407,7 @@ RouteTimeAxisView::route_group_click (GdkEventButton *ev)
 	WeakRouteList r;
 	r.push_back (route ());
 
+	route_group_menu->detach ();
 	route_group_menu->build (r);
 	if (ev->button == 1) {
 		Gtkmm2ext::anchored_menu_popup(route_group_menu->menu(),
@@ -628,28 +632,31 @@ RouteTimeAxisView::build_display_menu ()
 
 	TimeAxisView::build_display_menu ();
 
-	/* now fill it with our stuff */
+	bool active = _route->active ();
 
 	MenuList& items = display_menu->items();
 
-	items.push_back (MenuElem (_("Color..."), sigc::mem_fun (*this, &RouteUI::choose_color)));
+	/* now fill it with our stuff */
+	if (active) {
+		items.push_back (MenuElem (_("Color..."), sigc::mem_fun (*this, &RouteUI::choose_color)));
 
-	items.push_back (MenuElem (_("Comments..."), sigc::mem_fun (*this, &RouteUI::open_comment_editor)));
+		items.push_back (MenuElem (_("Comments..."), sigc::mem_fun (*this, &RouteUI::open_comment_editor)));
 
-	items.push_back (MenuElem (_("Inputs..."), sigc::mem_fun (*this, &RouteUI::edit_input_configuration)));
+		items.push_back (MenuElem (_("Inputs..."), sigc::mem_fun (*this, &RouteUI::edit_input_configuration)));
 
-	items.push_back (MenuElem (_("Outputs..."), sigc::mem_fun (*this, &RouteUI::edit_output_configuration)));
+		items.push_back (MenuElem (_("Outputs..."), sigc::mem_fun (*this, &RouteUI::edit_output_configuration)));
 
-	items.push_back (SeparatorElem());
+		items.push_back (SeparatorElem());
 
-	build_size_menu ();
-	items.push_back (MenuElem (_("Height"), *_size_menu));
-	items.push_back (SeparatorElem());
+		build_size_menu ();
+		items.push_back (MenuElem (_("Height"), *_size_menu));
+		items.push_back (SeparatorElem());
 
-	// Hook for derived classes to add type specific stuff
-	append_extra_display_menu_items ();
+		/* Hook for derived classes to add type specific stuff */
+		append_extra_display_menu_items ();
+	}
 
-	if (is_track()) {
+	if (active && is_track()) {
 
 		Menu* layers_menu = manage (new Menu);
 		MenuList &layers_items = layers_menu->items();
@@ -793,32 +800,34 @@ RouteTimeAxisView::build_display_menu ()
 		items.push_back (SeparatorElem());
 	}
 
-	route_group_menu->detach ();
 
-	WeakRouteList r;
-	for (TrackSelection::iterator i = _editor.get_selection().tracks.begin(); i != _editor.get_selection().tracks.end(); ++i) {
-		RouteTimeAxisView* rtv = dynamic_cast<RouteTimeAxisView*> (*i);
-		if (rtv) {
-			r.push_back (rtv->route ());
+	if (active) {
+		WeakRouteList r;
+		for (TrackSelection::iterator i = _editor.get_selection().tracks.begin(); i != _editor.get_selection().tracks.end(); ++i) {
+			RouteTimeAxisView* rtv = dynamic_cast<RouteTimeAxisView*> (*i);
+			if (rtv) {
+				r.push_back (rtv->route ());
+			}
 		}
+
+		if (r.empty ()) {
+			r.push_back (route ());
+		}
+
+		if (!_route->is_singleton ()) {
+			route_group_menu->detach ();
+			route_group_menu->build (r);
+			items.push_back (MenuElem (_("Group"), *route_group_menu->menu ()));
+		}
+
+		build_automation_action_menu (true);
+		items.push_back (MenuElem (_("Automation"), *automation_action_menu));
+		items.push_back (SeparatorElem());
 	}
 
-	if (r.empty ()) {
-		r.push_back (route ());
-	}
 
-	if (!_route->is_master()) {
-		route_group_menu->build (r);
-		items.push_back (MenuElem (_("Group"), *route_group_menu->menu ()));
-	}
-
-	build_automation_action_menu (true);
-	items.push_back (MenuElem (_("Automation"), *automation_action_menu));
-
-	items.push_back (SeparatorElem());
-
-	int active = 0;
-	int inactive = 0;
+	int n_active = 0;
+	int n_inactive = 0;
 	bool always_active = false;
 	TrackSelection const & s = _editor.get_selection().tracks;
 	for (TrackSelection::const_iterator i = s.begin(); i != s.end(); ++i) {
@@ -826,37 +835,73 @@ RouteTimeAxisView::build_display_menu ()
 		if (!r) {
 			continue;
 		}
-		always_active |= r->route()->is_master();
+		always_active |= r->route()->is_singleton ();
 #ifdef MIXBUS
 		always_active |= r->route()->mixbus() != 0;
 #endif
 		if (r->route()->active()) {
-			++active;
+			++n_active;
 		} else {
-			++inactive;
+			++n_inactive;
 		}
 	}
 
+	Gtk::CheckMenuItem* i;
+
+#if 0
+	/* Can't have these options until we have audio-timed MIDI and elastic audio */
+	Menu* time_domain_menu = manage (new Menu);
+	MenuList& time_domain_items = time_domain_menu->items();
+	time_domain_menu->set_name ("ArdourContextMenu");
+	time_domain_items.push_back (CheckMenuElem (_("Audio (wallclock) time")));
+	i = dynamic_cast<Gtk::CheckMenuItem *> (&time_domain_items.back());
+	if (_route->has_own_time_domain() && _route->time_domain() == Temporal::AudioTime) {
+		i->set_active (true);
+	} else {
+		i->set_active (false);
+	}
+	i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::set_time_domain), Temporal::AudioTime, true));
+	time_domain_items.push_back (CheckMenuElem (_("Musical (beat) time")));
+	i = dynamic_cast<Gtk::CheckMenuItem *> (&time_domain_items.back());
+	if (_route->has_own_time_domain() && _route->time_domain() == Temporal::BeatTime) {
+		i->set_active (true);
+	} else {
+		i->set_active (false);
+	}
+	i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::set_time_domain), Temporal::BeatTime, true));
+	time_domain_items.push_back (CheckMenuElem (_("Follow Session time domain")));
+	i = dynamic_cast<Gtk::CheckMenuItem *> (&time_domain_items.back());
+	if (!_route->has_own_time_domain()) {
+		i->set_active (true);
+	} else {
+		i->set_active (false);
+	}
+	i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::clear_time_domain), true));
+	items.push_back (MenuElem (_("Time Domain"), *time_domain_menu));
+#endif
+
 	items.push_back (CheckMenuElem (_("Active")));
-	Gtk::CheckMenuItem* i = dynamic_cast<Gtk::CheckMenuItem *> (&items.back());
+	i = dynamic_cast<Gtk::CheckMenuItem *> (&items.back());
 	bool click_sets_active = true;
-	if (active > 0 && inactive == 0) {
+	if (n_active > 0 && n_inactive == 0) {
 		i->set_active (true);
 		click_sets_active = false;
-	} else if (active > 0 && inactive > 0) {
+	} else if (n_active > 0 && n_inactive > 0) {
 		i->set_inconsistent (true);
 	}
 	i->set_sensitive(! _session->transport_rolling() && ! always_active);
-	i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::set_route_active), click_sets_active, true));
+	i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::set_route_active), click_sets_active, !_editor.get_selection().tracks.empty ()));
 
 	items.push_back (SeparatorElem());
-	items.push_back (MenuElem (_("Hide"), sigc::bind (sigc::mem_fun(_editor, &PublicEditor::hide_track_in_display), this, true)));
-	if (_route && !_route->is_master()) {
+	items.push_back (MenuElem (_("Hide"), sigc::bind (sigc::mem_fun(_editor, &PublicEditor::hide_track_in_display), this, !_editor.get_selection().tracks.empty ())));
+
+	if (active && _route && !_route->is_singleton ()) {
 		items.push_back (SeparatorElem());
 		items.push_back (MenuElem (_("Duplicate..."), boost::bind (&ARDOUR_UI::start_duplicate_routes, ARDOUR_UI::instance())));
+
+		items.push_back (SeparatorElem());
+		items.push_back (MenuElem (_("Remove"), sigc::mem_fun(_editor, &PublicEditor::remove_tracks)));
 	}
-	items.push_back (SeparatorElem());
-	items.push_back (MenuElem (_("Remove"), sigc::mem_fun(_editor, &PublicEditor::remove_tracks)));
 }
 
 void
@@ -1104,24 +1149,30 @@ RouteTimeAxisView::selection_click (GdkEventButton* ev)
 		return;
 	}
 
-	_editor.begin_reversible_selection_op (X_("Selection Click"));
-
 	switch (ArdourKeyboard::selection_type (ev->state)) {
-	case Selection::Toggle:
+	case SelectionToggle:
+		_editor.begin_reversible_selection_op (X_("Selection toggle"));
 		_editor.get_selection().toggle (this);
 		break;
 
-	case Selection::Set:
+	case SelectionSet:
+		_editor.begin_reversible_selection_op (X_("Selection set"));
 		_editor.get_selection().set (this);
 		break;
 
-	case Selection::Extend:
+	case SelectionExtend:
+		_editor.begin_reversible_selection_op (X_("Selection extend"));
 		_editor.extend_selection_to_track (*this);
 		break;
 
-	case Selection::Add:
+	case SelectionAdd:
+		_editor.begin_reversible_selection_op (X_("Selection add"));
 		_editor.get_selection().add (this);
 		break;
+
+	default:
+		/* remove not done here */
+		break;
 	}
 
 	_editor.commit_reversible_selection_op ();
@@ -1145,6 +1196,10 @@ RouteTimeAxisView::set_selected_regionviews (RegionSelection& regions)
 	if (_view) {
 		_view->set_selected_regionviews (regions);
 	}
+
+	for (auto & child : children) {
+		child->set_selected_regionviews (regions);
+	}
 }
 
 /** Add the selectable things that we have to a list.
@@ -1255,7 +1310,6 @@ RouteTimeAxisView::find_next_region_boundary (timepos_t const & pos, int32_t dir
 void
 RouteTimeAxisView::fade_range (TimeSelection& selection)
 {
-	std::shared_ptr<Playlist> what_we_got;
 	std::shared_ptr<Track> tr = track ();
 	std::shared_ptr<Playlist> playlist;
 
@@ -1273,11 +1327,7 @@ RouteTimeAxisView::fade_range (TimeSelection& selection)
 
 	playlist->fade_range (time);
 
-	vector<Command*> cmds;
-	playlist->rdiff (cmds);
-	_session->add_commands (cmds);
-	_session->add_command (new StatefulDiffCommand (playlist));
-
+	playlist->rdiff_and_add_command (_session);
 }
 
 void
@@ -1545,6 +1595,25 @@ RouteTimeAxisView::ensure_pan_views (bool show)
 			pan_tracks.push_back (automation_child (pan_control->parameter ()));
 		}
 	}
+
+	/* remove ATAV of no longer relevant pan ctrls (e.g. witdh, height); */
+	bool removed_one;
+	do {
+		removed_one = false;
+		for (auto const& j : children) {
+			std::shared_ptr<AutomationTimeAxisView> atv = std::dynamic_pointer_cast<AutomationTimeAxisView> (j);
+			if (!atv || !std::dynamic_pointer_cast<PanControllable> (atv->control ())) {
+				continue;
+			}
+			if (std::find (pan_tracks.begin (), pan_tracks.end(), atv) != pan_tracks.end ()) {
+				continue;
+			}
+			/* this invalidates the iterator */
+			remove_child (atv);
+			removed_one = true;
+			break;
+		}
+	} while (removed_one);
 }
 
 
@@ -1603,28 +1672,31 @@ RouteTimeAxisView::show_existing_automation (bool apply_to_selection)
 }
 
 void
-RouteTimeAxisView::maybe_hide_automation (bool hide, std::weak_ptr<PBD::Controllable> wctrl)
+RouteTimeAxisView::maybe_hide_automation (bool hide, WeakAutomationControlList wctrls)
 {
 	ctrl_autohide_connection.disconnect ();
 	if (!hide) {
 		/* disconnect only, leave lane visible */
 		return;
 	}
-	std::shared_ptr<AutomationControl> ac = std::dynamic_pointer_cast<AutomationControl> (wctrl.lock ());
-  if (!ac) {
-		return;
-	}
 
-	Gtk::CheckMenuItem* cmi = find_menu_item_by_ctrl (ac);
-	if (cmi) {
-		cmi->set_active (false);
-		return;
-	}
+	for (auto const& wctrl: wctrls) {
+		std::shared_ptr<AutomationControl> ac = std::dynamic_pointer_cast<AutomationControl> (wctrl.lock ());
+		if (!ac) {
+			continue;
+		}
 
-	std::shared_ptr<AutomationTimeAxisView> atav = find_atav_by_ctrl (ac);
-	if (atav) {
-		atav->set_marked_for_display (false);
-		request_redraw ();
+		Gtk::CheckMenuItem* cmi = find_menu_item_by_ctrl (ac);
+		if (cmi) {
+			cmi->set_active (false);
+			continue;
+		}
+
+		std::shared_ptr<AutomationTimeAxisView> atav = find_atav_by_ctrl (ac);
+		if (atav) {
+			atav->set_marked_for_display (false);
+			request_redraw ();
+		}
 	}
 }
 
@@ -1638,7 +1710,7 @@ RouteTimeAxisView::show_touched_automation (std::weak_ptr<PBD::Controllable> wct
 
 	if (!_editor.show_touched_automation ()) {
 		if (ctrl_autohide_connection.connected ()) {
-			signal_ctrl_touched (true);
+			signal_ctrl_touched (true); /* EMIT SIGNAL */
 		}
 		return;
 	}
@@ -1653,22 +1725,46 @@ RouteTimeAxisView::show_touched_automation (std::weak_ptr<PBD::Controllable> wct
 	}
 
 	/* hide any lanes */
-	signal_ctrl_touched (true);
+	signal_ctrl_touched (true); /* EMIT SIGNAL */
+
+	WeakAutomationControlList wctrls;
 
 	if (cmi && !cmi->get_active ()) {
 		cmi->set_active (true);
-		ctrl_autohide_connection = signal_ctrl_touched.connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::maybe_hide_automation), wctrl));
+		wctrls.push_back (ac);
 		/* search ctrl to scroll to */
 		atav = find_atav_by_ctrl (ac, false);
 	} else if (atav && ! string_to<bool>(atav->gui_property ("visible"))) {
 		atav->set_marked_for_display (true);
-		ctrl_autohide_connection = signal_ctrl_touched.connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::maybe_hide_automation), wctrl));
+		wctrls.push_back (ac);
 		request_redraw ();
 	}
 
+	for (auto const& i: ac->visually_linked_controls ()) {
+		std::shared_ptr<AutomationControl> wac = i.lock ();
+		if (!wac) {
+			continue;
+		}
+		cmi = find_menu_item_by_ctrl (wac);
+		if (cmi && !cmi->get_active ()) {
+			cmi->set_active (true);
+			wctrls.push_back (wac);
+			continue;
+		}
+		std::shared_ptr<AutomationTimeAxisView> datav = find_atav_by_ctrl (wac, false);
+		if (datav && ! string_to<bool>(datav->gui_property ("visible"))) {
+			datav->set_marked_for_display (true);
+			wctrls.push_back (wac);
+			request_redraw ();
+		}
+	}
+
+	if (!wctrls.empty ()) {
+		ctrl_autohide_connection = signal_ctrl_touched.connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::maybe_hide_automation), wctrls));
+	}
+
 	if (atav) {
 		_editor.ensure_time_axis_view_is_visible (*atav, false);
-		return;
 	}
 }
 
@@ -1704,10 +1800,6 @@ RouteTimeAxisView::region_view_added (RegionView* rv)
 			atv->add_ghost(rv);
 		}
 	}
-
-	for (UnderlayMirrorList::iterator i = _underlay_mirrors.begin(); i != _underlay_mirrors.end(); ++i) {
-		(*i)->add_ghost(rv);
-	}
 }
 
 RouteTimeAxisView::ProcessorAutomationInfo::~ProcessorAutomationInfo ()
@@ -1774,7 +1866,6 @@ RouteTimeAxisView::find_atav_by_ctrl (std::shared_ptr<ARDOUR::AutomationControl>
 	}
 
 	if (!pan_tracks.empty() && !ARDOUR::Profile->get_mixbus()) {
-		// XXX this can lead to inconsistent CheckMenuItem state (azimuth, width are treated separately)
 		for (list<std::shared_ptr<AutomationTimeAxisView> >::iterator i = pan_tracks.begin(); i != pan_tracks.end(); ++i) {
 			if ((*i)->control () == ac) {
 				return *i;
@@ -1855,6 +1946,9 @@ RouteTimeAxisView::add_existing_processor_automation_curves (std::weak_ptr<Proce
 		/* The Amp processor is a special case and is dealt with separately */
 		return;
 	}
+	if (!processor->display_to_user()) {
+		return;
+	}
 
 	set<Evoral::Parameter> existing;
 	processor->what_has_data (existing);
@@ -1897,16 +1991,19 @@ RouteTimeAxisView::add_processor_to_subplugin_menu (std::weak_ptr<Processor> p)
 {
 	std::shared_ptr<Processor> processor (p.lock ());
 
-	if (!processor || !processor->display_to_user ()) {
+	if (!processor) {
 		return;
 	}
 
-	/* we use this override to veto the Amp processor from the plugin menu,
-	   as its automation lane can be accessed using the special "Fader" menu
-	   option
-	*/
-
-	if (std::dynamic_pointer_cast<Amp> (processor) != 0) {
+	if (std::dynamic_pointer_cast<SurroundSend> (processor) != 0) {
+		/* OK, show surround send controls */
+	} else if (std::dynamic_pointer_cast<Amp> (processor) != 0) {
+			/* we use this override to veto the Amp processor from the plugin menu,
+			 * as its automation lane can be accessed using the special "Fader" menu
+			 * option
+			 */
+		return;
+	} else if (!processor->display_to_user ()) {
 		return;
 	}
 
@@ -2297,122 +2394,6 @@ RouteTimeAxisView::chan_count_changed ()
 	}
 }
 
-void
-RouteTimeAxisView::build_underlay_menu(Gtk::Menu* parent_menu)
-{
-	using namespace Menu_Helpers;
-
-	if (!_underlay_streams.empty()) {
-		MenuList& parent_items = parent_menu->items();
-		Menu* gs_menu = manage (new Menu);
-		gs_menu->set_name ("ArdourContextMenu");
-		MenuList& gs_items = gs_menu->items();
-
-		parent_items.push_back (MenuElem (_("Underlays"), *gs_menu));
-
-		for(UnderlayList::iterator it = _underlay_streams.begin(); it != _underlay_streams.end(); ++it) {
-			gs_items.push_back(MenuElem(string_compose(_("Remove \"%1\""), (*it)->trackview().name()),
-						    sigc::bind(sigc::mem_fun(*this, &RouteTimeAxisView::remove_underlay), *it)));
-		}
-	}
-}
-
-bool
-RouteTimeAxisView::set_underlay_state()
-{
-	if (!underlay_xml_node) {
-		return false;
-	}
-
-	XMLNodeList nlist = underlay_xml_node->children();
-	XMLNodeConstIterator niter;
-	XMLNode *child_node;
-
-	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
-		child_node = *niter;
-
-		if (child_node->name() != "Underlay") {
-			continue;
-		}
-
-		XMLProperty const * prop = child_node->property ("id");
-		if (prop) {
-			PBD::ID id (prop->value());
-
-			StripableTimeAxisView* v = _editor.get_stripable_time_axis_by_id (id);
-
-			if (v) {
-				add_underlay(v->view(), false);
-			}
-		}
-	}
-
-	return false;
-}
-
-void
-RouteTimeAxisView::add_underlay (StreamView* v, bool /*update_xml*/)
-{
-	if (!v) {
-		return;
-	}
-
-	RouteTimeAxisView& other = v->trackview();
-
-	if (find(_underlay_streams.begin(), _underlay_streams.end(), v) == _underlay_streams.end()) {
-		if (find(other._underlay_mirrors.begin(), other._underlay_mirrors.end(), this) != other._underlay_mirrors.end()) {
-			fatal << _("programming error: underlay reference pointer pairs are inconsistent!") << endmsg;
-			abort(); /*NOTREACHED*/
-		}
-
-		_underlay_streams.push_back(v);
-		other._underlay_mirrors.push_back(this);
-
-		v->foreach_regionview(sigc::mem_fun(*this, &RouteTimeAxisView::add_ghost));
-
-#ifdef GUI_OBJECT_STATE_FIX_REQUIRED
-		if (update_xml) {
-			if (!underlay_xml_node) {
-				underlay_xml_node = xml_node->add_child("Underlays");
-			}
-
-			XMLNode* node = underlay_xml_node->add_child("Underlay");
-			XMLProperty const * prop = node->add_property("id");
-			prop->set_value(v->trackview().route()->id().to_s());
-		}
-#endif
-	}
-}
-
-void
-RouteTimeAxisView::remove_underlay (StreamView* v)
-{
-	if (!v) {
-		return;
-	}
-
-	UnderlayList::iterator it = find(_underlay_streams.begin(), _underlay_streams.end(), v);
-	RouteTimeAxisView& other = v->trackview();
-
-	if (it != _underlay_streams.end()) {
-		UnderlayMirrorList::iterator gm = find(other._underlay_mirrors.begin(), other._underlay_mirrors.end(), this);
-
-		if (gm == other._underlay_mirrors.end()) {
-			fatal << _("programming error: underlay reference pointer pairs are inconsistent!") << endmsg;
-			abort(); /*NOTREACHED*/
-		}
-
-		v->foreach_regionview(sigc::mem_fun(*this, &RouteTimeAxisView::remove_ghost));
-
-		_underlay_streams.erase(it);
-		other._underlay_mirrors.erase(gm);
-
-		if (underlay_xml_node) {
-			underlay_xml_node->remove_nodes_and_delete("id", v->trackview().route()->id().to_s());
-		}
-	}
-}
-
 void
 RouteTimeAxisView::set_button_names ()
 {
