@@ -189,8 +189,7 @@ SessionDialog::meta_master_bus_profile (std::string script_path)
 		return UINT32_MAX;
 	}
 
-	LuaState lua;
-	lua.sandbox (true);
+	LuaState lua (true, true);
 	lua_State* L = lua.getState();
 
 	lua.do_command (
@@ -343,6 +342,12 @@ SessionDialog::session_folder ()
 	}
 }
 
+Temporal::TimeDomain
+SessionDialog::session_domain () const
+{
+	return timebase_chooser.get_active_row_number() == 1 ? Temporal::BeatTime : Temporal::AudioTime;
+}
+
 void
 SessionDialog::setup_recent_sessions ()
 {
@@ -463,6 +468,18 @@ SessionDialog::setup_initial_choice_box ()
 	archive_filter.set_name (_("Session Archives"));
 	existing_session_chooser.add_filter (archive_filter);
 
+	FileFilter aaf_filter;
+	aaf_filter.add_pattern (string_compose(X_("*%1"), ARDOUR::advanced_authoring_format_suffix));
+	aaf_filter.set_name (_("Advanced Authoring Format (AAF)"));
+	existing_session_chooser.add_filter (aaf_filter);
+
+	FileFilter all_filter;
+	all_filter.add_pattern (string_compose(X_("*%1"), ARDOUR::statefile_suffix));
+	all_filter.add_pattern (string_compose(X_("*%1"), ARDOUR::session_archive_suffix));
+	all_filter.add_pattern (string_compose(X_("*%1"), ARDOUR::advanced_authoring_format_suffix));
+	all_filter.set_name (_("All supported files"));
+	existing_session_chooser.add_filter (all_filter);
+
 	existing_session_chooser.set_filter (session_filter);
 
 	Gtkmm2ext::add_volume_shortcuts (existing_session_chooser);
@@ -660,6 +677,26 @@ SessionDialog::setup_new_session_page ()
 	new_folder_chooser.set_title (_("Select folder for session"));
 	Gtkmm2ext::add_volume_shortcuts (new_folder_chooser);
 
+	//Timebase for the new session
+	Label* session_domain_label = manage (new Label);
+	session_domain_label->set_text (_("Default Time Domain:"));
+	HBox* timebase_box = manage (new HBox);
+	timebase_box->set_spacing (8);
+	timebase_box->pack_start (*session_domain_label, false, false);
+	timebase_box->pack_start (timebase_chooser, true, true);
+
+	timebase_chooser.append (_("Audio Time"));
+	timebase_chooser.append (_("Beat Time"));
+	timebase_chooser.set_active (Config->get_preferred_time_domain() == Temporal::BeatTime ? 1 : 0);
+
+	set_tooltip (timebase_chooser, _(
+	"The time domain controls how some items on the timeline respond to tempo map editing.\n\n"
+	"If you choose Beat Time, some items (like markers) will move when you change tempo.\n\n"
+	"If you choose Audio Time, these items will not move when you change tempo.\n\n"
+	"The time domain also affects which ruler lanes will be initially visible.\n\n"
+	"You can change the session's timebase anytime in Session -> Properties."
+		));
+
 	//Template & Template Description area
 	HBox* template_hbox = manage (new HBox);
 
@@ -705,6 +742,7 @@ SessionDialog::setup_new_session_page ()
 	}
 
 	session_new_vbox.pack_start (*template_hbox, true, true);
+	session_new_vbox.pack_start (*timebase_box, false, true);
 	session_new_vbox.pack_start (*folder_box, false, true);
 	session_new_vbox.pack_start (*name_hbox, false, true);
 	session_new_vbox.show_all ();
