@@ -411,22 +411,50 @@ TimeAxisView::controls_ebox_scroll (GdkEventScroll* ev)
 bool
 TimeAxisView::controls_ebox_button_press (GdkEventButton* event)
 {
+	bool inside_name_label = false;
+
+	if (name_label.is_ancestor (controls_ebox)) {
+		int nlx;
+		int nly;
+		controls_ebox.translate_coordinates (name_label, event->x, event->y, nlx, nly);
+		Gtk::Allocation a = name_label.get_allocation ();
+
+		if (nlx > 0 && nlx < a.get_width() && nly > 0 && nly < a.get_height()) {
+			inside_name_label = true;
+		}
+	}
+
+	/* double-click inside the name area */
+
 	if ((event->button == 1 && event->type == GDK_2BUTTON_PRESS) || Keyboard::is_edit_event (event)) {
-		/* see if it is inside the name label */
-		if (name_label.is_ancestor (controls_ebox)) {
-			int nlx;
-			int nly;
-			controls_ebox.translate_coordinates (name_label, event->x, event->y, nlx, nly);
-			Gtk::Allocation a = name_label.get_allocation ();
-			if (nlx > 0 && nlx < a.get_width() && nly > 0 && nly < a.get_height()) {
+
+		/* Remember, for a dbl-click, X Window/GDK sends:
+
+		   button press
+		   button release
+		   button press
+		   2button press
+		   (and later, button release)
+
+		   since we would have "started" a track drag
+		   on the button press that precded the 2button press,
+		   we need to cancel it here.
+		*/
+
+		_editor.end_track_drag ();
+
+		if (inside_name_label) {
+
+			if ((event->type == GDK_2BUTTON_PRESS) || Keyboard::is_edit_event (event)) {
 				begin_name_edit ();
 				_ebox_release_can_act = false;
 				return true;
 			}
 		}
-
 	}
 
+	/* double-click outside the name area */
+
 	if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) {
 		if (_effective_height < preset_height (HeightLargest)) {
 			set_height_enum (HeightLargest);
@@ -439,6 +467,8 @@ TimeAxisView::controls_ebox_button_press (GdkEventButton* event)
 
 	if (maybe_set_cursor (event->y) > 0) {
 		_resize_drag_start = event->y_root;
+	} else if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
+		_editor.start_track_drag (*this, event->y, controls_ebox, !inside_name_label);
 	}
 
 	return true;
@@ -453,6 +483,12 @@ TimeAxisView::idle_resize (int32_t h)
 bool
 TimeAxisView::controls_ebox_motion (GdkEventMotion* ev)
 {
+	if (_editor.track_dragging()) {
+		_editor.mid_track_drag (ev, controls_ebox);
+		gdk_event_request_motions (ev);
+		return true;
+	}
+
 	if (_resize_drag_start >= 0) {
 
 		/* (ab)use the DragManager to do autoscrolling - basically we
@@ -468,13 +504,15 @@ TimeAxisView::controls_ebox_motion (GdkEventMotion* ev)
 		_editor.add_to_idle_resize (this, delta);
 		_resize_drag_start = ev->y_root;
 		_did_resize = true;
+		gdk_event_request_motions (ev);
+		return true;
 	} else {
 		/* not dragging but ... */
 		maybe_set_cursor (ev->y);
 	}
 
-	gdk_event_request_motions(ev);
-	return true;
+	gdk_event_request_motions (ev);
+	return false;
 }
 
 bool
@@ -518,23 +556,27 @@ TimeAxisView::maybe_set_cursor (int y)
 bool
 TimeAxisView::controls_ebox_button_release (GdkEventButton* ev)
 {
-	if (_resize_drag_start >= 0) {
-		if (_have_preresize_cursor) {
-			gdk_window_set_cursor (controls_ebox.get_window()->gobj(), _preresize_cursor);
-			_preresize_cursor = 0;
-			_have_preresize_cursor = false;
-		}
-		_editor.stop_canvas_autoscroll ();
-		_resize_drag_start = -1;
-		if (_did_resize) {
-			_did_resize = false;
-			// don't change selection
-			return true;
-		}
+	if (_editor.track_dragging()) {
+		_editor.end_track_drag ();
+	} else {
+		if (_resize_drag_start >= 0) {
+			if (_have_preresize_cursor) {
+				gdk_window_set_cursor (controls_ebox.get_window()->gobj(), _preresize_cursor);
+				_preresize_cursor = 0;
+				_have_preresize_cursor = false;
+			}
+			_editor.stop_canvas_autoscroll ();
+			_resize_drag_start = -1;
+			if (_did_resize) {
+				_did_resize = false;
+				// don't change selection
+				return true;
+			}
 	}
 
-	if (!_ebox_release_can_act) {
-		return true;
+		if (!_ebox_release_can_act) {
+			return true;
+		}
 	}
 
 	switch (ev->button) {
@@ -555,7 +597,7 @@ TimeAxisView::controls_ebox_button_release (GdkEventButton* ev)
 void
 TimeAxisView::selection_click (GdkEventButton* ev)
 {
-	Selection::Operation op = ArdourKeyboard::selection_type (ev->state);
+	SelectionOperation op = ArdourKeyboard::selection_type (ev->state);
 	_editor.set_selected_track (*this, op, false);
 }
 
@@ -763,7 +805,7 @@ TimeAxisView::conditionally_add_to_selection ()
 	Selection& s (_editor.get_selection ());
 
 	if (!s.selected (this)) {
-		_editor.set_selected_track (*this, Selection::Set);
+		_editor.set_selected_track (*this, SelectionSet);
 	}
 }
 
