@@ -124,7 +124,9 @@ TransportMastersWidget::~TransportMastersWidget ()
 void
 TransportMastersWidget::set_transport_master (std::shared_ptr<TransportMaster> tm)
 {
-	_session->request_sync_source (tm);
+	if (_session) {
+		_session->request_sync_source (tm);
+	}
 }
 
 void
@@ -254,7 +256,7 @@ TransportMastersWidget::rebuild ()
 			r->sclock_synced_button.signal_toggled().connect (sigc::mem_fun (*r, &TransportMastersWidget::Row::sync_button_toggled));
 		}
 
-		r->tm->PropertyChanged.connect (r->property_change_connection, invalidator (*this), boost::bind (&TransportMastersWidget::Row::prop_change, r, _1), gui_context());
+		r->tm->PropertyChanged.connect (r->property_change_connection, invalidator (*r), boost::bind (&TransportMastersWidget::Row::prop_change, r, _1), gui_context());
 
 		PropertyChange all_change;
 		all_change.add (Properties::locked);
@@ -271,6 +273,7 @@ TransportMastersWidget::rebuild ()
 	}
 
 	update_usability ();
+	allow_master_select (_session && !_session->config.get_external_sync());
 }
 
 bool
@@ -680,7 +683,9 @@ TransportMastersWidget::set_session (ARDOUR::Session* s)
 
 	if (_session) {
 		_session->config.ParameterChanged.connect (session_config_connection, invalidator (*this), boost::bind (&TransportMastersWidget::param_changed, this, _1), gui_context());
-		allow_master_select (!_session->config.get_external_sync());
+		rebuild ();
+	} else {
+		clear ();
 	}
 }
 
