@@ -31,18 +31,17 @@
 #include "gtk2ardour-config.h"
 #endif
 
-#include <cstdlib>
-#include <clocale>
-#include <cstring>
 #include <cctype>
+#include <clocale>
 #include <cmath>
+#include <cstdlib>
+#include <cstring>
 #include <list>
 #include <sys/stat.h>
 
 #include <boost/algorithm/string.hpp>
 
 #include <gtk/gtkpaned.h>
-#include <gtkmm/combo.h>
 #include <gtkmm/label.h>
 #include <gtkmm/paned.h>
 #include <gtkmm/rc.h>
@@ -52,10 +51,11 @@
 #include "pbd/basename.h"
 #include "pbd/file_utils.h"
 
-#include "ardour/auditioner.h"
 #include "ardour/audioengine.h"
+#include "ardour/auditioner.h"
 #include "ardour/filesystem_paths.h"
 #include "ardour/search_paths.h"
+#include "ardour/session.h"
 #include "ardour/triggerbox.h"
 
 #include "gtkmm2ext/colors.h"
@@ -64,16 +64,18 @@
 #include "canvas/item.h"
 
 #include "actions.h"
-#include "context_menu_helper.h"
-#include "debug.h"
-#include "public_editor.h"
-#include "keyboard.h"
-#include "utils.h"
-#include "pbd/i18n.h"
-#include "gui_thread.h"
-#include "ui_config.h"
 #include "ardour_dialog.h"
 #include "ardour_ui.h"
+#include "context_menu_helper.h"
+#include "debug.h"
+#include "gui_thread.h"
+#include "keyboard.h"
+#include "main_clock.h"
+#include "public_editor.h"
+#include "ui_config.h"
+#include "utils.h"
+
+#include "pbd/i18n.h"
 
 using namespace std;
 using namespace Gtk;
@@ -81,15 +83,15 @@ using namespace Glib;
 using namespace PBD;
 using Gtkmm2ext::Keyboard;
 
-namespace ARDOUR_UI_UTILS {
-	sigc::signal<void>  DPIReset;
+namespace ARDOUR_UI_UTILS
+{
+	sigc::signal<void> DPIReset;
 }
 
 #ifdef PLATFORM_WINDOWS
-#define random() rand()
+#define random() rand ()
 #endif
 
-
 /** Add an element to a menu, settings its sensitivity.
  * @param m Menu to add to.
  * @param e Element to add.
@@ -100,13 +102,12 @@ ARDOUR_UI_UTILS::add_item_with_sensitivity (Menu_Helpers::MenuList& m, Menu_Help
 {
 	m.push_back (e);
 	if (!s) {
-		m.back().set_sensitive (false);
+		m.back ().set_sensitive (false);
 	}
 }
 
-
 gint
-ARDOUR_UI_UTILS::just_hide_it (GdkEventAny */*ev*/, Gtk::Window *win)
+ARDOUR_UI_UTILS::just_hide_it (GdkEventAny* /*ev*/, Gtk::Window* win)
 {
 	win->hide ();
 	return 0;
@@ -118,12 +119,12 @@ idle_notify_engine_stopped ()
 	Glib::RefPtr<ToggleAction> tact = ActionManager::get_toggle_action ("Window", "toggle-audio-midi-setup");
 
 	MessageDialog msg (
-			_("The current operation is not possible because of an error communicating with the audio hardware."),
-			false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true);
+	    _("The current operation is not possible because of an error communicating with the audio hardware."),
+	    false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true);
 
 	msg.add_button (_("Cancel"), Gtk::RESPONSE_CANCEL);
 
-	if (tact && !tact->get_active()) {
+	if (tact && !tact->get_active ()) {
 		msg.add_button (_("Configure Hardware"), Gtk::RESPONSE_OK);
 	}
 
@@ -136,14 +137,13 @@ idle_notify_engine_stopped ()
 bool
 ARDOUR_UI_UTILS::engine_is_running ()
 {
-	if (ARDOUR::AudioEngine::instance()->running ()) {
+	if (ARDOUR::AudioEngine::instance ()->running ()) {
 		return true;
 	}
-	Glib::signal_idle().connect (sigc::ptr_fun (&idle_notify_engine_stopped));
+	Glib::signal_idle ().connect (sigc::ptr_fun (&idle_notify_engine_stopped));
 	return false;
 }
 
-
 /* xpm2rgb copied from nixieclock, which bore the legend:
 
     nixieclock - a nixie desktop timepiece
@@ -155,24 +155,24 @@ ARDOUR_UI_UTILS::engine_is_running ()
 unsigned char*
 ARDOUR_UI_UTILS::xpm2rgb (const char** xpm, uint32_t& w, uint32_t& h)
 {
-	static long vals[256], val;
-	uint32_t t, x, y, colors, cpp;
-	unsigned char c;
+	static long    vals[256], val;
+	uint32_t       t, x, y, colors, cpp;
+	unsigned char  c;
 	unsigned char *savergb, *rgb;
 
 	// PARSE HEADER
 
-	if ( sscanf(xpm[0], "%u%u%u%u", &w, &h, &colors, &cpp) != 4 ) {
+	if (sscanf (xpm[0], "%u%u%u%u", &w, &h, &colors, &cpp) != 4) {
 		error << string_compose (_("bad XPM header %1"), xpm[0])
 		      << endmsg;
 		return 0;
 	}
 
-	savergb = rgb = (unsigned char*) malloc (h * w * 3);
+	savergb = rgb = (unsigned char*)malloc (h * w * 3);
 
 	// LOAD XPM COLORMAP LONG ENOUGH TO DO CONVERSION
 	for (t = 0; t < colors; ++t) {
-		sscanf (xpm[t+1], "%c c #%lx", &c, &val);
+		sscanf (xpm[t + 1], "%c c #%lx", &c, &val);
 		vals[c] = val;
 	}
 
@@ -180,14 +180,15 @@ ARDOUR_UI_UTILS::xpm2rgb (const char** xpm, uint32_t& w, uint32_t& h)
 	//    Get low 3 bytes from vals[]
 	//
 
-	const char *p;
-	for (y = h-1; y > 0; --y) {
-
-		for (p = xpm[1+colors+(h-y-1)], x = 0; x < w; x++, rgb += 3) {
-			val = vals[(int)*p++];
-			*(rgb+2) = val & 0xff; val >>= 8;  // 2:B
-			*(rgb+1) = val & 0xff; val >>= 8;  // 1:G
-			*(rgb+0) = val & 0xff;             // 0:R
+	const char* p;
+	for (y = h - 1; y > 0; --y) {
+		for (p = xpm[1 + colors + (h - y - 1)], x = 0; x < w; x++, rgb += 3) {
+			val        = vals[(int)*p++];
+			*(rgb + 2) = val & 0xff;
+			val >>= 8; // 2:B
+			*(rgb + 1) = val & 0xff;
+			val >>= 8;               // 1:G
+			*(rgb + 0) = val & 0xff; // 0:R
 		}
 	}
 
@@ -197,21 +198,21 @@ ARDOUR_UI_UTILS::xpm2rgb (const char** xpm, uint32_t& w, uint32_t& h)
 unsigned char*
 ARDOUR_UI_UTILS::xpm2rgba (const char** xpm, uint32_t& w, uint32_t& h)
 {
-	static long vals[256], val;
-	uint32_t t, x, y, colors, cpp;
-	unsigned char c;
+	static long    vals[256], val;
+	uint32_t       t, x, y, colors, cpp;
+	unsigned char  c;
 	unsigned char *savergb, *rgb;
-	char transparent;
+	char           transparent;
 
 	// PARSE HEADER
 
-	if ( sscanf(xpm[0], "%u%u%u%u", &w, &h, &colors, &cpp) != 4 ) {
+	if (sscanf (xpm[0], "%u%u%u%u", &w, &h, &colors, &cpp) != 4) {
 		error << string_compose (_("bad XPM header %1"), xpm[0])
 		      << endmsg;
 		return 0;
 	}
 
-	savergb = rgb = (unsigned char*) malloc (h * w * 4);
+	savergb = rgb = (unsigned char*)malloc (h * w * 4);
 
 	// LOAD XPM COLORMAP LONG ENOUGH TO DO CONVERSION
 
@@ -220,11 +221,11 @@ ARDOUR_UI_UTILS::xpm2rgba (const char** xpm, uint32_t& w, uint32_t& h)
 		t = 1;
 	} else {
 		transparent = 0;
-		t = 0;
+		t           = 0;
 	}
 
 	for (; t < colors; ++t) {
-		sscanf (xpm[t+1], "%c c #%lx", &c, &val);
+		sscanf (xpm[t + 1], "%c c #%lx", &c, &val);
 		vals[c] = val;
 	}
 
@@ -232,25 +233,25 @@ ARDOUR_UI_UTILS::xpm2rgba (const char** xpm, uint32_t& w, uint32_t& h)
 	//    Get low 3 bytes from vals[]
 	//
 
-	const char *p;
-	for (y = h-1; y > 0; --y) {
-
+	const char* p;
+	for (y = h - 1; y > 0; --y) {
 		char alpha;
 
-		for (p = xpm[1+colors+(h-y-1)], x = 0; x < w; x++, rgb += 4) {
-
+		for (p = xpm[1 + colors + (h - y - 1)], x = 0; x < w; x++, rgb += 4) {
 			if (transparent && (*p++ == transparent)) {
 				alpha = 0;
-				val = 0;
+				val   = 0;
 			} else {
-				alpha = 255;
-				val = vals[(int)*p];
+				alpha = 0xff;
+				val   = vals[(int)*p];
 			}
 
-			*(rgb+3) = alpha;                  // 3: alpha
-			*(rgb+2) = val & 0xff; val >>= 8;  // 2:B
-			*(rgb+1) = val & 0xff; val >>= 8;  // 1:G
-			*(rgb+0) = val & 0xff;             // 0:R
+			*(rgb + 3) = alpha; // 3: alpha
+			*(rgb + 2) = val & 0xff;
+			val >>= 8; // 2:B
+			*(rgb + 1) = val & 0xff;
+			val >>= 8;               // 1:G
+			*(rgb + 0) = val & 0xff; // 0:R
 		}
 	}
 
@@ -276,7 +277,7 @@ ARDOUR_UI_UTILS::sanitized_font (std::string const& name)
 {
 	Pango::FontDescription fd (name);
 
-	if (fd.get_family().empty()) {
+	if (fd.get_family ().empty ()) {
 		/* default: "Sans" or "ArdourSans" */
 		fd.set_family (UIConfiguration::instance ().get_ui_font_family ());
 	}
@@ -288,7 +289,7 @@ Pango::FontDescription
 ARDOUR_UI_UTILS::ardour_font (std::string const& name)
 {
 	Pango::FontDescription fd (name);
-	if (!fd.get_family().empty() && fd.get_family().find ("Mon") != std::string::npos) {
+	if (!fd.get_family ().empty () && fd.get_family ().find ("Mon") != std::string::npos) {
 		/* matches "ArdourMono", "Monaco" */
 		fd.set_family ("ArdourMono");
 	} else {
@@ -298,31 +299,28 @@ ARDOUR_UI_UTILS::ardour_font (std::string const& name)
 	return fd;
 }
 
-
-
 Pango::FontDescription
 ARDOUR_UI_UTILS::get_font_for_style (string widgetname)
 {
-	Gtk::Window window (WINDOW_TOPLEVEL);
-	Gtk::Label foobar;
+	Gtk::Window              window (WINDOW_TOPLEVEL);
+	Gtk::Label               foobar;
 	Glib::RefPtr<Gtk::Style> style;
 
 	window.add (foobar);
 	foobar.set_name (widgetname);
-	foobar.ensure_style();
+	foobar.ensure_style ();
 
 	style = foobar.get_style ();
 
-	Glib::RefPtr<const Pango::Layout> layout = foobar.get_layout();
+	Glib::RefPtr<const Pango::Layout> layout = foobar.get_layout ();
 
-	PangoFontDescription *pfd = const_cast<PangoFontDescription *> (pango_layout_get_font_description(const_cast<PangoLayout *>(layout->gobj())));
+	PangoFontDescription* pfd = const_cast<PangoFontDescription*> (pango_layout_get_font_description (const_cast<PangoLayout*> (layout->gobj ())));
 
 	if (!pfd) {
-
 		/* layout inherited its font description from a PangoContext */
 
-		PangoContext* ctxt = (PangoContext*) pango_layout_get_context (const_cast<PangoLayout*>(layout->gobj()));
-		pfd =  pango_context_get_font_description (ctxt);
+		PangoContext* ctxt = (PangoContext*)pango_layout_get_context (const_cast<PangoLayout*> (layout->gobj ()));
+		pfd                = pango_context_get_font_description (ctxt);
 		return Pango::FontDescription (pfd); /* make a copy */
 	}
 
@@ -332,59 +330,62 @@ ARDOUR_UI_UTILS::get_font_for_style (string widgetname)
 bool
 ARDOUR_UI_UTILS::relay_key_press (GdkEventKey* ev, Gtk::Window* win)
 {
-	return ARDOUR_UI::instance()->key_event_handler (ev, win);
+	return ARDOUR_UI::instance ()->key_event_handler (ev, win);
 }
 
 bool
 ARDOUR_UI_UTILS::emulate_key_event (unsigned int keyval)
 {
-	GdkDisplay  *display = gtk_widget_get_display (GTK_WIDGET(ARDOUR_UI::instance()->main_window().gobj()));
-	GdkKeymap   *keymap  = gdk_keymap_get_for_display (display);
-	GdkKeymapKey *keymapkey = NULL;
-	gint n_keys;
+	GdkDisplay*   display   = gtk_widget_get_display (GTK_WIDGET (ARDOUR_UI::instance ()->main_window ().gobj ()));
+	GdkKeymap*    keymap    = gdk_keymap_get_for_display (display);
+	GdkKeymapKey* keymapkey = NULL;
+	gint          n_keys;
 
-	if (!gdk_keymap_get_entries_for_keyval(keymap, keyval, &keymapkey, &n_keys)) return false;
-	if (n_keys !=1) { g_free(keymapkey); return false;}
+	if (!gdk_keymap_get_entries_for_keyval (keymap, keyval, &keymapkey, &n_keys))
+		return false;
+	if (n_keys != 1) {
+		g_free (keymapkey);
+		return false;
+	}
 
-	Gtk::Window& main_window (ARDOUR_UI::instance()->main_window());
+	Gtk::Window& main_window (ARDOUR_UI::instance ()->main_window ());
 
 	GdkEventKey ev;
-	ev.type = GDK_KEY_PRESS;
-	ev.window = main_window.get_window()->gobj();
-	ev.send_event = FALSE;
-	ev.time = 0;
-	ev.state = 0;
-	ev.keyval = keyval;
-	ev.length = 0;
-	ev.string = const_cast<gchar*> ("");
+	ev.type             = GDK_KEY_PRESS;
+	ev.window           = main_window.get_window ()->gobj ();
+	ev.send_event       = FALSE;
+	ev.time             = 0;
+	ev.state            = 0;
+	ev.keyval           = keyval;
+	ev.length           = 0;
+	ev.string           = const_cast<gchar*> ("");
 	ev.hardware_keycode = keymapkey[0].keycode;
-	ev.group = keymapkey[0].group;
-	g_free(keymapkey);
+	ev.group            = keymapkey[0].group;
+	g_free (keymapkey);
 
 	relay_key_press (&ev, &main_window);
 	ev.type = GDK_KEY_RELEASE;
-	return relay_key_press(&ev, &main_window);
+	return relay_key_press (&ev, &main_window);
 }
 
 Glib::RefPtr<Gdk::Pixbuf>
 ARDOUR_UI_UTILS::get_xpm (std::string name)
 {
 	if (!xpm_map[name]) {
+		Searchpath spath (ARDOUR::ardour_data_search_path ());
 
-		Searchpath spath(ARDOUR::ardour_data_search_path());
-
-		spath.add_subdirectory_to_paths("pixmaps");
+		spath.add_subdirectory_to_paths ("pixmaps");
 
 		std::string data_file_path;
 
-		if(!find_file (spath, name, data_file_path)) {
+		if (!find_file (spath, name, data_file_path)) {
 			fatal << string_compose (_("cannot find XPM file for %1"), name) << endmsg;
 		}
 
 		try {
-			xpm_map[name] =  Gdk::Pixbuf::create_from_file (data_file_path);
+			xpm_map[name] = Gdk::Pixbuf::create_from_file (data_file_path);
 		} catch (const Glib::Error& e) {
-			warning << "Caught Glib::Error: " << e.what() << endmsg;
+			warning << "Caught Glib::Error: " << e.what () << endmsg;
 		}
 	}
 
@@ -392,24 +393,22 @@ ARDOUR_UI_UTILS::get_xpm (std::string name)
 }
 
 void
-ARDOUR_UI_UTILS::get_color_themes (map<std::string,std::string>& themes)
+ARDOUR_UI_UTILS::get_color_themes (map<std::string, std::string>& themes)
 {
-	Searchpath spath(ARDOUR::theme_search_path());
-
-	for (vector<string>::iterator s = spath.begin(); s != spath.end(); ++s) {
+	Searchpath spath (ARDOUR::theme_search_path ());
 
+	for (vector<string>::iterator s = spath.begin (); s != spath.end (); ++s) {
 		vector<string> entries;
 
 		find_files_matching_pattern (entries, *s, string ("*") + UIConfiguration::color_file_suffix);
 
-		for (vector<string>::iterator e = entries.begin(); e != entries.end(); ++e) {
-
+		for (vector<string>::iterator e = entries.begin (); e != entries.end (); ++e) {
 			XMLTree tree;
 
-			tree.read ((*e).c_str());
-			XMLNode* root = tree.root();
+			tree.read ((*e).c_str ());
+			XMLNode* root = tree.root ();
 
-			if (!root || root->name() != X_("Ardour")) {
+			if (!root || root->name () != X_("Ardour")) {
 				continue;
 			}
 
@@ -419,12 +418,12 @@ ARDOUR_UI_UTILS::get_color_themes (map<std::string,std::string>& themes)
 				continue;
 			}
 
-			std::string color_name = basename_nosuffix(*e);
-			size_t sep = color_name.find_first_of("-");
+			std::string color_name = basename_nosuffix (*e);
+			size_t      sep        = color_name.find_first_of ("-");
 			if (sep != string::npos) {
 				color_name = color_name.substr (0, sep);
 			}
-			themes.insert (make_pair (prop->value(), color_name));
+			themes.insert (make_pair (prop->value (), color_name));
 		}
 	}
 }
@@ -432,21 +431,20 @@ ARDOUR_UI_UTILS::get_color_themes (map<std::string,std::string>& themes)
 vector<string>
 ARDOUR_UI_UTILS::get_icon_sets ()
 {
-	Searchpath spath(ARDOUR::ardour_data_search_path());
+	Searchpath spath (ARDOUR::ardour_data_search_path ());
 	spath.add_subdirectory_to_paths ("icons");
 	vector<string> r;
 
 	r.push_back (_("default"));
 
-	for (vector<string>::iterator s = spath.begin(); s != spath.end(); ++s) {
-
+	for (vector<string>::iterator s = spath.begin (); s != spath.end (); ++s) {
 		vector<string> entries;
 
 		get_paths (entries, *s, false, false);
 
-		for (vector<string>::iterator e = entries.begin(); e != entries.end(); ++e) {
+		for (vector<string>::iterator e = entries.begin (); e != entries.end (); ++e) {
 			if (Glib::file_test (*e, Glib::FILE_TEST_IS_DIR)) {
-				r.push_back (Glib::filename_to_utf8 (Glib::path_get_basename(*e)));
+				r.push_back (Glib::filename_to_utf8 (Glib::path_get_basename (*e)));
 			}
 		}
 	}
@@ -458,16 +456,15 @@ std::string
 ARDOUR_UI_UTILS::get_icon_path (const char* cname, string icon_set, bool is_image)
 {
 	std::string data_file_path;
-	string name = cname;
+	string      name = cname;
 
 	if (is_image) {
 		name += X_(".png");
 	}
 
-	Searchpath spath(ARDOUR::ardour_data_search_path());
-
-	if (!icon_set.empty() && icon_set != _("default")) {
+	Searchpath spath (ARDOUR::ardour_data_search_path ());
 
+	if (!icon_set.empty () && icon_set != _("default")) {
 		/* add "icons/icon_set" but .. not allowed to add both of these at once */
 		spath.add_subdirectory_to_paths ("icons");
 		spath.add_subdirectory_to_paths (icon_set);
@@ -478,24 +475,23 @@ ARDOUR_UI_UTILS::get_icon_path (const char* cname, string icon_set, bool is_imag
 		find_file (spath, name, data_file_path);
 	}
 
-	if (data_file_path.empty()) {
-		Searchpath rc (ARDOUR::ardour_data_search_path());
+	if (data_file_path.empty ()) {
+		Searchpath rc (ARDOUR::ardour_data_search_path ());
 		rc.add_subdirectory_to_paths ("resources");
 		find_file (rc, name, data_file_path);
 	}
 
-	if (is_image && data_file_path.empty()) {
-
-		if (!icon_set.empty() && icon_set != _("default")) {
+	if (is_image && data_file_path.empty ()) {
+		if (!icon_set.empty () && icon_set != _("default")) {
 			warning << string_compose (_("icon \"%1\" not found for icon set \"%2\", fallback to default"), cname, icon_set) << endmsg;
 		}
 
-		Searchpath def (ARDOUR::ardour_data_search_path());
+		Searchpath def (ARDOUR::ardour_data_search_path ());
 		def.add_subdirectory_to_paths ("icons");
 
 		if (!find_file (def, name, data_file_path)) {
-			fatal << string_compose (_("cannot find icon image for %1 using %2"), name, spath.to_string()) << endmsg;
-			abort(); /*NOTREACHED*/
+			fatal << string_compose (_("cannot find icon image for %1 using %2"), name, spath.to_string ()) << endmsg;
+			abort (); /*NOTREACHED*/
 		}
 	}
 
@@ -508,8 +504,8 @@ ARDOUR_UI_UTILS::get_icon (const char* cname, string icon_set)
 	Glib::RefPtr<Gdk::Pixbuf> img;
 	try {
 		img = Gdk::Pixbuf::create_from_file (get_icon_path (cname, icon_set));
-	} catch (const Gdk::PixbufError &e) {
-		cerr << "Caught PixbufError: " << e.what() << endl;
+	} catch (const Gdk::PixbufError& e) {
+		cerr << "Caught PixbufError: " << e.what () << endl;
 	} catch (...) {
 		error << string_compose (_("Caught exception while loading icon named %1"), cname) << endmsg;
 	}
@@ -517,42 +513,42 @@ ARDOUR_UI_UTILS::get_icon (const char* cname, string icon_set)
 	return img;
 }
 
-namespace ARDOUR_UI_UTILS {
+namespace ARDOUR_UI_UTILS
+{
 Glib::RefPtr<Gdk::Pixbuf>
 get_icon (const char* cname)
 {
 	Glib::RefPtr<Gdk::Pixbuf> img;
 	try {
 		img = Gdk::Pixbuf::create_from_file (get_icon_path (cname));
-	} catch (const Gdk::PixbufError &e) {
-		cerr << "Caught PixbufError: " << e.what() << endl;
+	} catch (const Gdk::PixbufError& e) {
+		cerr << "Caught PixbufError: " << e.what () << endl;
 	} catch (...) {
 		error << string_compose (_("Caught exception while loading icon named %1"), cname) << endmsg;
 	}
 
 	return img;
 }
-}
+} // namespace ARDOUR_UI_UTILS
 
 string
 ARDOUR_UI_UTILS::longest (vector<string>& strings)
 {
-	if (strings.empty()) {
+	if (strings.empty ()) {
 		return string ("");
 	}
 
-	vector<string>::iterator longest = strings.begin();
-	string::size_type longest_length = (*longest).length();
+	vector<string>::iterator longest        = strings.begin ();
+	string::size_type        longest_length = (*longest).length ();
 
 	vector<string>::iterator i = longest;
 	++i;
 
-	while (i != strings.end()) {
-
-		string::size_type len = (*i).length();
+	while (i != strings.end ()) {
+		string::size_type len = (*i).length ();
 
 		if (len > longest_length) {
-			longest = i;
+			longest        = i;
 			longest_length = len;
 		}
 
@@ -565,107 +561,142 @@ ARDOUR_UI_UTILS::longest (vector<string>& strings)
 bool
 ARDOUR_UI_UTILS::key_is_legal_for_numeric_entry (guint keyval)
 {
-	/* we assume that this does not change over the life of the process
-	 */
-
+	/* we assume that this does not change over the life of the process */
 	static int comma_decimal = -1;
 
 	switch (keyval) {
-	case GDK_period:
-	case GDK_comma:
-		if (comma_decimal < 0) {
-			std::lconv* lc = std::localeconv();
-			if (strchr (lc->decimal_point, ',') != 0) {
-				comma_decimal = 1;
-			} else {
-				comma_decimal = 0;
+		case GDK_period:
+		case GDK_comma:
+			if (comma_decimal < 0) {
+				std::lconv* lc = std::localeconv ();
+				if (strchr (lc->decimal_point, ',') != 0) {
+					comma_decimal = 1;
+				} else {
+					comma_decimal = 0;
+				}
 			}
-		}
-		break;
-	default:
-		break;
+			break;
+		default:
+			break;
 	}
 
 	switch (keyval) {
-	case GDK_decimalpoint:
-	case GDK_KP_Separator:
-		return true;
-
-	case GDK_period:
-		if (comma_decimal) {
-			return false;
-		} else {
+		case GDK_decimalpoint:
+		case GDK_KP_Separator:
 			return true;
-		}
-		break;
-	case GDK_comma:
-		if (comma_decimal) {
-			return true;
-		} else {
-			return false;
-		}
-		break;
-	case GDK_minus:
-	case GDK_plus:
-	case GDK_0:
-	case GDK_1:
-	case GDK_2:
-	case GDK_3:
-	case GDK_4:
-	case GDK_5:
-	case GDK_6:
-	case GDK_7:
-	case GDK_8:
-	case GDK_9:
-	case GDK_KP_Add:
-	case GDK_KP_Subtract:
-	case GDK_KP_Decimal:
-	case GDK_KP_0:
-	case GDK_KP_1:
-	case GDK_KP_2:
-	case GDK_KP_3:
-	case GDK_KP_4:
-	case GDK_KP_5:
-	case GDK_KP_6:
-	case GDK_KP_7:
-	case GDK_KP_8:
-	case GDK_KP_9:
-	case GDK_Return:
-	case GDK_BackSpace:
-	case GDK_Delete:
-	case GDK_KP_Enter:
-	case GDK_Home:
-	case GDK_End:
-	case GDK_Left:
-	case GDK_Right:
-		return true;
 
-	default:
-		break;
+		case GDK_period:
+			if (comma_decimal) {
+				return false;
+			} else {
+				return true;
+			}
+			break;
+		case GDK_comma:
+			if (comma_decimal) {
+				return true;
+			} else {
+				return false;
+			}
+			break;
+		case GDK_minus:
+		case GDK_plus:
+		case GDK_0:
+		case GDK_1:
+		case GDK_2:
+		case GDK_3:
+		case GDK_4:
+		case GDK_5:
+		case GDK_6:
+		case GDK_7:
+		case GDK_8:
+		case GDK_9:
+		case GDK_KP_Add:
+		case GDK_KP_Subtract:
+		case GDK_KP_Decimal:
+		case GDK_KP_0:
+		case GDK_KP_1:
+		case GDK_KP_2:
+		case GDK_KP_3:
+		case GDK_KP_4:
+		case GDK_KP_5:
+		case GDK_KP_6:
+		case GDK_KP_7:
+		case GDK_KP_8:
+		case GDK_KP_9:
+		case GDK_Return:
+		case GDK_BackSpace:
+		case GDK_Delete:
+		case GDK_KP_Enter:
+		case GDK_Home:
+		case GDK_End:
+		case GDK_Left:
+		case GDK_Right:
+			return true;
+
+		default:
+			break;
 	}
 
 	return false;
 }
 
+int
+ARDOUR_UI_UTILS::guess_default_ui_scale ()
+{
+#ifdef __APPLE__
+	return 100;
+#else
+	gint width = 0;
+	gint height = 0;
+	GdkScreen* screen = gdk_display_get_screen (gdk_display_get_default (), 0);
+	gint n_monitors = gdk_screen_get_n_monitors (screen);
+
+	if (!screen) {
+		return 100;
+	}
+
+	for (gint i = 0; i < n_monitors; ++i) {
+		GdkRectangle rect;
+		gdk_screen_get_monitor_geometry (screen, i, &rect);
+		width = std::max (width, rect.width);
+		height = std::max (height, rect.height);
+	}
+
+	float wx = width  / 1920.f;
+	float hx = height / 1080.f;
+	float sx = std::min (wx, hx);
+
+	if (sx < 1.25) {
+		return 100;
+	} else if (sx < 1.6) {
+		return 150;
+	} else if (sx < 2.1) {
+		return 200;
+	} else {
+		return 250;
+	}
+#endif
+}
+
 void
 ARDOUR_UI_UTILS::resize_window_to_proportion_of_monitor (Gtk::Window* window, int max_width, int max_height)
 {
 	Glib::RefPtr<Gdk::Screen> screen = window->get_screen ();
-	Gdk::Rectangle monitor_rect;
+	Gdk::Rectangle            monitor_rect;
 	screen->get_monitor_geometry (0, monitor_rect);
 
-	int const w = std::min (int (monitor_rect.get_width() * 0.8), max_width);
-	int const h = std::min (int (monitor_rect.get_height() * 0.8), max_height);
+	int const w = std::min (int (monitor_rect.get_width () * 0.8), max_width);
+	int const h = std::min (int (monitor_rect.get_height () * 0.8), max_height);
 
 	window->resize (w, h);
 }
 
-
 /** Replace _ with __ in a string; for use with menu item text to make underscores displayed correctly */
 string
-ARDOUR_UI_UTILS::escape_underscores (string const & s)
+ARDOUR_UI_UTILS::escape_underscores (string const& s)
 {
-	string o;
+	string                  o;
 	string::size_type const N = s.length ();
 
 	for (string::size_type i = 0; i < N; ++i) {
@@ -682,13 +713,13 @@ ARDOUR_UI_UTILS::escape_underscores (string const & s)
 Gdk::Color
 ARDOUR_UI_UTILS::round_robin_palette_color ()
 {
-	Gdk::Color newcolor;
-	string cp = UIConfiguration::instance().get_stripable_color_palette ();
-	Gdk::ArrayHandle_Color gc = ColorSelection::palette_from_string (cp);
-	std::vector<Gdk::Color> c (gc);
+	Gdk::Color                                newcolor;
+	string                                    cp = UIConfiguration::instance ().get_stripable_color_palette ();
+	Gdk::ArrayHandle_Color                    gc = ColorSelection::palette_from_string (cp);
+	std::vector<Gdk::Color>                   c (gc);
 	static std::vector<Gdk::Color>::size_type index = 0;
 
-	if (index >= c.size()) {
+	if (index >= c.size ()) {
 		index = 0;
 	}
 
@@ -701,31 +732,30 @@ ARDOUR_UI_UTILS::unique_random_color (list<Gdk::Color>& used_colors)
 	Gdk::Color newcolor;
 
 	while (1) {
-
 		double h, s, v;
 
-		h = fmod (random(), 360.0);
-		s = (random() % 65535) / 65535.0;
-		v = (random() % 65535) / 65535.0;
+		h = fmod (random (), 360.0);
+		s = (random () % 65535) / 65535.0;
+		v = (random () % 65535) / 65535.0;
 
 		s = min (0.5, s); /* not too saturated */
-		v = max (0.9, v);  /* not too bright */
+		v = max (0.9, v); /* not too bright */
 		newcolor.set_hsv (h, s, v);
 
-		if (used_colors.size() == 0) {
+		if (used_colors.size () == 0) {
 			used_colors.push_back (newcolor);
 			return newcolor;
 		}
 
-		for (list<Gdk::Color>::iterator i = used_colors.begin(); i != used_colors.end(); ++i) {
-		  Gdk::Color c = *i;
-			float rdelta, bdelta, gdelta;
+		for (list<Gdk::Color>::iterator i = used_colors.begin (); i != used_colors.end (); ++i) {
+			Gdk::Color c = *i;
+			float      rdelta, bdelta, gdelta;
 
-			rdelta = newcolor.get_red() - c.get_red();
-			bdelta = newcolor.get_blue() - c.get_blue();
-			gdelta = newcolor.get_green() - c.get_green();
+			rdelta = newcolor.get_red () - c.get_red ();
+			bdelta = newcolor.get_blue () - c.get_blue ();
+			gdelta = newcolor.get_green () - c.get_green ();
 
-			if (sqrt (rdelta*rdelta + bdelta*bdelta + gdelta*gdelta) > 25.0) {
+			if (sqrt (rdelta * rdelta + bdelta * bdelta + gdelta * gdelta) > 25.0) {
 				/* different enough */
 				used_colors.push_back (newcolor);
 				return newcolor;
@@ -740,10 +770,12 @@ string
 ARDOUR_UI_UTILS::rate_as_string (float r)
 {
 	char buf[32];
-	if (fmod (r, 1000.0f)) {
-		snprintf (buf, sizeof (buf), "%.1f kHz", r/1000.0);
+	if (fmod (r, 100.0f)) {
+		snprintf (buf, sizeof (buf), "%.2f kHz", r / 1000.0);
+	} else if (fmod (r, 1000.0f)) {
+		snprintf (buf, sizeof (buf), "%.1f kHz", r / 1000.0);
 	} else {
-		snprintf (buf, sizeof (buf), "%.0f kHz", r/1000.0);
+		snprintf (buf, sizeof (buf), "%.0f kHz", r / 1000.0);
 	}
 	return buf;
 }
@@ -758,7 +790,7 @@ ARDOUR_UI_UTILS::samples_as_time_string (samplecnt_t s, float rate, bool show_sa
 		snprintf (buf, sizeof (buf), "0");
 	} else if (s < 1000 && show_samples) {
 		/* 0 .. 999 spl */
-		snprintf (buf, sizeof (buf), "%" PRId64" spl", s);
+		snprintf (buf, sizeof (buf), "%" PRId64 " spl", s);
 	} else if (s < (rate / 1000.f)) {
 		/* 0 .. 999 usec */
 		snprintf (buf, sizeof (buf), u8"%.0f \u00B5s", s * 1e+6f / rate);
@@ -794,22 +826,21 @@ ARDOUR_UI_UTILS::midi_channels_as_string (std::bitset<16> channels)
 
 	string rv;
 
-	for (int i = 0; i<16; i++) {
-
-		bool prior = i<1 ? false : channels.test(i-1);
-		bool current = channels.test(i);
-		bool next = i>14 ? false : channels.test(i+1);
-		bool nextnext = i>13 ? false : channels.test(i+2);
-		bool future = false;
-		for (int f = i+1; f<16; f++) {
-			if (channels.test(f)) {
+	for (int i = 0; i < 16; i++) {
+		bool prior    = i < 1 ? false : channels.test (i - 1);
+		bool current  = channels.test (i);
+		bool next     = i > 14 ? false : channels.test (i + 1);
+		bool nextnext = i > 13 ? false : channels.test (i + 2);
+		bool future   = false;
+		for (int f = i + 1; f < 16; f++) {
+			if (channels.test (f)) {
 				future = true;
 			}
 		}
 
 		if (prior && current && next) {
 			/* I'm in the middle of a consecutive chain, maybe just add a dash */
-			if (!rv.empty() && (rv.rfind("-") != rv.length()-1 )) {
+			if (!rv.empty () && (rv.rfind ("-") != rv.length () - 1)) {
 				rv += "-";
 			}
 			continue;
@@ -817,7 +848,7 @@ ARDOUR_UI_UTILS::midi_channels_as_string (std::bitset<16> channels)
 
 		if (current) {
 			/* here I am! */
-			rv+=to_string<int> (i+1);
+			rv += to_string<int> (i + 1);
 		}
 
 		if (current && future && !(next && nextnext)) {
@@ -830,13 +861,12 @@ ARDOUR_UI_UTILS::midi_channels_as_string (std::bitset<16> channels)
 }
 
 bool
-ARDOUR_UI_UTILS::windows_overlap (Gtk::Window *a, Gtk::Window *b)
+ARDOUR_UI_UTILS::windows_overlap (Gtk::Window* a, Gtk::Window* b)
 {
-
 	if (!a || !b) {
 		return false;
 	}
-	if (a->get_screen() == b->get_screen()) {
+	if (a->get_screen () == b->get_screen ()) {
 		gint ex, ey, ew, eh;
 		gint mx, my, mw, mh;
 
@@ -849,14 +879,14 @@ ARDOUR_UI_UTILS::windows_overlap (Gtk::Window *a, Gtk::Window *b)
 		GdkRectangle m;
 		GdkRectangle r;
 
-		e.x = ex;
-		e.y = ey;
-		e.width = ew;
+		e.x      = ex;
+		e.y      = ey;
+		e.width  = ew;
 		e.height = eh;
 
-		m.x = mx;
-		m.y = my;
-		m.width = mw;
+		m.x      = mx;
+		m.y      = my;
+		m.width  = mw;
 		m.height = mh;
 
 		if (gdk_rectangle_intersect (&e, &m, &r)) {
@@ -870,44 +900,44 @@ bool
 ARDOUR_UI_UTILS::overwrite_file_dialog (Gtk::Window& parent, string title, string text)
 {
 	ArdourDialog dialog (parent, title, true);
-	Label label (text);
+	Label        label (text);
 
-	dialog.get_vbox()->pack_start (label, true, true);
+	dialog.get_vbox ()->pack_start (label, true, true);
 	dialog.add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 	dialog.add_button (_("Overwrite"), Gtk::RESPONSE_ACCEPT);
 	dialog.show_all ();
 
-	switch (dialog.run()) {
-	case RESPONSE_ACCEPT:
-		return true;
-	default:
-		return false;
+	switch (dialog.run ()) {
+		case RESPONSE_ACCEPT:
+			return true;
+		default:
+			return false;
 	}
 }
 
 bool
 ARDOUR_UI_UTILS::running_from_source_tree ()
 {
-	gchar const *x = g_getenv ("ARDOUR_THEMES_PATH");
+	gchar const* x = g_getenv ("ARDOUR_THEMES_PATH");
 	return x && (string (x).find ("gtk2_ardour") != string::npos);
 }
 
 Gtk::Menu*
 ARDOUR_UI_UTILS::shared_popup_menu ()
 {
-	return ARDOUR_UI::instance()->shared_popup_menu ();
+	return ARDOUR_UI::instance ()->shared_popup_menu ();
 }
 
 bool
 ARDOUR_UI_UTILS::convert_drop_to_paths (vector<string>& paths, const SelectionData& data)
 {
-	vector<string> uris = data.get_uris();
+	vector<string> uris = data.get_uris ();
 
 	if (uris.empty ()) {
 		/* This is seriously fucked up. Nautilus doesn't say that its URI lists
 		 * are actually URI lists. So do it by hand.
 		 */
-		if (data.get_target() != "text/plain") {
+		if (data.get_target () != "text/plain") {
 			return false;
 		}
 
@@ -916,14 +946,14 @@ ARDOUR_UI_UTILS::convert_drop_to_paths (vector<string>& paths, const SelectionDa
 		 *
 		 * THERE MAY BE NO NULL TERMINATING CHAR!!!
 		 */
-		string txt = data.get_text();
+		string txt = data.get_text ();
 
 		/* copy to char* for easy char-wise checks and modification */
-		char* tmp = (char *) malloc (txt.length() + 1);
+		char* tmp = (char*)malloc (txt.length () + 1);
 		char* p   = tmp;
 
-		txt.copy (p, txt.length(), 0);
-		p[txt.length()] = '\0';
+		txt.copy (p, txt.length (), 0);
+		p[txt.length ()] = '\0';
 
 		while (p) {
 			if (*p != '#') {
@@ -955,13 +985,13 @@ ARDOUR_UI_UTILS::convert_drop_to_paths (vector<string>& paths, const SelectionDa
 
 		free ((void*)tmp);
 
-		if (uris.empty()) {
+		if (uris.empty ()) {
 			return false;
 		}
 	}
 
-	for (vector<string>::iterator i = uris.begin(); i != uris.end(); ++i) {
-		if ((*i).substr (0,7) == "file://") {
+	for (vector<string>::iterator i = uris.begin (); i != uris.end (); ++i) {
+		if ((*i).substr (0, 7) == "file://") {
 			paths.push_back (Glib::filename_from_uri (*i));
 		}
 	}
@@ -972,16 +1002,86 @@ ARDOUR_UI_UTILS::convert_drop_to_paths (vector<string>& paths, const SelectionDa
 void
 ARDOUR_UI_UTILS::copy_patch_changes (std::shared_ptr<ARDOUR::Auditioner> a, std::shared_ptr<ARDOUR::Trigger> t)
 {
-	std::shared_ptr<ARDOUR::MIDITrigger> mt = std::dynamic_pointer_cast <ARDOUR::MIDITrigger> (t);
+	std::shared_ptr<ARDOUR::MIDITrigger> mt = std::dynamic_pointer_cast<ARDOUR::MIDITrigger> (t);
 
 	if (!mt || !a) {
 		return;
 	}
 	for (uint8_t c = 0; c < 16; ++c) {
-		if (a->patch_change (c).is_set()) {
+		if (a->patch_change (c).is_set ()) {
 			mt->set_patch_change (a->patch_change (c));
 		} else {
 			mt->unset_patch_change (c);
 		}
 	}
 }
+
+void
+ARDOUR_UI_UTILS::format_position (ARDOUR::Session* s, Temporal::timepos_t const& p, char* buf, size_t bufsize, bool onoff)
+{
+	Temporal::BBT_Time bbt;
+	Timecode::Time     timecode;
+	samplepos_t        pos (p.samples ());
+
+	if (pos < 0) {
+		error << string_compose (_("format_position: negative timecode position: %1"), pos) << endmsg;
+		snprintf (buf, bufsize, "invalid");
+		return;
+	}
+
+	switch (ARDOUR_UI::instance ()->primary_clock->mode ()) {
+		case AudioClock::BBT:
+			bbt = Temporal::TempoMap::use ()->bbt_at (p);
+			if (onoff) {
+				snprintf (buf, bufsize, "%03d|%02d|%04d", bbt.bars, bbt.beats, bbt.ticks);
+			} else {
+				snprintf (buf, bufsize, "(%03d|%02d|%04d)", bbt.bars, bbt.beats, bbt.ticks);
+			}
+			break;
+
+		case AudioClock::MinSec:
+			samplepos_t left;
+			int         hrs;
+			int         mins;
+			float       secs;
+
+			left = pos;
+			hrs  = (int)floor (left / (s->sample_rate () * 60.0f * 60.0f));
+			left -= (samplecnt_t)floor (hrs * s->sample_rate () * 60.0f * 60.0f);
+			mins = (int)floor (left / (s->sample_rate () * 60.0f));
+			left -= (samplecnt_t)floor (mins * s->sample_rate () * 60.0f);
+			secs = left / (float)s->sample_rate ();
+			if (onoff) {
+				snprintf (buf, bufsize, "%02d:%02d:%06.3f", hrs, mins, secs);
+			} else {
+				snprintf (buf, bufsize, "(%02d:%02d:%06.3f)", hrs, mins, secs);
+			}
+			break;
+
+		case AudioClock::Seconds:
+			if (onoff) {
+				snprintf (buf, bufsize, "%.1f", pos / (float)s->sample_rate ());
+			} else {
+				snprintf (buf, bufsize, "(%.1f)", pos / (float)s->sample_rate ());
+			}
+			break;
+
+		case AudioClock::Samples:
+			if (onoff) {
+				snprintf (buf, bufsize, "%" PRId64, pos);
+			} else {
+				snprintf (buf, bufsize, "(%" PRId64 ")", pos);
+			}
+			break;
+
+		case AudioClock::Timecode:
+		default:
+			s->timecode_time (pos, timecode);
+			if (onoff) {
+				snprintf (buf, bufsize, "%02d:%02d:%02d:%02d", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
+			} else {
+				snprintf (buf, bufsize, "(%02d:%02d:%02d:%02d)", timecode.hours, timecode.minutes, timecode.seconds, timecode.frames);
+			}
+			break;
+	}
+}
