@@ -161,7 +161,10 @@ public:
 	 */
 	virtual bool is_realtime () const = 0;
 
-	virtual int client_real_time_priority () { return PBD_RT_PRI_PROC; }
+	/** Return true if the backed is JACK */
+	virtual bool is_jack () const { return false; }
+
+	virtual int client_real_time_priority () { return 0; }
 
 	/* Discovering devices and parameters */
 
@@ -365,12 +368,12 @@ public:
 	/* Returns the default sample rate that will be shown to the user when
 	 * configuration options are first presented. If the derived class
 	 * needs or wants to override this, it can. It also MUST override this
-	 * if there is any chance that an SR of 44.1kHz is not in the list
+	 * if there is any chance that an SR of 48kHz is not in the list
 	 * returned by available_sample_rates()
 	 */
 	virtual float default_sample_rate () const
 	{
-		return 44100.0;
+		return 48000.0;
 	}
 
 	/** Returns a collection of uint32 identifying buffer sizes that are
@@ -406,23 +409,6 @@ public:
 	{
 		return 1024;
 	}
-
-	/** Returns the maximum number of input channels that are potentially
-	 * usable with the hardware identified by \p device . Any number from 1
-	 * to the value returned may be supplied in other calls to this backend as
-	 * the input channel count to use with the name device, but the requested
-	 * count may turn out to be unavailable, or become invalid at any time.
-	 */
-	virtual uint32_t available_input_channel_count (const std::string& device) const = 0;
-
-	/** Returns the maximum number of output channels that are potentially
-	 * usable with the hardware identified by \p device . Any number from 1
-	 * to the value returned may be supplied in other calls to this backend as
-	 * the output channel count to use with the name device, but the requested
-	 * count may turn out to be unavailable, or become invalid at any time.
-	 */
-	virtual uint32_t available_output_channel_count (const std::string& device) const = 0;
-
 	/* Return true if the derived class can change the sample rate of the
 	 * device in use while the device is already being used. Return false
 	 * otherwise. (example: JACK cannot do this as of September 2013)
@@ -511,12 +497,6 @@ public:
 	 */
 	virtual int set_interleaved (bool yn) = 0;
 
-	/** Set the number of input channels that should be used */
-	virtual int set_input_channels (uint32_t) = 0;
-
-	/** Set the number of output channels that should be used */
-	virtual int set_output_channels (uint32_t) = 0;
-
 	/** Set the (additional) input latency that cannot be determined via
 	 * the implementation's underlying code (e.g. latency from
 	 * external D-A/D-A converters. Units are samples.
@@ -555,8 +535,6 @@ public:
 	virtual float    sample_rate () const                                   = 0;
 	virtual uint32_t buffer_size () const                                   = 0;
 	virtual bool     interleaved () const                                   = 0;
-	virtual uint32_t input_channels () const                                = 0;
-	virtual uint32_t output_channels () const                               = 0;
 	virtual uint32_t systemic_input_latency () const                        = 0;
 	virtual uint32_t systemic_output_latency () const                       = 0;
 	virtual uint32_t systemic_midi_input_latency (std::string const) const  = 0;
