@@ -23,7 +23,6 @@
 #include <string>
 
 #include <lo/lo.h>
-#include <boost/enable_shared_from_this.hpp>
 #include <rubberband/RubberBandStretcher.h>
 #include <vamp-hostsdk/Plugin.h>
 
@@ -169,11 +168,10 @@ namespace ARDOUR { namespace LuaAPI {
 
 	/** get a plugin property (LV2 plugins only)
 	 *
-	 * @param p two arguments: Plugin-Inster, URI of the property
-	 * @param value the value to set (boolean, integer, float, string/path)
+	 * @param p two arguments: Plugin-Insert, URI of the property
 	 * @returns value, depending on datatype or nil if property is not found
 	 */
-	int get_plugin_insert_property (lua_State *lua);
+	int get_plugin_insert_property (lua_State *p);
 
 	/**
 	 * A convenience function to get a Automation Lists and ParameterDescriptor
@@ -431,6 +429,13 @@ namespace ARDOUR { namespace LuaAPI {
 	std::list<std::shared_ptr< Evoral::Note<Temporal::Beats> > >
 		note_list (std::shared_ptr<ARDOUR::MidiModel>);
 
+	std::list<std::shared_ptr< Evoral::Event<Temporal::Beats> > >
+		sysex_list (std::shared_ptr<ARDOUR::MidiModel>);
+
+	std::list<std::shared_ptr< Evoral::PatchChange<Temporal::Beats> > >
+		patch_change_list (std::shared_ptr<ARDOUR::MidiModel>);
+
+
 } } /* namespace */
 
 namespace ARDOUR { namespace LuaOSC {
