@@ -37,7 +37,6 @@
 #include <sys/stat.h>
 
 #include <boost/optional.hpp>
-#include <boost/utility.hpp>
 
 #include <glib.h>
 
@@ -88,7 +87,7 @@ private:
 	Playlist& _playlist;
 };
 
-class LIBARDOUR_API Playlist : public SessionObject, public std::enable_shared_from_this<Playlist>
+class LIBARDOUR_API Playlist : public SessionObject, public Temporal::TimeDomainProvider, public std::enable_shared_from_this<Playlist>
 {
 public:
 	static void make_property_quarks ();
@@ -102,7 +101,7 @@ public:
 
 	void update (const RegionListProperty::ChangeRecord&);
 	void clear_owned_changes ();
-	void rdiff (std::vector<Command*>&) const;
+	void rdiff (std::vector<PBD::Command*>&) const;
 
 	void rdiff_and_add_command (Session*);
 
@@ -113,8 +112,6 @@ public:
 	bool set_name (const std::string& str);
 	void set_region_ownership ();
 
-	Temporal::TimeDomain time_domain() const;
-
 	/*playlist group IDs (pgroup_id) is a group identifier that is implicitly
 	 * or explicitly assigned to playlists so they can be associated with each other.
 	 *
@@ -179,7 +176,7 @@ public:
 	void replace_region (std::shared_ptr<Region> old, std::shared_ptr<Region> newr, timepos_t const & pos);
 	void split_region (std::shared_ptr<Region>, timepos_t const & position);
 	void split (timepos_t const & at);
-	void shift (timepos_t const & at, timecnt_t const & distance, bool move_intersected, bool ignore_music_glue);
+	void shift (timepos_t const & at, timecnt_t const & distance, bool move_intersected);
 	void partition (timepos_t const & start, timepos_t const & end, bool cut = false);
 	void duplicate (std::shared_ptr<Region>, timepos_t & position, float times);
 	void duplicate (std::shared_ptr<Region>, timepos_t & position, timecnt_t const & gap, float times);
@@ -302,7 +299,9 @@ public:
 
 	void set_capture_insertion_in_progress (bool yn);
 
-	void globally_change_time_domain (Temporal::TimeDomain from, Temporal::TimeDomain to);
+	void time_domain_changed ();
+	void start_domain_bounce (Temporal::DomainBounceInfo&);
+	void finish_domain_bounce (Temporal::DomainBounceInfo&);
 
 protected:
 	friend class Session;
@@ -402,7 +401,7 @@ protected:
 
 	void _set_sort_id ();
 
-	std::shared_ptr<RegionList> regions_touched_locked (timepos_t const & start, timepos_t const & end);
+	std::shared_ptr<RegionList> regions_touched_locked (timepos_t const & start, timepos_t const & end, bool with_tail);
 
 	void notify_region_removed (std::shared_ptr<Region>);
 	void notify_region_added (std::shared_ptr<Region>);
@@ -435,7 +434,7 @@ protected:
 	int  remove_region_internal (std::shared_ptr<Region>, ThawList& thawlist);
 	void copy_regions (RegionList&) const;
 
-	void partition_internal (timepos_t const & start, timepos_t const & end, bool cutting, ThawList& thawlist);
+	void partition_internal (timepos_t start, timepos_t end, bool cutting, ThawList& thawlist);
 
 	std::pair<timepos_t, timepos_t> _get_extent() const;
 
