@@ -40,6 +40,7 @@
 #include "ardour/midi_ring_buffer.h"
 #include "ardour/midi_state_tracker.h"
 #include "ardour/parameter_descriptor.h"
+#include "ardour/tailtime.h"
 #include "ardour/types.h"
 #include "ardour/variant.h"
 
@@ -53,9 +54,11 @@ class AudioEngine;
 class Session;
 class BufferSet;
 class IOPlug;
+class PlugInsertBase;
 class PluginInsert;
 class Plugin;
 class PluginInfo;
+class RegionFxPlugin;
 class AutomationControl;
 class SessionObject;
 
@@ -72,7 +75,7 @@ typedef std::set<uint32_t>            PluginOutputConfiguration;
  *
  * Plugins are not used directly in Ardour but always wrapped by a PluginInsert.
  */
-class LIBARDOUR_API Plugin : public PBD::StatefulDestructible, public HasLatency
+class LIBARDOUR_API Plugin : public PBD::StatefulDestructible, public HasLatency, public HasTailTime
 {
 public:
 	Plugin (ARDOUR::AudioEngine&, ARDOUR::Session&);
@@ -85,12 +88,12 @@ public:
 	virtual void set_insert_id (PBD::ID id) {}
 	virtual void set_state_dir (const std::string& d = "") {}
 
-	void set_insert (PluginInsert* pi, uint32_t num) {
-		_pi = pi;
+	void set_insert (PlugInsertBase* pib, uint32_t num) {
+		_pib = pib;
 		_num = num;
 	}
 
-	PluginInsert* plugin_insert () const { return _pi; }
+	PlugInsertBase* plugin_insert () const { return _pib; }
 	uint32_t plugin_number () const { return _num; }
 
 	virtual std::string unique_id () const                   = 0;
@@ -170,6 +173,11 @@ public:
 		return plugin_latency ();
 	}
 
+	samplecnt_t signal_tailtime () const
+	{
+		return plugin_tailtime ();
+	}
+
 	/** the max possible latency a plugin will have */
 	virtual samplecnt_t max_latency () const { return 0; }
 
@@ -190,6 +198,9 @@ public:
 	void realtime_locate (bool);
 	void monitoring_changed ();
 
+	/* use plugin for offline processing */
+	virtual void set_non_realtime (bool) {}
+
 	virtual void add_slave (std::shared_ptr<Plugin>, bool realtime) {}
 	virtual void remove_slave (std::shared_ptr<Plugin>) {}
 
@@ -373,8 +384,9 @@ public:
 	PBD::Signal1<void, uint32_t> EndTouch;
 
 protected:
-	friend class IOPlug;
 	friend class PluginInsert;
+	friend class PlugInsertBase;
+	friend class RegionFxPlugin;
 	friend class Session;
 
 	/* Called when a parameter of the plugin is changed outside of this
@@ -417,6 +429,11 @@ protected:
 
 private:
 	virtual samplecnt_t plugin_latency () const = 0;
+	/** tail duration in samples. e.g. for reverb or delay plugins.
+	 *
+	 * The default when unknown is 2 sec */
+	virtual samplecnt_t plugin_tailtime () const;
+
 
 	/** Fill _presets with our presets */
 	virtual void find_presets () = 0;
@@ -437,8 +454,8 @@ private:
 	void invalidate_preset_cache (std::string const&, Plugin*, bool);
 	void resolve_midi ();
 
-	PluginInsert* _pi;
-	uint32_t      _num;
+	PlugInsertBase* _pib;
+	uint32_t        _num;
 
 	PBD::ScopedConnection _preset_connection;
 };
