@@ -49,8 +49,6 @@
 
 #include <boost/dynamic_bitset.hpp>
 #include <boost/scoped_ptr.hpp>
-#include <boost/utility.hpp>
-
 #include <glibmm/threads.h>
 
 #include <ltc.h>
@@ -60,11 +58,15 @@
 #include "pbd/event_loop.h"
 #include "pbd/file_archive.h"
 #include "pbd/rcu.h"
-#include "pbd/reallocpool.h"
 #include "pbd/statefuldestructible.h"
 #include "pbd/signals.h"
 #include "pbd/undo.h"
 
+#ifdef USE_TLSF
+#  include "pbd/tlsf.h"
+#else
+#  include "pbd/reallocpool.h"
+#endif
 #include "lua/luastate.h"
 
 #include "temporal/range.h"
@@ -86,7 +88,6 @@
 #include "ardour/rc_configuration.h"
 #include "ardour/session_configuration.h"
 #include "ardour/session_event.h"
-#include "ardour/interpolation.h"
 #include "ardour/plugin.h"
 #include "ardour/presentation_info.h"
 #include "ardour/route.h"
@@ -149,6 +150,7 @@ struct GraphChain;
 class IO;
 class IOPlug;
 class IOProcessor;
+class IOTaskList;
 class ImportStatus;
 class MidiClockTicker;
 class MidiControlUI;
@@ -257,6 +259,7 @@ public:
 	bool cannot_save () const                      { return _state_of_the_state & CannotSave; }
 	bool in_cleanup () const                       { return _state_of_the_state & InCleanup; }
 	bool inital_connect_or_deletion_in_progress () const { return _state_of_the_state & (InitialConnecting | Deletion); }
+	bool have_external_connections_for_current_backend (bool tracks_only = true) const;
 	bool unnamed() const;
 
 	void end_unnamed_status () const;
@@ -296,7 +299,7 @@ public:
 	RouteList new_route_from_template (uint32_t how_many, PresentationInfo::order_t insert_at, const std::string& template_path, const std::string& name, PlaylistDisposition pd = NewPlaylist);
 	RouteList new_route_from_template (uint32_t how_many, PresentationInfo::order_t insert_at, XMLNode&, const std::string& name, PlaylistDisposition pd = NewPlaylist);
 	std::vector<std::string> get_paths_for_new_sources (bool allow_replacing, const std::string& import_file_path,
-	                                                    uint32_t channels, std::vector<std::string> const & smf_track_names);
+	                                                    uint32_t channels, std::vector<std::string> const & smf_track_names, bool use_smf_file_names);
 
 	int bring_all_sources_into_session (boost::function<void(uint32_t,uint32_t,std::string)> callback);
 
@@ -311,6 +314,8 @@ public:
 	BufferSet& get_route_buffers (ChanCount count = ChanCount::ZERO, bool silence = true);
 	BufferSet& get_mix_buffers (ChanCount count = ChanCount::ZERO);
 
+	void ensure_buffers_unlocked (ChanCount howmany);
+
 	bool have_rec_enabled_track () const;
 	bool have_rec_disabled_track () const;
 
@@ -332,6 +337,7 @@ public:
 	}
 
 	std::shared_ptr<RTTaskList> rt_tasklist () { return _rt_tasklist; }
+	std::shared_ptr<IOTaskList> io_tasklist () { return _io_tasklist; }
 
 	RouteList get_routelist (bool mixer_order = false, PresentationInfo::Flag fl = PresentationInfo::MixerRoutes) const;
 
@@ -805,7 +811,7 @@ public:
 
 	double engine_speed() const { return _engine_speed; }
 	double actual_speed() const;
-	double transport_speed() const;
+	double transport_speed (bool incl_preroll = false) const;
 	/** @return true if the transport state (TFSM) is stopped */
 	bool   transport_stopped() const;
 	/** @return true if the transport state (TFSM) is stopped or stopping */
@@ -1195,8 +1201,6 @@ public:
 	gain_t* scratch_automation_buffer () const;
 	pan_t** pan_automation_buffer () const;
 
-	void ensure_buffer_set (BufferSet& buffers, const ChanCount& howmany);
-
 	/* VST support */
 
 	static int  vst_current_loading_id;
@@ -1515,6 +1519,8 @@ private:
 	void setup_engine_resampling ();
 
 	void ensure_buffers (ChanCount howmany = ChanCount::ZERO);
+	ChanCount _required_thread_buffers;
+	size_t    _required_thread_buffersize;
 
 	void process_without_events (pframes_t);
 	void process_with_events    (pframes_t);
@@ -1627,10 +1633,11 @@ private:
 	XMLTree*         state_tree;
 	StateOfTheState _state_of_the_state;
 
-	friend class    StateProtector;
-	std::atomic<int>  _suspend_save;
-	volatile bool      _save_queued;
-	volatile bool      _save_queued_pending;
+	friend class     StateProtector;
+	std::atomic<int> _suspend_save;
+	volatile bool    _save_queued;
+	volatile bool    _save_queued_pending;
+	bool             _no_save_signal;
 
 	Glib::Threads::Mutex save_state_lock;
 	Glib::Threads::Mutex save_source_lock;
@@ -1648,7 +1655,11 @@ private:
 	bool              pending_abort;
 	bool              pending_auto_loop;
 
+#ifdef USE_TLSF
+	PBD::TLSF _mempool;
+#else
 	PBD::ReallocPool _mempool;
+#endif
 	LuaState lua;
 	mutable Glib::Threads::Mutex lua_lock;
 	luabridge::LuaRef * _lua_run;
@@ -1661,6 +1672,7 @@ private:
 	uint32_t            _n_lua_scripts;
 
 	void setup_lua ();
+	void luabindings_session_rt (lua_State*);
 	void try_run_lua (pframes_t);
 
 	SerializedRCUManager<IOPlugList> _io_plugins;
@@ -2108,7 +2120,7 @@ private:
 
 	std::vector<space_and_path> session_dirs;
 	std::vector<space_and_path>::iterator last_rr_session_dir;
-	uint32_t _total_free_4k_blocks;
+	uint64_t _total_free_4k_blocks;
 	/** If this is true, _total_free_4k_blocks is not definite,
 	    as one or more of the session directories' filesystems
 	    could not report free space.
@@ -2341,6 +2353,7 @@ private:
 	std::shared_ptr<Port>  _ltc_output_port;
 
 	std::shared_ptr<RTTaskList> _rt_tasklist;
+	std::shared_ptr<IOTaskList> _io_tasklist;
 
 	/* Scene Changing */
 	SceneChanger* _scene_changer;
@@ -2368,6 +2381,7 @@ private:
 	std::shared_ptr<Route> get_midi_nth_route_by_id (PresentationInfo::order_t n) const;
 
 	std::string created_with;
+	mutable std::string modified_with;
 
 	void midi_track_presentation_info_changed (PBD::PropertyChange const &, std::weak_ptr<MidiTrack>);
 	void rewire_selected_midi (std::shared_ptr<MidiTrack>);
