@@ -26,8 +26,6 @@
 #include <memory>
 #include <string>
 
-#include <boost/utility.hpp>
-
 #include "pbd/signals.h"
 
 #include "ardour/automatable.h"
@@ -53,6 +51,9 @@ class MonitorProcessor;
 class RecordEnableControl;
 class RecordSafeControl;
 
+enum WellKnownCtrl : int;
+enum WellKnownData : int;
+
 /* This is a virtual base class for any object that needs to be potentially
  * represented by a control-centric user interface using the general model of a
  * mixing console "strip" - a collection of controls that determine the state
@@ -76,6 +77,9 @@ class LIBARDOUR_API Stripable : public SessionObject,
 	bool is_master() const { return _presentation_info.flags() & PresentationInfo::MasterOut; }
 	bool is_monitor() const { return _presentation_info.flags() & PresentationInfo::MonitorOut; }
 	bool is_foldbackbus() const { return _presentation_info.flags() & PresentationInfo::FoldbackBus; }
+	bool is_surround_master() const { return _presentation_info.flags() & PresentationInfo::SurroundMaster; }
+	bool is_main_bus() const { return _presentation_info.flags() & PresentationInfo::MainBus; }
+	bool is_singleton () const { return _presentation_info.flags() & PresentationInfo::Singleton; }
 
 	int set_state (XMLNode const&, int);
 
@@ -135,60 +139,17 @@ class LIBARDOUR_API Stripable : public SessionObject,
 	virtual std::shared_ptr<AutomationControl> pan_frontback_control() const = 0;
 	virtual std::shared_ptr<AutomationControl> pan_lfe_control() const = 0;
 
-	/* "well-known" controls for an EQ in this route. Any or all may
-	 * be null. eq_band_cnt() must return 0 if there is no EQ present.
-	 * Passing an @p band value >= eq_band_cnt() will guarantee the
-	 * return of a null ptr (or an empty string for eq_band_name()).
-	 */
+	/* "well-known" controls. Any or all may NULL. */
 	virtual uint32_t eq_band_cnt () const = 0;
 	virtual std::string eq_band_name (uint32_t) const = 0;
-	virtual std::shared_ptr<AutomationControl> eq_enable_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> eq_gain_controllable (uint32_t band) const = 0;
-	virtual std::shared_ptr<AutomationControl> eq_freq_controllable (uint32_t band) const = 0;
-	virtual std::shared_ptr<AutomationControl> eq_q_controllable (uint32_t band) const = 0;
-	virtual std::shared_ptr<AutomationControl> eq_shape_controllable (uint32_t band) const = 0;
 
-	virtual std::shared_ptr<AutomationControl> filter_freq_controllable (bool hp /* false for LPF*/) const = 0;
-	virtual std::shared_ptr<AutomationControl> filter_slope_controllable (bool hp) const = 0;
-	virtual std::shared_ptr<AutomationControl> filter_enable_controllable (bool hp) const = 0;
+	virtual std::shared_ptr<AutomationControl> mapped_control (enum WellKnownCtrl, uint32_t band = 0) const = 0;
+	virtual std::shared_ptr<ReadOnlyControl>   mapped_output (enum WellKnownData) const = 0;
 
-	virtual std::shared_ptr<AutomationControl> tape_drive_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> tape_drive_mode_controllable () const = 0;
-	virtual std::shared_ptr<ReadOnlyControl> tape_drive_mtr_controllable () const = 0;
-
-	/* "well-known" controls for a compressor in this route. Any or all may
-	 * be null.
+	/* ACs mapped to any control have changed. API user is to drop references,
+	 * and query mapped ctrl again
 	 */
-
-	virtual std::shared_ptr<AutomationControl> comp_enable_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_threshold_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_speed_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_mode_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_makeup_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_ratio_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_attack_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_release_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_key_filter_freq_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> comp_lookahead_controllable () const = 0;
-	virtual std::shared_ptr<ReadOnlyControl>   comp_meter_controllable () const = 0;
-	virtual std::shared_ptr<ReadOnlyControl>   comp_redux_controllable () const = 0;
-
-	virtual std::shared_ptr<AutomationControl> gate_enable_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_mode_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_ratio_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_knee_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_threshold_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_depth_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_hysteresis_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_hold_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_attack_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_release_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_key_listen_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_key_filter_enable_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_key_filter_freq_controllable () const = 0;
-	virtual std::shared_ptr<AutomationControl> gate_lookahead_controllable () const = 0;
-	virtual std::shared_ptr<ReadOnlyControl>   gate_meter_controllable () const = 0;
-	virtual std::shared_ptr<ReadOnlyControl>   gate_redux_controllable () const = 0;
+	PBD::Signal0<void> MappedControlsChanged;
 
 	/* "well-known" controls for sends to well-known busses in this route. Any or all may
 	 * be null.
@@ -197,7 +158,7 @@ class LIBARDOUR_API Stripable : public SessionObject,
 	 * In Ardour, these are user-created sends that connect to user-created
 	 * Aux busses.
 	 */
-	virtual std::shared_ptr<AutomationControl> send_level_controllable (uint32_t n) const = 0;
+	virtual std::shared_ptr<AutomationControl> send_level_controllable (uint32_t n, bool locked = false) const = 0;
 	virtual std::shared_ptr<AutomationControl> send_enable_controllable (uint32_t n) const = 0;
 	virtual std::shared_ptr<AutomationControl> send_pan_azimuth_controllable (uint32_t n) const = 0;
 	virtual std::shared_ptr<AutomationControl> send_pan_azimuth_enable_controllable (uint32_t n) const = 0;
@@ -215,31 +176,6 @@ class LIBARDOUR_API Stripable : public SessionObject,
 	 */
 	virtual std::shared_ptr<AutomationControl> master_send_enable_controllable () const = 0;
 
-	/* well known control for mixbus's correlation meter.
-	 *
-	 * In Ardour, this returns null.
-	 * In Mixbus, it will return a suitable control, or null depending on the route.
-	 * @param mm min/max of the correlation range, true for upper value
-	 */
-	virtual std::shared_ptr<ReadOnlyControl> master_correlation_mtr_controllable (bool mm) const { return std::shared_ptr<ReadOnlyControl>(); }
-
-	/* well known control for mixbus's limiter.
-	 *
-	 * In Ardour, this returns null.
-	 * In Mixbus, it will return a suitable control, or null depending on
-	 * the route.
-	 */
-	virtual std::shared_ptr<AutomationControl> master_limiter_enable_controllable () const { return std::shared_ptr<AutomationControl>(); }
-	virtual std::shared_ptr<ReadOnlyControl> master_limiter_mtr_controllable () const { return std::shared_ptr<ReadOnlyControl>(); }
-
-	/* well known control for mixbus's k-meter.
-	 *
-	 * In Ardour, this returns null.
-	 * In Mixbus, it will return a suitable control, or null depending on
-	 * the route.
-	 */
-	virtual std::shared_ptr<ReadOnlyControl> master_k_mtr_controllable () const { return std::shared_ptr<ReadOnlyControl>(); }
-
 	virtual bool muted_by_others_soloing () const = 0;
 
 	virtual std::shared_ptr<MonitorProcessor> monitor_control() const = 0;
