@@ -176,7 +176,14 @@ ARDOUR::timecnt_t
 AudioPlaylist::read (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, timepos_t const & start, timecnt_t const & cnt, uint32_t chan_n)
 {
 	DEBUG_TRACE (DEBUG::AudioPlayback, string_compose ("Playlist %1 read @ %2 for %3, channel %4, regions %5 mixdown @ %6 gain @ %7\n",
-							   name(), start, cnt, chan_n, regions.size(), mixdown_buffer, gain_buffer));
+							   name(), start.samples(), cnt.samples(), chan_n, regions.size(), mixdown_buffer, gain_buffer));
+
+	DEBUG_TRACE (DEBUG::AudioCacheRefill, string_compose ("Playlist '%1' chn: %2 from %3 to %4 [s] PH@ %5\n",
+				name (), chan_n,
+				std::setprecision (3), std::fixed,
+				start.samples() / (float)_session.sample_rate (),
+				(start.samples() + cnt.samples()) / (float)_session.sample_rate (),
+				_session.transport_sample () / (float)_session.sample_rate ()));
 
 	samplecnt_t const scnt (cnt.samples ());
 
@@ -204,7 +211,7 @@ AudioPlaylist::read (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, ti
 	/* Find all the regions that are involved in the bit we are reading,
 	   and sort them by descending layer and ascending position.
 	*/
-	std::shared_ptr<RegionList> all = regions_touched_locked (start, start + cnt);
+	std::shared_ptr<RegionList> all = regions_touched_locked (start, start + cnt, true);
 	all->sort (ReadSorter ());
 
 	/* This will be a list of the bits of our read range that we have
@@ -236,7 +243,7 @@ AudioPlaylist::read (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, ti
 		*/
 		Temporal::Range rrange = ar->range_samples ();
 		Temporal::Range region_range (max (rrange.start(), start),
-		                              min (rrange.end(), start + cnt));
+		                              min (rrange.end() + ar->tail (), start + cnt));
 
 		/* ... and then remove the bits that are already done */
 
@@ -256,9 +263,9 @@ AudioPlaylist::read (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, ti
 				/* Cut this range down to just the body and mark it done */
 				Temporal::Range body = ar->body_range ();
 
-				if (body.start() < d.end() && body.end() > d.start()) {
+				if (body.start() < d.end().earlier (ar->tail ()) && body.end() > d.start()) {
 					d.set_start (max (d.start(), body.start()));
-					d.set_end (min (d.end(), body.end()));
+					d.set_end (min (d.end().earlier (ar->tail ()), body.end()));
 					done.add (d);
 				}
 			}
@@ -275,10 +282,23 @@ AudioPlaylist::read (Sample *buf, Sample *mixdown_buffer, float *gain_buffer, ti
 
 		samplepos_t read_pos (i->range.start().samples());
 		samplecnt_t read_cnt (i->range.start().distance (i->range.end()).samples());
-		assert (start.distance (i->range.start()).samples() < scnt);
-		assert (start.distance (i->range.start()).samples() + read_cnt <= scnt);
-		samplecnt_t nread = i->region->read_at (buf + start.distance (i->range.start()).samples(), mixdown_buffer, gain_buffer, read_pos, read_cnt, chan_n);
+		samplecnt_t soffset = start.distance (i->range.start()).samples();
+
+		assert (soffset < scnt);
+
+		if (soffset + read_cnt > scnt) {
+			read_cnt = scnt - soffset;
+		}
+		assert (soffset + read_cnt <= scnt);
+		samplecnt_t nread = i->region->read_at (buf + soffset, mixdown_buffer, gain_buffer, read_pos, read_cnt, chan_n);
 		if (nread != read_cnt) {
+			std::cerr << name() << " tried to read " << read_cnt
+				<< " got " << nread
+				<< " in " << i->region->name()
+				<< " for chn " << chan_n
+				<< " to offset " << soffset
+				<< " using range " << i->range.start().samples() << " .. " << i->range.end().samples()
+				<< " len " << i->range.length().samples() << std::endl;
 #ifndef NDEBUG
 			/* forward error to DiskReader::audio_read. This does 2 things:
 			 *  - error "DiskReader %1: when refilling, cannot read ..."
