@@ -360,6 +360,7 @@ AUPlugin::AUPlugin (AudioEngine& engine, Session& session, std::shared_ptr<CACom
 	, comp (_comp)
 	, unit (new CAAudioUnit)
 	, initialized (false)
+	, process_offline (false)
 	, _last_nframes (0)
 	, _requires_fixed_size_buffers (false)
 	, buffers (0)
@@ -401,6 +402,7 @@ AUPlugin::AUPlugin (const AUPlugin& other)
 	, comp (other.get_comp())
 	, unit (new CAAudioUnit)
 	, initialized (false)
+	, process_offline (false)
 	, _last_nframes (0)
 	, _requires_fixed_size_buffers (false)
 	, buffers (0)
@@ -869,6 +871,33 @@ AUPlugin::requires_fixed_size_buffers() const
 	return _requires_fixed_size_buffers;
 }
 
+void
+AUPlugin::set_non_realtime (bool yn)
+{
+	if (process_offline == yn) {
+		return;
+	}
+	process_offline = yn;
+
+	bool was_initialized = initialized;
+	if (initialized) {
+		deactivate ();
+	}
+
+	OSErr err;
+	UInt32 isOffline = yn ? 1 : 0;
+	if ((err = unit->SetProperty (/*kAudioUnitProperty_OfflineRender*/ 37, kAudioUnitScope_Global, 0, &isOffline, sizeof (isOffline))) != noErr) {
+		info << string_compose (_("AU: cannot set offline rendering(err = %1)"), err) << endmsg;
+	}
+	if (yn) {
+		UInt32 numSamples = _session.get_block_size();
+		unit->SetProperty (/*kAudioUnitOfflineProperty_InputSize*/ 3020, kAudioUnitScope_Global, 0, &numSamples, sizeof(numSamples));
+	}
+
+	if (was_initialized) {
+		activate ();
+	}
+}
 
 int
 AUPlugin::set_block_size (pframes_t nframes)
@@ -888,6 +917,10 @@ AUPlugin::set_block_size (pframes_t nframes)
 		return -1;
 	}
 
+	if (process_offline) {
+		unit->SetProperty (/*kAudioUnitOfflineProperty_InputSize*/ 3020, kAudioUnitScope_Global, 0, &numSamples, sizeof(numSamples));
+	}
+
 	if (was_initialized) {
 		activate ();
 	}
@@ -1417,7 +1450,7 @@ AUPlugin::render_callback(AudioUnitRenderActionFlags*,
 		if (valid) {
 			ioData->mBuffers[i].mData = input_buffers->get_audio (idx).data (cb_offsets[bus] + input_offset);
 		} else {
-			ioData->mBuffers[i].mData = silent_bufs.get_audio(0).data (cb_offsets[bus] + input_offset);
+			ioData->mBuffers[i].mData = silent_bufs.get_audio(0).data (0);
 		}
 	}
 	cb_offsets[bus] += inNumberSamples;
@@ -1489,10 +1522,10 @@ AUPlugin::connect_and_run (BufferSet& bufs,
 			MidiBuffer& m = bufs.get_midi (i);
 			for (MidiBuffer::iterator i = m.begin(); i != m.end(); ++i) {
 				Evoral::Event<samplepos_t> ev (*i);
-				if (ev.is_channel_event()) {
+				if (ev.is_channel_event () && ev.time() >= offset && ev.time() < offset + nframes) {
 					const uint8_t* b = ev.buffer();
 					DEBUG_TRACE (DEBUG::AudioUnitProcess, string_compose ("%1: MIDI event %2\n", name(), ev));
-					unit->MIDIEvent (b[0], b[1], b[2], ev.time());
+					unit->MIDIEvent (b[0], b[1], b[2], ev.time() - offset);
 				}
 				/* XXX need to handle sysex and other message types */
 			}
@@ -1536,7 +1569,7 @@ AUPlugin::connect_and_run (BufferSet& bufs,
 				if (valid) {
 					buffers->mBuffers[i].mData = bufs.get_audio (idx).data (offset);
 				} else {
-					buffers->mBuffers[i].mData = scratch_bufs.get_audio(0).data(offset);
+					buffers->mBuffers[i].mData = scratch_bufs.get_audio(0).data(0);
 				}
 			}
 		}
@@ -1813,9 +1846,9 @@ AUPlugin::describe_io_port (ARDOUR::DataType dt, bool input, uint32_t id) const
 	}
 
 	if (input) {
-		ss << " " << _("In") << " ";
+		ss << " " << S_("IO|In") << " ";
 	} else {
-		ss << " " << _("Out") << " ";
+		ss << " " << S_("IO|Out") << " ";
 	}
 
 	ss << (id + 1);
