@@ -678,7 +678,6 @@ void
 AudioEngine::do_reset_backend()
 {
 	SessionEvent::create_per_thread_pool (X_("Backend reset processing thread"), 1024);
-	pthread_set_name ("EngineWatchdog");
 
 	Glib::Threads::Mutex::Lock guard (_reset_request_lock);
 
@@ -739,7 +738,6 @@ void
 AudioEngine::do_devicelist_update()
 {
 	SessionEvent::create_per_thread_pool (X_("Device list update processing thread"), 512);
-	pthread_set_name ("DeviceList");
 
 	Glib::Threads::Mutex::Lock guard (_devicelist_update_lock);
 
@@ -769,13 +767,13 @@ AudioEngine::start_hw_event_processing()
 	if (_hw_reset_event_thread == 0) {
 		_hw_reset_request_count.store (0);
 		_stop_hw_reset_processing.store (0);
-		_hw_reset_event_thread = PBD::Thread::create (boost::bind (&AudioEngine::do_reset_backend, this));
+		_hw_reset_event_thread = PBD::Thread::create (boost::bind (&AudioEngine::do_reset_backend, this), "EngineWatchdog");
 	}
 
 	if (_hw_devicelist_update_thread == 0) {
 		_hw_devicelist_update_count.store (0);
 		_stop_hw_devicelist_processing.store (0);
-		_hw_devicelist_update_thread = PBD::Thread::create (boost::bind (&AudioEngine::do_devicelist_update, this));
+		_hw_devicelist_update_thread = PBD::Thread::create (boost::bind (&AudioEngine::do_devicelist_update, this), "DeviceList");
 	}
 }
 
@@ -834,7 +832,7 @@ AudioEngine::remove_session ()
 		SessionHandlePtr::set_session (0);
 	}
 
-	remove_all_ports ();
+	remove_session_ports ();
 }
 
 void
@@ -986,6 +984,12 @@ AudioEngine::current_backend_name() const
 	return string();
 }
 
+bool
+AudioEngine::is_jack() const
+{
+	return _backend && _backend->is_jack();
+}
+
 void
 AudioEngine::drop_backend ()
 {
@@ -1063,6 +1067,12 @@ AudioEngine::start (bool for_latency)
 		return -1;
 	}
 
+	if (_backend->is_realtime ()) {
+		pbd_set_engine_rt_priority (_backend->client_real_time_priority ());
+	} else {
+		pbd_set_engine_rt_priority (0);
+	}
+
 	_running = true;
 
 	if (_session) {
@@ -1186,38 +1196,6 @@ AudioEngine::get_dsp_load() const
 	return _backend->dsp_load ();
 }
 
-bool
-AudioEngine::is_realtime() const
-{
-	if (!_backend) {
-		return false;
-	}
-
-	return _backend->is_realtime();
-}
-
-int
-AudioEngine::client_real_time_priority ()
-{
-	if (!_backend) {
-		assert (0);
-		return PBD_RT_PRI_PROC;
-	}
-	if (!_backend->is_realtime ()) {
-		/* this is only an issue with the Dummy backend.
-		 * - with JACK, we require rt permissions.
-		 * - with ALSA/PulseAudio this can only happen if rt permissions
-		 *   are n/a. Other attempts to get rt will fail likewise.
-		 *
-		 * perhaps:
-		 * TODO: use is_realtime () ? PBD_SCHED_FIFO : PBD_SCHED_OTHER
-		 */
-		return PBD_RT_PRI_PROC; // XXX
-	}
-
-	return _backend->client_real_time_priority();
-}
-
 void
 AudioEngine::transport_start ()
 {
@@ -1411,24 +1389,6 @@ AudioEngine::set_interleaved (bool yn)
 	return _backend->set_interleaved  (yn);
 }
 
-int
-AudioEngine::set_input_channels (uint32_t ic)
-{
-	if (!_backend) {
-		return -1;
-	}
-	return _backend->set_input_channels  (ic);
-}
-
-int
-AudioEngine::set_output_channels (uint32_t oc)
-{
-	if (!_backend) {
-		return -1;
-	}
-	return _backend->set_output_channels (oc);
-}
-
 int
 AudioEngine::set_systemic_input_latency (uint32_t il)
 {
