@@ -32,6 +32,7 @@
 #include "ardour/audiofile_tagger.h"
 #include "ardour/audio_port.h"
 #include "ardour/debug.h"
+#include "ardour/disk_reader.h"
 #include "ardour/export_graph_builder.h"
 #include "ardour/export_handler.h"
 #include "ardour/export_timespan.h"
@@ -120,14 +121,69 @@ ExportHandler::ExportHandler (Session & session)
   , cue_tracknum (0)
   , cue_indexnum (0)
 {
+	pthread_mutex_init (&_timespan_mutex, 0);
+	pthread_cond_init (&_timespan_cond, 0);
+	_timespan_thread_active.store (1);
+	_timespan_thread = PBD::Thread::create (boost::bind (_timespan_thread_run, this), "ExportHandler");
+	if (!_timespan_thread) {
+		_timespan_thread_active.store (0);
+		fatal << "Cannot create export handler helper thread" << endmsg;
+		abort(); /* NOTREACHED*/
+	}
 }
 
 ExportHandler::~ExportHandler ()
 {
 	if (export_status->aborted () && !current_timespan->vapor ().empty () && session.surround_master ()) {
+		Glib::Threads::Mutex::Lock l (export_status->lock());
 		session.surround_master ()->surround_return ()->finalize_export ();
 	}
 	graph_builder->cleanup (export_status->aborted () );
+
+	pthread_mutex_lock (&_timespan_mutex);
+	_timespan_thread_active.store (0);
+	pthread_cond_signal (&_timespan_cond);
+	pthread_mutex_unlock (&_timespan_mutex);
+
+	_timespan_thread->join ();
+
+	pthread_cond_destroy (&_timespan_cond);
+	pthread_mutex_destroy (&_timespan_mutex);
+}
+
+void*
+ExportHandler::_timespan_thread_run (void* me)
+{
+	ExportHandler* self = static_cast<ExportHandler*> (me);
+
+	SessionEvent::create_per_thread_pool ("ExportHandler", 512);
+	PBD::notify_event_loops_about_thread_creation (pthread_self(), "ExportHandler", 512);
+
+	pthread_mutex_lock (&self->_timespan_mutex);
+	while (self->_timespan_thread_active.load ()) {
+		pthread_cond_wait (&self->_timespan_cond, &self->_timespan_mutex);
+		if (!self->_timespan_thread_active.load ()) {
+			break;
+		} else {
+			Temporal::TempoMap::fetch ();
+			self->process_connection.disconnect ();
+			Glib::Threads::Mutex::Lock l (self->export_status->lock());
+			DiskReader::allocate_working_buffers ();
+			self->start_timespan ();
+			DiskReader::free_working_buffers ();
+		}
+	}
+	pthread_mutex_unlock (&self->_timespan_mutex);
+	return 0;
+}
+
+void
+ExportHandler::timespan_thread_wakeup ()
+{
+	if (pthread_mutex_trylock (&_timespan_mutex) == 0) {
+		pthread_cond_signal (&_timespan_cond);
+		pthread_mutex_unlock (&_timespan_mutex);
+	}
 }
 
 /** Add an export to the `to-do' list */
@@ -370,19 +426,6 @@ ExportHandler::command_output(std::string output, size_t size)
 	info << output << endmsg;
 }
 
-void*
-ExportHandler::start_timespan_bg (void* eh)
-{
-	char name[64];
-	snprintf (name, 64, "Export-TS-%p", (void*)DEBUG_THREAD_SELF);
-	pthread_set_name (name);
-	ExportHandler* self = static_cast<ExportHandler*> (eh);
-	self->process_connection.disconnect ();
-	Glib::Threads::Mutex::Lock l (self->export_status->lock());
-	self->start_timespan ();
-	return 0;
-}
-
 void
 ExportHandler::finish_timespan ()
 {
@@ -446,17 +489,8 @@ ExportHandler::finish_timespan ()
 		if (!fmt->command().empty()) {
 			SessionMetadata const & metadata (*SessionMetadata::Metadata());
 
-#if 0 // would be nicer with C++11 initialiser...
-			std::map<char, std::string> subs {
-				{ 'f', filename },
-				{ 'd', Glib::path_get_dirname(filename)  + G_DIR_SEPARATOR },
-				{ 'b', PBD::basename_nosuffix(filename) },
-				...
-			};
-#endif
 			export_status->active_job = ExportStatus::Command;
 			PBD::ScopedConnection command_connection;
-			std::map<char, std::string> subs;
 
 			std::stringstream track_number;
 			track_number << metadata.track_number ();
@@ -465,30 +499,32 @@ ExportHandler::finish_timespan ()
 			std::stringstream year;
 			year << metadata.year ();
 
-			subs.insert (std::pair<char, std::string> ('a', metadata.artist ()));
-			subs.insert (std::pair<char, std::string> ('b', PBD::basename_nosuffix (filename)));
-			subs.insert (std::pair<char, std::string> ('c', metadata.copyright ()));
-			subs.insert (std::pair<char, std::string> ('d', Glib::path_get_dirname (filename) + G_DIR_SEPARATOR));
-			subs.insert (std::pair<char, std::string> ('f', filename));
-			subs.insert (std::pair<char, std::string> ('l', metadata.lyricist ()));
-			subs.insert (std::pair<char, std::string> ('n', session.name ()));
-			subs.insert (std::pair<char, std::string> ('s', session.path ()));
-			subs.insert (std::pair<char, std::string> ('o', metadata.conductor ()));
-			subs.insert (std::pair<char, std::string> ('t', metadata.title ()));
-			subs.insert (std::pair<char, std::string> ('z', metadata.organization ()));
-			subs.insert (std::pair<char, std::string> ('A', metadata.album ()));
-			subs.insert (std::pair<char, std::string> ('C', metadata.comment ()));
-			subs.insert (std::pair<char, std::string> ('E', metadata.engineer ()));
-			subs.insert (std::pair<char, std::string> ('G', metadata.genre ()));
-			subs.insert (std::pair<char, std::string> ('L', total_tracks.str ()));
-			subs.insert (std::pair<char, std::string> ('M', metadata.mixer ()));
-			subs.insert (std::pair<char, std::string> ('N', current_timespan->name())); // =?= config_map.begin()->first->name ()
-			subs.insert (std::pair<char, std::string> ('O', metadata.composer ()));
-			subs.insert (std::pair<char, std::string> ('P', metadata.producer ()));
-			subs.insert (std::pair<char, std::string> ('S', metadata.disc_subtitle ()));
-			subs.insert (std::pair<char, std::string> ('T', track_number.str ()));
-			subs.insert (std::pair<char, std::string> ('Y', year.str ()));
-			subs.insert (std::pair<char, std::string> ('Z', metadata.country ()));
+			std::map<char, std::string> subs {
+				{'a', metadata.artist ()},
+				{'b', PBD::basename_nosuffix (filename)},
+				{'c', metadata.copyright ()},
+				{'d', Glib::path_get_dirname (filename) + G_DIR_SEPARATOR},
+				{'f', filename},
+				{'l', metadata.lyricist ()},
+				{'n', session.name ()},
+				{'s', session.path ()},
+				{'o', metadata.conductor ()},
+				{'t', metadata.title ()},
+				{'z', metadata.organization ()},
+				{'A', metadata.album ()},
+				{'C', metadata.comment ()},
+				{'E', metadata.engineer ()},
+				{'G', metadata.genre ()},
+				{'L', total_tracks.str ()},
+				{'M', metadata.mixer ()},
+				{'N', current_timespan->name()}, // =?= config_map.begin()->first->name ()
+				{'O', metadata.composer ()},
+				{'P', metadata.producer ()},
+				{'S', metadata.disc_subtitle ()},
+				{'T', track_number.str ()},
+				{'Y', year.str ()},
+				{'Z', metadata.country ()}
+			};
 
 			ARDOUR::SystemExec *se = new ARDOUR::SystemExec(fmt->command(), subs, true);
 			info << "Post-export command line : {" << se->to_s () << "}" << endmsg;
@@ -545,9 +581,7 @@ ExportHandler::finish_timespan ()
 	/* finish timespan is called in freewheeling rt-context,
 	 * we cannot start a new export from here */
 	assert (AudioEngine::instance()->freewheeling ());
-	pthread_t tid;
-	pthread_create (&tid, NULL, ExportHandler::start_timespan_bg, this);
-	pthread_detach (tid);
+	timespan_thread_wakeup ();
 }
 
 void
