@@ -64,6 +64,7 @@
 #include "ardour/session_playlists.h"
 #include "ardour/types_convert.h"
 #include "ardour/utils.h"
+#include "ardour/velocity_control.h"
 
 #include "pbd/i18n.h"
 
@@ -81,7 +82,9 @@ using namespace PBD;
 MidiTrack::MidiTrack (Session& sess, string name, TrackMode mode)
 	: Track (sess, name, PresentationInfo::MidiTrack, mode, DataType::MIDI)
 	, _immediate_events(6096) // FIXME: size?
+	, _user_immediate_events(2048) // FIXME: size?
 	, _immediate_event_buffer(6096)
+	, _user_immediate_event_buffer(2048)
 	, _step_edit_ring_buffer(64) // FIXME: size?
 	, _note_mode (Sustained)
 	, _step_editing (false)
@@ -108,6 +111,9 @@ MidiTrack::init ()
 		return -1;
 	}
 
+	_velocity_control.reset (new VelocityControl (_session));
+	add_control (_velocity_control);
+
 	_input->changed.connect_same_thread (*this, boost::bind (&MidiTrack::track_input_active, this, _1, _2));
 
 	_disk_writer->set_note_mode (_note_mode);
@@ -469,9 +475,8 @@ MidiTrack::non_realtime_locate (samplepos_t spos)
 void
 MidiTrack::push_midi_input_to_step_edit_ringbuffer (samplecnt_t nframes)
 {
-	PortSet& ports (_input->ports());
-
-	for (PortSet::iterator p = ports.begin(DataType::MIDI); p != ports.end(DataType::MIDI); ++p) {
+	std::shared_ptr<PortSet> ports (_input->ports());
+	for (PortSet::iterator p = ports->begin (DataType::MIDI); p != ports->end (DataType::MIDI); ++p) {
 
 		Buffer& b (p->get_buffer (nframes));
 		const MidiBuffer* const mb = dynamic_cast<MidiBuffer*>(&b);
@@ -498,7 +503,7 @@ MidiTrack::snapshot_out_of_band_data (samplecnt_t nframes)
 {
 	_immediate_event_buffer.clear ();
 	if (0 == _immediate_events.read_space()) {
-		return;
+		goto user;
 	}
 
 	assert (nframes > 0);
@@ -515,6 +520,28 @@ MidiTrack::snapshot_out_of_band_data (samplecnt_t nframes)
 	 */
 
 	_immediate_events.read (_immediate_event_buffer, 0, 1, nframes - 1, true);
+
+  user:
+	_user_immediate_event_buffer.clear ();
+	if (0 == _user_immediate_events.read_space()) {
+		return;
+	}
+
+	assert (nframes > 0);
+
+	DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 has %2 of user immediate events to deliver\n", name(), _user_immediate_events.read_space()));
+
+	/* write as many of the user immediate events as we can, but give "true" as
+	 * the last argument ("stop on overflow in destination") so that we'll
+	 * ship the rest out next time.
+	 *
+	 * the (nframes-1) argument puts all these events at the last
+	 * possible position of the output buffer, so that we do not
+	 * violate monotonicity when writing.
+	 */
+
+	_user_immediate_events.read (_user_immediate_event_buffer, 0, 1, nframes - 1, true);
+
 }
 
 void
@@ -522,6 +549,7 @@ MidiTrack::write_out_of_band_data (BufferSet& bufs, samplecnt_t nframes) const
 {
 	MidiBuffer& buf (bufs.get_midi (0));
 	buf.merge_from (_immediate_event_buffer, nframes);
+	buf.merge_from (_user_immediate_event_buffer, nframes);
 }
 
 int
@@ -575,22 +603,16 @@ MidiTrack::export_stuff (BufferSet&                   buffers,
 	return 0;
 }
 
-std::shared_ptr<Region>
-MidiTrack::bounce (InterThreadInfo& itt, std::string const& name)
+bool
+MidiTrack::bounceable (std::shared_ptr<Processor> endpoint, bool include_endpoint) const
 {
-	return bounce_range (_session.current_start_sample(), _session.current_end_sample(), itt, main_outs(), false, name);
-}
-
-std::shared_ptr<Region>
-MidiTrack::bounce_range (samplepos_t                  start,
-                         samplepos_t                  end,
-                         InterThreadInfo&             itt,
-                         std::shared_ptr<Processor> endpoint,
-                         bool                         include_endpoint,
-                         std::string const&           name)
-{
-	vector<std::shared_ptr<Source> > srcs;
-	return _session.write_one_track (*this, start, end, false, srcs, itt, endpoint, include_endpoint, false, false, name);
+	if (!endpoint && !include_endpoint) {
+		/* no processing - just read from the playlist and create new
+		 * files: always possible.
+		 */
+		return true;
+	}
+	return false; // a lie, Session::write_one_track can handle this.
 }
 
 void
@@ -646,6 +668,16 @@ MidiTrack::write_immediate_event(Evoral::EventType event_type, size_t size, cons
 	return (_immediate_events.write (0, event_type, size, buf) == size);
 }
 
+bool
+MidiTrack::write_user_immediate_event(Evoral::EventType event_type, size_t size, const uint8_t* buf)
+{
+	if (!Evoral::midi_event_is_valid(buf, size)) {
+		cerr << "WARNING: Ignoring illegal immediate MIDI event" << endl;
+		return false;
+	}
+	return (_user_immediate_events.write (0, event_type, size, buf) == size);
+}
+
 void
 MidiTrack::set_parameter_automation_state (Evoral::Parameter param, AutoState state)
 {
@@ -841,9 +873,8 @@ MidiTrack::map_input_active (bool yn)
 		return;
 	}
 
-	PortSet& ports (_input->ports());
-
-	for (PortSet::iterator p = ports.begin(DataType::MIDI); p != ports.end(DataType::MIDI); ++p) {
+	std::shared_ptr<PortSet> ports (_input->ports());
+	for (PortSet::iterator p = ports->begin (DataType::MIDI); p != ports->end (DataType::MIDI); ++p) {
 		std::shared_ptr<MidiPort> mp = std::dynamic_pointer_cast<MidiPort> (*p);
 		if (yn != mp->input_active()) {
 			mp->set_input_active (yn);
@@ -914,10 +945,8 @@ MidiTrack::monitoring_changed (bool self, Controllable::GroupControlDisposition
 	 * port level.
 	 */
 
-	PortSet& ports (_output->ports());
-
-	for (PortSet::iterator p = ports.begin(); p != ports.end(); ++p) {
-		std::shared_ptr<MidiPort> mp = std::dynamic_pointer_cast<MidiPort> (*p);
+	for (auto const& p : *_output->ports()) {
+		std::shared_ptr<MidiPort> mp = std::dynamic_pointer_cast<MidiPort> (p);
 		if (mp) {
 			mp->require_resolve ();
 		}
@@ -948,6 +977,7 @@ MidiTrack::realtime_handle_transport_stopped ()
 {
 	Route::realtime_handle_transport_stopped ();
 	_disk_reader->resolve_tracker (_immediate_events, 0);
+	_disk_reader->resolve_tracker (_user_immediate_events, 0);
 }
 
 void
