@@ -143,6 +143,7 @@ RegionListProperty::get_content_from_xml (XMLNode const& node) const
 
 Playlist::Playlist (Session& sess, string nom, DataType type, bool hide)
 	: SessionObject (sess, nom)
+	, Temporal::TimeDomainProvider ((type == DataType::AUDIO ? Temporal::AudioTime : Temporal::BeatTime), sess)
 	, regions (*this)
 	, _type (type)
 {
@@ -154,6 +155,7 @@ Playlist::Playlist (Session& sess, string nom, DataType type, bool hide)
 
 Playlist::Playlist (Session& sess, const XMLNode& node, DataType type, bool hide)
 	: SessionObject (sess, "unnamed playlist")
+	, Temporal::TimeDomainProvider ((type == DataType::AUDIO ? Temporal::AudioTime : Temporal::BeatTime), sess)
 	, regions (*this)
 	, _type (type)
 {
@@ -171,6 +173,7 @@ Playlist::Playlist (Session& sess, const XMLNode& node, DataType type, bool hide
 
 Playlist::Playlist (std::shared_ptr<const Playlist> other, string namestr, bool hide)
 	: SessionObject (other->_session, namestr)
+	, Temporal::TimeDomainProvider ((other->_type == DataType::AUDIO ? Temporal::AudioTime : Temporal::BeatTime), other->_session)
 	, regions (*this)
 	, _type (other->_type)
 	, _orig_track_id (other->_orig_track_id)
@@ -182,6 +185,9 @@ Playlist::Playlist (std::shared_ptr<const Playlist> other, string namestr, bool
 	ThawList   thawlist;
 	other->copy_regions (tmp);
 
+	RegionSortByLayer cmp;
+	tmp.sort (cmp);
+
 	in_set_state++;
 
 	for (list<std::shared_ptr<Region> >::iterator x = tmp.begin(); x != tmp.end(); ++x) {
@@ -204,6 +210,7 @@ Playlist::Playlist (std::shared_ptr<const Playlist> other, string namestr, bool
 
 Playlist::Playlist (std::shared_ptr<const Playlist> other, timepos_t const & start, timepos_t const & cnt, string str, bool hide)
 	: SessionObject(other->_session, str)
+	, Temporal::TimeDomainProvider ((other->_type == DataType::AUDIO ? Temporal::AudioTime : Temporal::BeatTime), other->_session)
 	, regions (*this)
 	, _type (other->_type)
 	, _orig_track_id (other->_orig_track_id)
@@ -269,10 +276,10 @@ Playlist::Playlist (std::shared_ptr<const Playlist> other, timepos_t const & sta
 			new_region = RegionFactory::create (region, offset, plist, true, &rlock.thawlist);
 
 			add_region_internal (new_region, position, rlock.thawlist);
+			set_layer (new_region, region->layer ());
 		}
 	}
 
-
 	/* keep track of any dead space at end (for pasting into Ripple or
 	 * RippleAll mode) at the end of construction, any length of cnt beyond
 	 * the extents of the regions is end_space
@@ -282,6 +289,8 @@ Playlist::Playlist (std::shared_ptr<const Playlist> other, timepos_t const & sta
 
 	in_set_state--;
 	first_set_state = false;
+
+	relayer ();
 }
 
 void
@@ -775,6 +784,7 @@ Playlist::add_region_internal (std::shared_ptr<Region> region, timepos_t const &
 	}
 
 	region->set_position_unchecked (position);
+	region->set_position_time_domain (time_domain());
 
 	regions.insert (upper_bound (regions.begin (), regions.end (), region, cmp), region);
 	all_regions.insert (region);
@@ -915,37 +925,54 @@ Playlist::remove_gaps (timecnt_t const & gap_threshold, timecnt_t const & leave_
 }
 
 void
-Playlist::get_equivalent_regions (std::shared_ptr<Region> other, vector<std::shared_ptr<Region> >& results)
+Playlist::get_equivalent_regions (std::shared_ptr<Region> basis, vector<std::shared_ptr<Region>>& results)
 {
-	switch (Config->get_region_equivalence ()) {
-		case Exact:
-			for (auto const & r : regions) {
-				if (r->exact_equivalent (other)) {
-					results.push_back (r);
+	if (basis->is_explicitly_ungrouped ()) {
+		/*user explicitly ungrouped this region, so we bail */
+		return;
+	}
+
+	if (basis->is_implicitly_ungrouped ()) {
+		/* no group defined ... we must guess ... fallback to 'region equivalence' behavior */
+
+		switch (Config->get_region_equivalence ()) {
+			case Exact:
+				for (auto const& r : regions) {
+					if (r->exact_equivalent (basis)) {
+						results.push_back (r);
+					}
 				}
-			}
-			break;
-		case LayerTime:
-			for (auto const & r : regions) {
-				if (r->layer_and_time_equivalent (other)) {
-					results.push_back (r);
+				break;
+			case LayerTime:
+				for (auto const& r : regions) {
+					if (r->layer_and_time_equivalent (basis)) {
+						results.push_back (r);
+					}
 				}
-			}
-			break;
-		case Enclosed:
-			for (auto const & r : regions) {
-				if (r->enclosed_equivalent (other)) {
-					results.push_back (r);
+				break;
+			case Enclosed:
+				for (auto const& r : regions) {
+					if (r->enclosed_equivalent (basis)) {
+						results.push_back (r);
+					}
 				}
-			}
-			break;
-		case Overlap:
-			for (auto const & r : regions) {
-				if (r->overlap_equivalent (other)) {
-					results.push_back (r);
+				break;
+			case Overlap:
+				for (auto const& r : regions) {
+					if (r->overlap_equivalent (basis)) {
+						results.push_back (r);
+					}
 				}
-			}
-			break;
+				break;
+		}
+		return;
+	}
+
+	/* region has an implicit or explicit group-id; return all regions with the same group-id */
+	for (auto const& r : regions) {
+		if (r->region_group () == basis->region_group ()) {
+			results.push_back (r);
+		}
 	}
 }
 
@@ -961,8 +988,15 @@ Playlist::partition (timepos_t const & start, timepos_t const & end, bool cut)
  *  removed.
  */
 void
-Playlist::partition_internal (timepos_t const & start, timepos_t const & end, bool cutting, ThawList& thawlist)
+Playlist::partition_internal (timepos_t start, timepos_t end, bool cutting, ThawList& thawlist)
 {
+	if (start.time_domain () != end.time_domain () || start.time_domain () != time_domain () || end.time_domain () != time_domain ()) {
+		warning << string_compose (_("Playlist::partition_internal() with time domains mismatch %1 %2 (expected %3 time)"), start, end, time_domain () == Temporal::AudioTime ? "audio" : " music") << endmsg;
+	}
+
+	start.set_time_domain (time_domain ());
+	end.set_time_domain (time_domain ());
+
 	RegionList new_regions;
 
 	{
@@ -1041,6 +1075,7 @@ Playlist::partition_internal (timepos_t const & start, timepos_t const & end, bo
 					plist.add (Properties::automatic, true);
 					plist.add (Properties::left_of_split, true);
 					plist.add (Properties::right_of_split, true);
+					plist.add (Properties::reg_group, Region::get_region_operation_group_id (current->region_group(), InnerSplit));
 
 					/* see note in ::_split_region()
 					 */
@@ -1060,6 +1095,7 @@ Playlist::partition_internal (timepos_t const & start, timepos_t const & end, bo
 				plist.add (Properties::name, new_name);
 				plist.add (Properties::automatic, true);
 				plist.add (Properties::right_of_split, true);
+				plist.add (Properties::reg_group, Region::get_region_operation_group_id (current->region_group(), RightOfSplit));
 
 				region = RegionFactory::create (current, pos1.distance (pos3), plist, true, &thawlist );
 
@@ -1137,6 +1173,7 @@ Playlist::partition_internal (timepos_t const & start, timepos_t const & end, bo
 					plist.add (Properties::name, new_name);
 					plist.add (Properties::automatic, true);
 					plist.add (Properties::right_of_split, true);
+					plist.add (Properties::reg_group, Region::get_region_operation_group_id (current->region_group(), RightOfSplit));
 
 					region = RegionFactory::create (current, plist, true, &thawlist);
 
@@ -1248,7 +1285,7 @@ Playlist::cut (timepos_t const & start, timecnt_t const & cnt)
 
 	{
 		RegionWriteLock rlock (this);
-		partition_internal (start, (start+cnt).decrement(), true, rlock.thawlist);
+		partition_internal (start, start+cnt, true, rlock.thawlist);
 	}
 
 	return the_copy;
@@ -1288,12 +1325,16 @@ Playlist::paste (std::shared_ptr<Playlist> other, timepos_t const & position, fl
 				for (auto const & r : other->regions) {
 					std::shared_ptr<Region> copy_of_region = RegionFactory::create (r, true, false, &rl1.thawlist);
 
+					/* we want newly-pasted regions to retain prior group-sharing relationship, but with a new group-id */
+					copy_of_region->set_region_group (Region::get_region_operation_group_id(r->region_group(), Paste));
+
 					/* put these new regions on top of all existing ones, but preserve
 					   the ordering they had in the original playlist.
 					*/
 
 					add_region_internal (copy_of_region, r->position() + pos, rl1.thawlist);
-					set_layer (copy_of_region, copy_of_region->layer() + top);
+					copy_of_region->set_layer (r->layer() + top);
+					set_layer (copy_of_region, copy_of_region->layer());
 				}
 				pos += shift;
 			}
@@ -1319,6 +1360,7 @@ Playlist::duplicate (std::shared_ptr<Region> region, timepos_t & position, timec
 
 	while (itimes--) {
 		std::shared_ptr<Region> copy = RegionFactory::create (region, true, false, &rl.thawlist);
+		copy->set_region_group(Region::get_region_operation_group_id(region->region_group(), Paste));
 		add_region_internal (copy, position, rl.thawlist);
 		set_layer (copy, DBL_MAX);
 		position += gap;
@@ -1334,6 +1376,7 @@ Playlist::duplicate (std::shared_ptr<Region> region, timepos_t & position, timec
 			plist.add (Properties::length, length);
 
 			std::shared_ptr<Region> sub = RegionFactory::create (region, plist, true, &rl.thawlist);
+			sub->set_region_group(Region::get_region_operation_group_id(region->region_group(), Paste));
 			add_region_internal (sub, position, rl.thawlist);
 			set_layer (sub, DBL_MAX);
 		}
@@ -1350,6 +1393,7 @@ Playlist::duplicate_until (std::shared_ptr<Region> region, timepos_t & position,
 	while ((position + region->length()).decrement() < end) {
 		std::shared_ptr<Region> copy = RegionFactory::create (region, true, false, &rl.thawlist);
 		add_region_internal (copy, position, rl.thawlist);
+		copy->set_region_group(Region::get_region_operation_group_id(region->region_group(), Paste));
 		set_layer (copy, DBL_MAX);
 		position += gap;
 	}
@@ -1363,6 +1407,7 @@ Playlist::duplicate_until (std::shared_ptr<Region> region, timepos_t & position,
 			plist.add (Properties::length, length);
 
 			std::shared_ptr<Region> sub = RegionFactory::create (region, plist, false, &rl.thawlist);
+			sub->set_region_group(Region::get_region_operation_group_id(region->region_group(), Paste));
 			add_region_internal (sub, position, rl.thawlist);
 			set_layer (sub, DBL_MAX);
 		}
@@ -1407,7 +1452,7 @@ Playlist::duplicate_ranges (std::list<TimelineRange>& ranges, float times)
 }
 
 void
-Playlist::shift (timepos_t const & at, timecnt_t const & distance, bool move_intersected, bool ignore_music_glue)
+Playlist::shift (timepos_t const & at, timecnt_t const & distance, bool move_intersected)
 {
 	PBD::Unwinder<bool> uw (_playlist_shift_active, true);
 	RegionWriteLock rlock (this);
@@ -1428,23 +1473,9 @@ Playlist::shift (timepos_t const & at, timecnt_t const & distance, bool move_int
 			}
 		}
 
-		/* do not move regions glued to music time - that
-		 * has to be done separately.
-		 */
-
-		if (!ignore_music_glue && r->position().time_domain() != Temporal::AudioTime) {
-			fixup.push_back (r);
-			continue;
-		}
-
 		rlock.thawlist.add (r);
 		r->set_position (r->position() + distance);
 	}
-
-	/* XXX: may not be necessary; Region::post_set should do this, I think */
-	for (auto & r : fixup) {
-		r->recompute_position_from_time_domain ();
-	}
 }
 
 void
@@ -1511,6 +1542,7 @@ Playlist::_split_region (std::shared_ptr<Region> region, timepos_t const &  play
 		plist.add (Properties::length, after);
 		plist.add (Properties::name, after_name);
 		plist.add (Properties::right_of_split, true);
+		plist.add (Properties::reg_group, Region::get_region_operation_group_id (region->region_group(), RightOfSplit));
 
 		/* same note as above */
 		right = RegionFactory::create (region, before, plist, true, &thawlist);
@@ -1602,24 +1634,28 @@ Playlist::region_bounds_changed (const PropertyChange& what_changed, std::shared
 	}
 
 	if (what_changed.contains (Properties::length)) {
-		/* remove it from the list then add it back in
-		 * the right place again.
-		 */
-
-		RegionSortByPosition cmp;
-
-		RegionList::iterator i = find (regions.begin (), regions.end (), region);
-
-		if (i == regions.end ()) {
-			/* the region bounds are being modified but its not currently
-			 * in the region list. we will use its bounds correctly when/if
-			 * it is added
+		{
+			/* remove it from the list then add it back in
+			 * the right place again.
 			 */
-			return;
+
+			RegionWriteLock rl (this, false);
+			RegionSortByPosition cmp;
+
+			RegionList::iterator i = find (regions.begin (), regions.end (), region);
+
+			if (i == regions.end ()) {
+				/* the region bounds are being modified but its not currently
+				 * in the region list. we will use its bounds correctly when/if
+				 * it is added
+				 */
+				return;
+			}
+
+			regions.erase (i);
+			regions.insert (upper_bound (regions.begin (), regions.end (), region, cmp), region);
 		}
 
-		regions.erase (i);
-		regions.insert (upper_bound (regions.begin (), regions.end (), region, cmp), region);
 
 		if (holding_state ()) {
 			pending_bounds.push_back (region);
@@ -1664,6 +1700,7 @@ Playlist::region_changed (const PropertyChange& what_changed, std::shared_ptr<Re
 	our_interests.add (Properties::opaque);
 	our_interests.add (Properties::contents);
 	our_interests.add (Properties::time_domain);
+	our_interests.add (Properties::region_fx);
 
 	bounds.add (Properties::start);
 	bounds.add (Properties::length);
@@ -1952,16 +1989,16 @@ std::shared_ptr<RegionList>
 Playlist::regions_touched (timepos_t const & start, timepos_t const & end)
 {
 	RegionReadLock rlock (this);
-	return regions_touched_locked (start, end);
+	return regions_touched_locked (start, end, false);
 }
 
 std::shared_ptr<RegionList>
-Playlist::regions_touched_locked (timepos_t const & start, timepos_t const & end)
+Playlist::regions_touched_locked (timepos_t const & start, timepos_t const & end, bool with_tail)
 {
 	std::shared_ptr<RegionList> rlist (new RegionList);
 
 	for (auto & r : regions) {
-		if (r->coverage (start, end) != Temporal::OverlapNone) {
+		if (r->coverage (start, end, with_tail) != Temporal::OverlapNone) {
 			rlist->push_back (r);
 		}
 	}
@@ -2631,6 +2668,7 @@ Playlist::relayer ()
 		}
 
 		assert (divisions == 0 || end_division < divisions);
+		assert (start_division >= 0 && end_division >= start_division);
 
 		/* find the lowest layer that this region can go on */
 		size_t j = layers.size ();
@@ -3373,18 +3411,19 @@ Playlist::uncombine (std::shared_ptr<Region> target)
 void
 Playlist::fade_range (list<TimelineRange>& ranges)
 {
+	ThawList thawlist;
 	RegionReadLock rlock (this);
-	for (list<TimelineRange>::iterator r = ranges.begin(); r != ranges.end(); ) {
-		list<TimelineRange>::iterator tmpr = r;
-		++tmpr;
-		for (RegionList::const_iterator i = regions.begin (); i != regions.end ();) {
-			RegionList::const_iterator tmpi = i;
-			++tmpi;
-			(*i)->fade_range ((*r).start().samples(), (*r).end().samples());
-			i = tmpi;
-		}
-		r = tmpr;
+	/* add regions only once, not for each range */
+	for (auto const& r : regions) {
+		thawlist.add (r);
 	}
+	for (auto const& t: ranges) {
+		for (auto const& r : regions) {
+			r->fade_range (t.start().samples(), t.end().samples());
+		}
+	}
+	rlock.release ();
+	thawlist.release ();
 }
 
 uint32_t
@@ -3500,24 +3539,48 @@ Playlist::rdiff_and_add_command (Session* session)
 	session->add_command (new StatefulDiffCommand (shared_from_this ()));
 }
 
-Temporal::TimeDomain
-Playlist::time_domain() const
-{
-	switch (_type) {
-	case DataType::AUDIO:
-		return Temporal::AudioTime;
-	default:
-		break;
-	}
-
-	return Temporal::BeatTime;
-}
-
 void
-Playlist::globally_change_time_domain (Temporal::TimeDomain from, Temporal::TimeDomain to)
+Playlist::start_domain_bounce (Temporal::DomainBounceInfo& cmd)
 {
 	RegionReadLock rlock (this);
 	for (auto & region  : regions) {
-		region->globally_change_time_domain (from, to);
+		region->start_domain_bounce (cmd);
 	}
 }
+
+
+void
+Playlist::finish_domain_bounce (Temporal::DomainBounceInfo& cmd)
+{
+	ThawList thawlist;
+
+	clear_changes ();
+
+	{
+		RegionWriteLock rlock (this);
+		for (auto & region  : regions) {
+			thawlist.add (region);
+			region->finish_domain_bounce (cmd);
+		}
+	}
+
+	thawlist.release ();
+	rdiff_and_add_command (&_session);
+}
+
+void
+Playlist::time_domain_changed ()
+{
+#if 0
+	using namespace Temporal;
+
+	TimeDomainProvider::time_domain_changed ();
+
+	Temporal::TimeDomain to = time_domain();
+	Temporal::TimeDomain from = (to == AudioTime ? BeatTime : AudioTime);
+
+	for (auto & region  : regions) {
+		region->swap_time_domain (from, to);
+	}
+#endif
+}
