@@ -23,6 +23,7 @@
 #include "pbd/error.h"
 
 #include "ardour/port_engine_shared.h"
+#include "ardour/port_manager.h"
 
 #include "pbd/i18n.h"
 
@@ -828,6 +829,16 @@ PortEngineSharedImpl::update_system_port_latencies ()
 	}
 }
 
+void
+PortEngineSharedImpl::process_connection_queue_locked (PortManager& mgr)
+{
+	for (auto& c : _port_connection_queue) {
+		mgr.connect_callback (c->a, c->b, c->c);
+		delete c;
+	}
+	_port_connection_queue.clear ();
+}
+
 #ifndef NDEBUG
 void
 PortEngineSharedImpl::list_ports () const
