@@ -274,7 +274,7 @@ PortManager::clear_pending_port_deletions ()
 }
 
 void
-PortManager::remove_all_ports ()
+PortManager::remove_session_ports ()
 {
 	/* make sure that JACK callbacks that will be invoked as we cleanup
 	 * ports know that they have nothing to do.
@@ -282,13 +282,20 @@ PortManager::remove_all_ports ()
 
 	PBD::Unwinder<bool> uw (_port_remove_in_progress, true);
 
-	/* process lock MUST be held by caller
-	*/
+	/* process lock MUST be held by caller */
 
 	{
-		RCUWriter<Ports>         writer (_ports);
+		RCUWriter<Ports>            writer (_ports);
 		std::shared_ptr<Ports> ps = writer.get_copy ();
-		ps->clear ();
+
+		for (auto i = ps->begin (); i != ps->end (); ) {
+			std::shared_ptr<Port> port = i->second;
+			if (i->second->flags () & TransportMasterPort) {
+				++i;
+			} else {
+				i = ps->erase (i);
+			}
+		}
 	}
 
 	/* clear dead wood list in RCU */
@@ -742,7 +749,7 @@ PortManager::physically_connected (const string& port_name)
 }
 
 int
-PortManager::get_connections (const string& port_name, std::vector<std::string>& s)
+PortManager::get_connections (const string& port_name, std::vector<std::string>& s, bool process_context_safe)
 {
 	if (!_backend) {
 		s.clear ();
@@ -756,7 +763,7 @@ PortManager::get_connections (const string& port_name, std::vector<std::string>&
 		return 0;
 	}
 
-	return _backend->get_connections (handle, s);
+	return _backend->get_connections (handle, s, process_context_safe);
 }
 
 int
@@ -857,7 +864,7 @@ PortManager::reestablish_ports ()
 
 	if (i != p->end ()) {
 		/* failed */
-		remove_all_ports ();
+		remove_session_ports ();
 		return -1;
 	}
 
@@ -879,7 +886,7 @@ PortManager::reestablish_ports ()
 		set_pretty_names (port_names, DataType::MIDI, false);
 	}
 
-	if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->current_backend_name () == X_("JACK")) {
+	if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->is_jack ()) {
 		port_engine ().register_port (X_("physical_audio_input_monitor_enable"), DataType::AUDIO, ARDOUR::PortFlags (IsInput | IsTerminal | Hidden));
 		port_engine ().register_port (X_("physical_midi_input_monitor_enable"), DataType::MIDI, ARDOUR::PortFlags (IsInput | IsTerminal | Hidden));
 	}
@@ -925,6 +932,10 @@ PortManager::reconnect_ports ()
 	if (s && s->monitor_out() && !s->monitor_out ()->output()->has_ext_connection()) {
 		s->auto_connect_monitor_bus ();
 	}
+	if (s && s->click_io() && !s->click_io ()->has_ext_connection()) {
+		s->auto_connect_io (s->click_io());
+	}
+
 
 	for (auto const& i : *p) {
 		if (i.second->reconnect ()) {
@@ -932,7 +943,7 @@ PortManager::reconnect_ports ()
 		}
 	}
 
-	if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->current_backend_name () == X_("JACK")) {
+	if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->is_jack ()) {
 		std::string const        audio_port = AudioEngine::instance ()->make_port_name_non_relative (X_("physical_audio_input_monitor_enable"));
 		std::string const        midi_port  = AudioEngine::instance ()->make_port_name_non_relative (X_("physical_midi_input_monitor_enable"));
 		std::vector<std::string> audio_ports;
@@ -1134,7 +1145,7 @@ PortManager::update_input_ports (bool clear)
 #endif
 			mpw->insert (make_pair (*p, MIDIInputPort (32)));
 
-			if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->current_backend_name () == X_("JACK")) {
+			if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->is_jack ()) {
 				physical_midi_connection_list.push_back (*p);
 			}
 		}
@@ -1147,6 +1158,9 @@ PortManager::update_input_ports (bool clear)
 		 * do this when called from ::reestablish_ports()
 		 * "JACK: Cannot connect ports owned by inactive clients"
 		 */
+		/* .. but take the opportunity to clear out dead wood */
+		_audio_input_ports.flush ();
+		_midi_input_ports.flush ();
 		return;
 	}
 
@@ -1523,7 +1537,7 @@ PortManager::port_is_virtual_piano (std::string const& name)
 bool
 PortManager::port_is_physical_input_monitor_enable (std::string const& name)
 {
-	if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->current_backend_name () == X_("JACK")) {
+	if (Config->get_work_around_jack_no_copy_optimization () && AudioEngine::instance ()->is_jack ()) {
 		if (ends_with (name, X_(":physical_midi_input_monitor_enable"))) {
 			return true;
 		}
@@ -1997,7 +2011,7 @@ PortManager::run_input_meters (pframes_t n_samples, samplecnt_t rate)
 		assert (!port_is_mine (p.first));
 
 		PortEngine::PortHandle ph = _backend->get_port_by_name (p.first);
-		if (!ph || !_backend->connected (ph)) {
+		if (!ph) {
 			continue;
 		}
 
