@@ -64,7 +64,7 @@ using namespace PBD;
 
 Track::Track (Session& sess, string name, PresentationInfo::Flag flag, TrackMode mode, DataType default_type)
 	: Route (sess, name, flag, default_type)
-	, _saved_meter_point (_meter_point)
+	, _record_prepared (false)
 	, _mode (mode)
 	, _alignment_choice (Automatic)
 	, _pending_name_change (false)
@@ -76,6 +76,12 @@ Track::~Track ()
 {
 	DEBUG_TRACE (DEBUG::Destruction, string_compose ("track %1 destructor\n", _name));
 
+	for (auto const& p : _playlists) {
+		if (p) {
+			p->clear_time_domain_parent ();
+		}
+	}
+
 	if (_disk_reader) {
 		_disk_reader.reset ();
 	}
@@ -99,7 +105,7 @@ Track::init ()
 
 	DiskIOProcessor::Flag dflags = DiskIOProcessor::Recordable;
 
-	_disk_reader.reset (new DiskReader (_session, *this, name(), Config->get_default_automation_time_domain(), dflags));
+	_disk_reader.reset (new DiskReader (_session, *this, name(), Temporal::TimeDomainProvider (Config->get_default_automation_time_domain()), dflags));
 	_disk_reader->set_block_size (_session.get_block_size ());
 	_disk_reader->set_owner (this);
 
@@ -116,13 +122,13 @@ Track::init ()
 	std::shared_ptr<Route> rp (std::dynamic_pointer_cast<Route> (shared_from_this()));
 	std::shared_ptr<Track> rt = std::dynamic_pointer_cast<Track> (rp);
 
-	_record_enable_control.reset (new RecordEnableControl (_session, EventTypeMap::instance().to_symbol (RecEnableAutomation), *this, time_domain()));
+	_record_enable_control.reset (new RecordEnableControl (_session, EventTypeMap::instance().to_symbol (RecEnableAutomation), *this, *this));
 	add_control (_record_enable_control);
 
-	_record_safe_control.reset (new RecordSafeControl (_session, EventTypeMap::instance().to_symbol (RecSafeAutomation), *this, time_domain()));
+	_record_safe_control.reset (new RecordSafeControl (_session, EventTypeMap::instance().to_symbol (RecSafeAutomation), *this, *this));
 	add_control (_record_safe_control);
 
-	_monitoring_control.reset (new MonitorControl (_session, EventTypeMap::instance().to_symbol (MonitoringAutomation), *this, time_domain()));
+	_monitoring_control.reset (new MonitorControl (_session, EventTypeMap::instance().to_symbol (MonitoringAutomation), *this, *this));
 	add_control (_monitoring_control);
 
 	if (!name().empty()) {
@@ -136,6 +142,8 @@ Track::init ()
 	}
 
 	_session.config.ParameterChanged.connect_same_thread (*this, boost::bind (&Track::parameter_changed, this, _1));
+	_session.RecordStateChanged.connect_same_thread (*this, boost::bind (&Track::update_input_meter, this));
+	_session.TransportStateChange.connect_same_thread (*this, boost::bind (&Track::update_input_meter, this));
 
 	_monitoring_control->Changed.connect_same_thread (*this, boost::bind (&Track::monitoring_changed, this, _1, _2));
 	_record_safe_control->Changed.connect_same_thread (*this, boost::bind (&Track::record_safe_changed, this, _1, _2));
@@ -196,7 +204,9 @@ Track::state (bool save_template) const
 	root.add_child_nocopy (_record_safe_control->get_state ());
 	root.add_child_nocopy (_record_enable_control->get_state ());
 
-	root.set_property (X_("saved-meter-point"), _saved_meter_point);
+	if (_saved_meter_point) {
+		root.set_property (X_("saved-meter-point"), _saved_meter_point.value ());
+	}
 	root.set_property (X_("alignment-choice"), _alignment_choice);
 
 	return root;
@@ -262,15 +272,20 @@ Track::set_state (const XMLNode& node, int version)
 				_record_safe_control->set_state (*child, version);
 			} else if (name == _monitoring_control->name()) {
 				_monitoring_control->set_state (*child, version);
+			} else if (name == "recenable" && version <= 3002) {
+				float value;
+				if (child->get_property ("value", value)) {
+					_record_enable_control->set_value (value, Controllable::NoGroup);
+				}
 			}
 		}
 	}
 
-	if (!node.get_property (X_("saved-meter-point"), _saved_meter_point)) {
-		_saved_meter_point = _meter_point;
+	MeterPoint mp;
+	if (node.get_property (X_("saved-meter-point"), mp)) {
+		_saved_meter_point = mp;
 	}
 
-
 	AlignChoice ac;
 
 	if (node.get_property (X_("alignment-choice"), ac)) {
@@ -303,11 +318,12 @@ bool
 Track::can_record()
 {
 	bool will_record = true;
-	for (PortSet::iterator i = _input->ports().begin(); i != _input->ports().end() && will_record; ++i) {
-		if (!i->connected())
+	for (auto const& p : *_input->ports()) {
+		if (!p->connected()) {
 			will_record = false;
+			break;
+		}
 	}
-
 	return will_record;
 }
 
@@ -322,11 +338,6 @@ Track::prep_record_enabled (bool yn)
 		return -1;
 	}
 
-	/* keep track of the meter point as it was before we rec-enabled */
-	if (!_disk_writer->record_enabled()) {
-		_saved_meter_point = _meter_point;
-	}
-
 	bool will_follow;
 
 	if (yn) {
@@ -335,17 +346,65 @@ Track::prep_record_enabled (bool yn)
 		will_follow = _disk_writer->prep_record_disable ();
 	}
 
-	if (will_follow) {
-		if (yn) {
-			if (_meter_point != MeterCustom) {
-				set_meter_point (MeterInput);
-			}
-		} else {
-			set_meter_point (_saved_meter_point);
+	if (!will_follow) {
+		return -1;
+	}
+
+	_record_prepared = yn;
+	update_input_meter ();
+
+	return 0;
+}
+
+void
+Track::update_input_meter ()
+{
+	if (_session.loading ()) {
+		return;
+	}
+	/* meter input if _record_prepared,
+	 * except if Rolling, but not recording (master-rec-enable is off) and auto-input is enabled
+	 */
+	bool monitor_input = false;
+
+	if (_record_prepared) {
+		/* actually rolling (no count-in, pre-roll) */
+		bool const rolling     = 0 != _session.transport_speed();
+		bool const recording   = _session.actively_recording ();
+		bool const auto_input  = _session.config.get_auto_input ();
+
+		if (!(rolling && !recording && auto_input)) {
+			monitor_input = true;
 		}
 	}
 
-	return 0;
+	if (monitor_input) {
+		if (_saved_meter_point) {
+			/* already monitoring input */
+			return;
+		}
+		MeterPoint mp = meter_point ();
+		if (mp == MeterInput) {
+			/* user explicitly monitors input, do nothing */
+			return;
+		}
+
+		/* keep track of the meter point as it was before we rec-enabled */
+		_saved_meter_point = mp;
+
+		if (mp != MeterCustom) {
+			set_meter_point (MeterInput);
+		}
+
+	} else {
+		if (!_saved_meter_point) {
+			return;
+		}
+		if (_saved_meter_point != MeterCustom) {
+			set_meter_point (_saved_meter_point.value ());
+		}
+		_saved_meter_point.reset ();
+	}
 }
 
 void
@@ -385,6 +444,8 @@ Track::parameter_changed (string const & p)
 		if (_session.config.get_track_name_take()) {
 			resync_take_name ();
 		}
+	} else if (p == "auto-input") {
+		update_input_meter ();
 	}
 }
 
@@ -429,11 +490,22 @@ Track::resync_take_name (std::string n)
 bool
 Track::set_name (const string& str)
 {
+	if (_record_enable_control->get_value()) {
+		/* cannot rename rec-armed track - see also Track::resync_take_name */
+		return false;
+	}
+
 	if (str.empty ()) {
 		return false;
 	}
 
-	switch (resync_take_name (str)) {
+	string newname = Route::ensure_track_or_route_name (str);
+
+	if (newname == name()) {
+		return true;
+	}
+
+	switch (resync_take_name (newname)) {
 		case -1:
 			return false;
 		case 1:
@@ -444,8 +516,8 @@ Track::set_name (const string& str)
 
 	std::shared_ptr<Track> me = std::dynamic_pointer_cast<Track> (shared_from_this ());
 
-	_disk_reader->set_name (str);
-	_disk_writer->set_name (str);
+	_disk_reader->set_name (newname);
+	_disk_writer->set_name (newname);
 
 
 	/* When creating a track during session-load, do not change playlist's name.
@@ -456,7 +528,7 @@ Track::set_name (const string& str)
 	 * (new track name -> new playlist name  != old playlist)
 	 */
 	if (_session.loading ()) {
-		return Route::set_name (str);
+		return Route::set_name (newname);
 	}
 
 	for (uint32_t n = 0; n < DataType::num_types; ++n) {
@@ -475,11 +547,11 @@ Track::set_name (const string& str)
 			 * If (b) is not followed, we rename the current playlist and not
 			 * the other ones, which is a bit confusing (see mantis #4977).
 			 */
-			_playlists[n]->set_name (str);
+			_playlists[n]->set_name (newname);
 		}
 	}
 
-	return Route::set_name (str);
+	return Route::set_name (newname);
 }
 
 std::shared_ptr<Playlist>
@@ -491,16 +563,16 @@ Track::playlist ()
 void
 Track::request_input_monitoring (bool m)
 {
-	for (PortSet::iterator i = _input->ports().begin(); i != _input->ports().end(); ++i) {
-		AudioEngine::instance()->request_input_monitoring ((*i)->name(), m);
+	for (auto const& p : *_input->ports()) {
+		AudioEngine::instance()->request_input_monitoring (p->name(), m);
 	}
 }
 
 void
 Track::ensure_input_monitoring (bool m)
 {
-	for (PortSet::iterator i = _input->ports().begin(); i != _input->ports().end(); ++i) {
-		AudioEngine::instance()->ensure_input_monitoring ((*i)->name(), m);
+	for (auto const& p : *_input->ports()) {
+		AudioEngine::instance()->ensure_input_monitoring (p->name(), m);
 	}
 }
 
@@ -712,12 +784,32 @@ Track::use_playlist (DataType dt, std::shared_ptr<Playlist> p, bool set_orig)
 		if (rl->size () > 0) {
 			Region::RegionsPropertyChanged (rl, Properties::hidden);
 		}
+		/* we don't know for certain that we controlled the old
+		 * playlist's time domain, but it's a pretty good guess. If it
+		 * has an actual parent, revert to using its parent's domain
+		 */
+		if (old->time_domain_parent()) {
+			old->clear_time_domain_parent ();
+		}
 	}
+
 	if (p) {
 		std::shared_ptr<RegionList> rl (new RegionList (p->region_list_property ().rlist ()));
 		if (rl->size () > 0) {
 			Region::RegionsPropertyChanged (rl, Properties::hidden);
 		}
+
+		/* If the playlist has no time domain parent or its parent is
+		 * the session, reset to the explicit time domain of this
+		 * track.
+		 */
+
+		if (!p->time_domain_parent() || p->time_domain_parent() == &_session) {
+			/* XXX DANGER : track could go away leaving playlist
+			 * with dead parent time domain provider
+			 */
+			p->set_time_domain_parent (*this);
+		}
 	}
 
 	_session.set_dirty ();
@@ -885,8 +977,6 @@ Track::set_processor_state (XMLNode const& node, int version, XMLProperty const*
 		return true;
 	}
 
-	cerr << name() << " looking for state for track procs, DR = " << _disk_reader << endl;
-
 	if (prop->value() == "diskreader") {
 		if (_disk_reader) {
 			_disk_reader->set_state (node, version);
@@ -905,6 +995,32 @@ Track::set_processor_state (XMLNode const& node, int version, XMLProperty const*
 	return false;
 }
 
+std::shared_ptr<Region>
+Track::bounce (InterThreadInfo& itt, std::string const& name)
+{
+	return bounce_range (_session.current_start_sample(), _session.current_end_sample(), itt, main_outs(), false, name);
+}
+
+std::shared_ptr<Region>
+Track::bounce_range (samplepos_t start,
+                          samplepos_t end,
+                          InterThreadInfo& itt,
+                          std::shared_ptr<Processor> endpoint,
+                          bool include_endpoint,
+                          std::string const& nm, bool prefix_track_name)
+{
+	std::string source_name;
+
+	if (prefix_track_name && nm.length() > 0) {
+		source_name = string_compose ("%1 - %2", name(), nm);
+	} else {
+		source_name = nm;
+	}
+
+	vector<std::shared_ptr<Source> > srcs;
+	return _session.write_one_track (*this, start, end, false, srcs, itt, endpoint, include_endpoint, false, false, source_name, nm);
+}
+
 void
 Track::use_captured_sources (SourceList& srcs, CaptureInfos const & capture_info)
 {
@@ -1001,7 +1117,8 @@ Track::use_captured_midi_sources (SourceList& srcs, CaptureInfos const & capture
 	const samplepos_t preroll_off = _session.preroll_record_trim_len ();
 	const timepos_t cstart (timepos_t (capture_info.front()->start).beats());
 
-	for (ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
+	int cnt = 0;
+	for (ci = capture_info.begin(); ci != capture_info.end(); ++ci, ++cnt) {
 
 		string region_name;
 
@@ -1024,7 +1141,20 @@ Track::use_captured_midi_sources (SourceList& srcs, CaptureInfos const & capture
 			if (time_domain() == Temporal::BeatTime) {
 
 				const timepos_t ss (start_off);
-				const timecnt_t ll ((*ci)->samples, ss);
+				/* 2nd argument is the timeline position of the
+				 * start of the region in samples. We have to
+				 * get this right so that the conversion of
+				 * the capture duration (samples) to beats is
+				 * using the actual position where the region
+				 * will end up, rather than using its
+				 * source-relative start offset as a timeline position.
+				 *
+				 * This matters if the region ought to cover
+				 * part of the timeline where the tempo is
+				 * different from the value at the natural
+				 * position of the source.
+				 */
+				const timecnt_t ll ((*ci)->samples, timepos_t (initial_capture + start_off));
 
 				s = timepos_t (ss.beats());
 				l = timecnt_t (ll.beats(), s);
@@ -1039,6 +1169,7 @@ Track::use_captured_midi_sources (SourceList& srcs, CaptureInfos const & capture
 			plist.add (Properties::length, l);
 			plist.add (Properties::opaque, rmode != RecSoundOnSound);
 			plist.add (Properties::name, region_name);
+			plist.add (Properties::reg_group, Region::get_retained_group_id (cnt));
 
 			std::shared_ptr<Region> rx (RegionFactory::create (srcs, plist));
 			midi_region = std::dynamic_pointer_cast<MidiRegion> (rx);
@@ -1126,7 +1257,8 @@ Track::use_captured_audio_sources (SourceList& srcs, CaptureInfos const & captur
 	samplecnt_t buffer_position = afs->last_capture_start_sample ();
 	CaptureInfos::const_iterator ci;
 
-	for (ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
+	int cnt = 0;
+	for (ci = capture_info.begin(); ci != capture_info.end(); ++ci, ++cnt) {
 
 		string region_name;
 
@@ -1143,6 +1275,7 @@ Track::use_captured_audio_sources (SourceList& srcs, CaptureInfos const & captur
 			plist.add (Properties::length, timecnt_t ((*ci)->samples, timepos_t::zero (false)));
 			plist.add (Properties::name, region_name);
 			plist.add (Properties::opaque, rmode != RecSoundOnSound);
+			plist.add (Properties::reg_group, Region::get_retained_group_id (cnt));
 
 			std::shared_ptr<Region> rx (RegionFactory::create (srcs, plist));
 			region = std::dynamic_pointer_cast<AudioRegion> (rx);
@@ -1166,3 +1299,22 @@ Track::use_captured_audio_sources (SourceList& srcs, CaptureInfos const & captur
 	pl->set_capture_insertion_in_progress (false);
 	_session.add_command (new StatefulDiffCommand (pl));
 }
+
+void
+Track::time_domain_changed ()
+{
+	Route::time_domain_changed ();
+
+	std::shared_ptr<Playlist> pl = _playlists[DataType::AUDIO];
+	if (pl) {
+		if (pl->time_domain_parent() == this) {
+			pl->time_domain_changed ();
+		}
+	}
+	pl = _playlists[DataType::MIDI];
+	if (pl) {
+		if (pl->time_domain_parent() == this) {
+			pl->time_domain_changed ();
+		}
+	}
+}
