@@ -57,6 +57,7 @@
 #include "ardour/session.h"
 #include "ardour/session_configuration.h"
 #include "ardour/track.h"
+#include "ardour/well_known_enum.h"
 
 #include "cc121.h"
 
@@ -257,8 +258,6 @@ CC121::stop ()
 void
 CC121::thread_init ()
 {
-	pthread_set_name (event_loop_name().c_str());
-
 	PBD::notify_event_loops_about_thread_creation (pthread_self(), event_loop_name(), 2048);
 	ARDOUR::SessionEvent::create_per_thread_pool (event_loop_name(), 128);
 
@@ -397,35 +396,35 @@ CC121::encoder_handler (MIDI::Parser &, MIDI::EventTwoBytes* tb)
 	  break;
 	case 0x20:
 	  /* EQ 1 Q */
-	  if (r) { set_controllable (r->eq_q_controllable(0), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandQ, 0), adj); }
 	  break;
 	case 0x21:
 	  /* EQ 2 Q */
-	  if (r) { set_controllable (r->eq_q_controllable(1), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandQ, 1), adj); }
 	  break;
 	case 0x22:
 	  /* EQ 3 Q */
-	  if (r) { set_controllable (r->eq_q_controllable(2), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandQ, 2), adj); }
 	  break;
 	case 0x23:
 	  /* EQ 4 Q */
-	  if (r) { set_controllable (r->eq_q_controllable(3), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandQ, 3), adj); }
 	  break;
 	case 0x30:
 	  /* EQ 1 Frequency */
-	  if (r) { set_controllable (r->eq_freq_controllable(0), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandFreq, 0), adj); }
 	  break;
 	case 0x31:
 	  /* EQ 2 Frequency */
-	  if (r) { set_controllable (r->eq_freq_controllable(1), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandFreq, 1), adj); }
 	  break;
 	case 0x32:
 	  /* EQ 3 Frequency */
-	  if (r) { set_controllable (r->eq_freq_controllable(2), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandFreq, 2), adj); }
 	  break;
 	case 0x33:
 	  /* EQ 4 Frequency */
-	  if (r) { set_controllable (r->eq_freq_controllable(3), adj); }
+	  if (r) { set_controllable (r->mapped_control (EQ_BandFreq, 3), adj); }
 	  break;
 	case 0x3C:
 	  /* AI */
@@ -448,19 +447,19 @@ CC121::encoder_handler (MIDI::Parser &, MIDI::EventTwoBytes* tb)
 	  break;
 	case 0x40:
 	  /* EQ 1 Gain */
-	  if (r) { set_controllable (r->eq_gain_controllable(0), adj); }
+	  if (r) { set_controllable (r->mapped_control(EQ_BandGain, 0), adj); }
 	  break;
 	case 0x41:
 	  /* EQ 2 Gain */
-	  if (r) { set_controllable (r->eq_gain_controllable(1), adj); }
+	  if (r) { set_controllable (r->mapped_control(EQ_BandGain, 1), adj); }
 	  break;
 	case 0x42:
 	  /* EQ 3 Gain */
-	  if (r) { set_controllable (r->eq_gain_controllable(2), adj); }
+	  if (r) { set_controllable (r->mapped_control(EQ_BandGain, 2), adj); }
 	  break;
 	case 0x43:
 	  /* EQ 4 Gain */
-	  if (r) { set_controllable (r->eq_gain_controllable(3), adj); }
+	  if (r) { set_controllable (r->mapped_control(EQ_BandGain, 3), adj); }
 	  break;
 	case 0x50:
 	  /* Value */
