@@ -276,7 +276,7 @@ FaderPort8::button_automation (ARDOUR::AutoState as)
 	StripableList all;
 	session->get_stripables (all);
 	for (StripableList::const_iterator i = all.begin(); i != all.end(); ++i) {
-		if ((*i)->is_master() || (*i)->is_monitor()) {
+		if ((*i)->is_singleton ()) {
 			continue;
 		}
 		if (!(*i)->is_selected()) {
@@ -328,7 +328,7 @@ FaderPort8::button_solo_clear ()
 		StripableList all;
 		session->get_stripables (all);
 		for (StripableList::const_iterator i = all.begin(); i != all.end(); ++i) {
-			if ((*i)->is_master() || (*i)->is_auditioner() || (*i)->is_monitor()) {
+			if ((*i)->is_singleton () || (*i)->is_auditioner()) {
 				continue;
 			}
 			std::shared_ptr<SoloControl> sc = (*i)->solo_control();
@@ -339,7 +339,7 @@ FaderPort8::button_solo_clear ()
 		cancel_all_solo (); // AccessAction ("Main", "cancel-solo");
 	} else {
 		/* restore solo */
-		std::shared_ptr<ControlList> cl (new ControlList);
+		std::shared_ptr<AutomationControlList> cl (new AutomationControlList);
 		for (std::vector <std::weak_ptr<AutomationControl> >::const_iterator i = _solo_state.begin(); i != _solo_state.end(); ++i) {
 			std::shared_ptr<AutomationControl> ac = (*i).lock();
 			if (!ac) {
@@ -363,7 +363,7 @@ FaderPort8::button_mute_clear ()
 		_mute_state = session->cancel_all_mute ();
 	} else {
 		/* restore mute */
-		std::shared_ptr<ControlList> cl (new ControlList);
+		std::shared_ptr<AutomationControlList> cl (new AutomationControlList);
 		for (std::vector <std::weak_ptr<AutomationControl> >::const_iterator i = _mute_state.begin(); i != _mute_state.end(); ++i) {
 			std::shared_ptr<AutomationControl> ac = (*i).lock();
 			if (!ac) {
