@@ -604,19 +604,27 @@
 
 - (void) setNeedsDisplay:(BOOL)yn
 {
-  GdkWindowObject *private;
-  GdkWindowImplQuartz *impl;
+  if (!gdk_window || GDK_WINDOW_DESTROYED (gdk_window)) {
+    [super setNeedsDisplay:yn];
+    return;
+  }
+
+  GdkWindowObject* private = GDK_WINDOW_OBJECT (gdk_window);
+  GdkWindowImplQuartz* impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
+
+  if (!impl) {
+    [super setNeedsDisplay:yn];
+    return;
+  }
+
   NSRect nsrect = [self bounds];
   GdkRectangle r = { nsrect.origin.x, nsrect.origin.y, nsrect.size.width, nsrect.size.height };
-  
-  private = GDK_WINDOW_OBJECT (gdk_window);
-  impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
 
   GDK_NOTE (EVENTS, g_print ("setNeedsDisplay, current NDR %p\n", impl->needs_display_region));
 
   if (!impl->needs_display_region)
     impl->needs_display_region = gdk_region_rectangle (&r);
-  else 
+  else
     gdk_region_union_with_rect (impl->needs_display_region, &r);
 
 #if 0
@@ -635,8 +643,19 @@
 
 - (void) setNeedsDisplayInRect:(NSRect)rect
 {
-  GdkWindowObject *private;
-  GdkWindowImplQuartz *impl;
+  if (!gdk_window || GDK_WINDOW_DESTROYED (gdk_window)) {
+	  [super setNeedsDisplayInRect:rect];
+	  return;
+  }
+
+  GdkWindowObject* private = GDK_WINDOW_OBJECT (gdk_window);
+  GdkWindowImplQuartz* impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
+
+  if (!impl) {
+	  [super setNeedsDisplayInRect:rect];
+	  return;
+  }
+
   GdkRectangle r = { rect.origin.x, rect.origin.y, rect.size.width, rect.size.height };
 
   if (r.width >= 2147483647 || r.height >= 2147483647) {
@@ -646,15 +665,12 @@
     r.width = bounds.size.width;
     r.height = bounds.size.height;
   }
-  
-  private = GDK_WINDOW_OBJECT (gdk_window);
-  impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
 
   GDK_NOTE (EVENTS, g_print ("setNeedsDisplayInRect, current NDR %p\n", impl->needs_display_region));
 
   if (!impl->needs_display_region)
     impl->needs_display_region = gdk_region_rectangle (&r);
-  else 
+  else
     gdk_region_union_with_rect (impl->needs_display_region, &r);
 
 #if 0
@@ -668,7 +684,6 @@
   }
 #endif
 
-  
   [super setNeedsDisplayInRect:rect];
 }
 
@@ -744,7 +759,7 @@
     printf ("\t%d,%d %d x %d\n", rects[n].x, rects[n].y, rects[n].width, rects[n].height);
   }
 #endif
-  
+
   impl->in_paint_rect_count++;
   /* this essentially generates an expose event */
   _gdk_window_process_updates_recurse (gdk_window, region);
@@ -754,7 +769,7 @@
     {
       impl->needs_display_region = NULL;
     }
-  
+
   gdk_region_destroy (region);
 
   if (needsInvalidateShadow)
@@ -790,7 +805,7 @@
 
   if (impl->needs_display_region)
     gdk_region_destroy (impl->needs_display_region);
-  
+
   impl->needs_display_region = gdk_region_rectangle (&r);
 
 #if 0
